package net.wanji.opt.cache;

import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.databus.vo.CrossInfoVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/03/14 10:42
 */
@Component
@Slf4j
public class BaseCrossInfoCache implements CommandLineRunner {

    @Resource
    private BaseCrossInfoMapper baseCrossInfoMapper;

    private static final List<BaseCrossInfoPO> crossInfoList = new ArrayList<>();

    /**
     * 获取所有路口信息
     *
     * @return
     */
    public List<BaseCrossInfoPO> getCrossInfoCache() {
        return crossInfoList;
    }

    /**
     * 通过路口编号查询经纬度
     */
    public String getCoordinateByCrossId(String crossId) {
        for (BaseCrossInfoPO baseCrossInfoPO : crossInfoList) {
            if (StringUtils.equals(crossId, baseCrossInfoPO.getId())) {
                return baseCrossInfoPO.getLocation();
            }
        }
        return null;
    }

    /**
     * 获取所有信控路口的路口编号
     *
     * @return
     */
    public List<String> getIsSignalCrossIdList() {
        return crossInfoList.stream().filter(crossInfoPO -> crossInfoPO.getIsSignal() == 1).map(BaseCrossInfoPO::getId).collect(Collectors.toList());
    }

    public String getCrossName(String crossId) {
        for (BaseCrossInfoPO baseCrossInfoPO : crossInfoList) {
            if (StringUtils.equals(crossId, baseCrossInfoPO.getId())) {
                return baseCrossInfoPO.getName();
            }
        }
        return "";
    }

    @Override
    public void run(String... args) throws Exception {
        init();
    }

    public void init() {
        List<BaseCrossInfoPO> baseCrossInfoPOS = baseCrossInfoMapper.selectAll(new CrossInfoVO());
        if (!CollectionUtils.isEmpty(baseCrossInfoPOS)) {
            crossInfoList.addAll(baseCrossInfoPOS);
        }
    }
}
