package net.wanji.opt.common;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import lombok.extern.slf4j.Slf4j;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.net.URLEncoder;
import java.util.List;

/**
* @author duanruiming
* @date 2024/06/13 9:35
*/
@Slf4j
public class ExcelExportUtils {

    public static <T> void exportExcel(HttpServletResponse response, String startStr, String endStr,
                                       List<T> dataList, String sheetName, Class<T> clazz) throws Exception {
        // 文件名 sheet名称添加日期
        String fileName = sheetName.concat(startStr).concat("~").concat(endStr)
                .replaceAll(":", " ").replaceAll(" ", "-").concat(".xlsx"); // 注意文件扩展名应为.xlsx，因为你设置了xlsx的MIME类型

        // 设置响应头
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setHeader("Content-Disposition", "attachment; filename=" + URLEncoder.encode(fileName, "UTF-8").replaceAll("\\+", "%20"));

        // 使用EasyExcel进行导出
        try (ServletOutputStream outputStream = response.getOutputStream()) {
            // 解决服务器导出文件没数据问题 核心这个配置开始内存处理模式 inMemory少量数据
            //ExcelWriter excelWriter = EasyExcel.write(outputStream, clazz).inMemory(Boolean.TRUE).build();
            ExcelWriter excelWriter = EasyExcel.write(outputStream, clazz).build();
            WriteSheet writeSheet = EasyExcel.writerSheet(sheetName).build();

            // 填充数据
            excelWriter.write(dataList, writeSheet);

            // 刷新并关闭流，不需要重复关闭outputStream
            excelWriter.finish();
        } catch (Exception e) {
            log.error("{}导出excel异常：", fileName, e);
            throw new Exception("导出excel异常：" + e.getMessage(), e);
        }
    }
}
