package net.wanji.opt.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.bo.CrossIdAndStartEndDateBO;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.bo.HeatMapBO;
import net.wanji.databus.vo.RunningEvaluateCrossListVO;
import net.wanji.opt.bo.CrossNameBO;
import net.wanji.opt.bo.MetricsDetailBO;
import net.wanji.opt.service.impl.RunningEvaluateServiceImpl;
import net.wanji.opt.vo.*;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.core.MediaType;
import java.util.List;

/**
 * 运行评价
 *
 * @author Kent HAN
 */
@Api(value = "RunningEvaluateController", description = "运行评价")
@RequestMapping("/runningEvaluate")
@RestController
public class RunningEvaluateController {

    private final RunningEvaluateServiceImpl runningEvaluateService;

    public RunningEvaluateController(RunningEvaluateServiceImpl runningEvaluateService) {
        this.runningEvaluateService = runningEvaluateService;
    }

    @ApiOperation(value = "路口列表", notes = "路口列表", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/crossList",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = RunningEvaluateCrossListVO.class),
    })
    public JsonViewObject crossList(@RequestBody CrossNameBO crossNameBO) {
        List<RunningEvaluateCrossListVO> res = runningEvaluateService.crossList(crossNameBO);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "路口评价", notes = "路口评价", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/crossEvaluate",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = RunningEvaluateCrossEvaluateVO.class),
    })
    public JsonViewObject crossEvaluate(@RequestBody CrossIdAndStartEndDateBO bo) {
        RunningEvaluateCrossEvaluateVO res = runningEvaluateService.crossEvaluate(bo);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "拥堵情况", notes = "拥堵情况", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/congestionStatus",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = RunningEvaluateStatusVO.class),
    })
    public JsonViewObject congestionStatus(@RequestBody CrossIdAndStartEndDateBO bo) {
        RunningEvaluateStatusVO res = runningEvaluateService.congestionStatus(bo);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "失衡情况", notes = "失衡情况", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/unbalanceStatus",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = RunningEvaluateStatusVO.class),
    })
    public JsonViewObject unbalanceStatus(@RequestBody CrossIdAndStartEndDateBO bo) {
        RunningEvaluateStatusVO res = runningEvaluateService.unbalanceStatus(bo);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "溢出情况", notes = "溢出情况", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/spilloverStatus",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = RunningEvaluateStatusVO.class),
    })
    public JsonViewObject spilloverStatus(@RequestBody CrossIdAndStartEndDateBO bo) {
        RunningEvaluateStatusVO res = runningEvaluateService.spilloverStatus(bo);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "方案问题", notes = "方案问题", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/schemeProblems",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = RunningEvaluateSchemeProblemsVO.class),
    })
    public JsonViewObject schemeProblems(@RequestBody CrossIdAndStartEndDateBO bo) {
        List<RunningEvaluateSchemeProblemsVO> res = runningEvaluateService.schemeProblems(bo);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "详细指标查询-范围下拉列表", notes = "详细指标查询-范围下拉列表", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/scopeTree",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = RunningEvaluateScopeTreeVO.class),
    })
    public JsonViewObject scopeTree(@RequestBody CrossIdBO bo) {
        RunningEvaluateScopeTreeVO res = runningEvaluateService.scopeTree(bo);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "详细指标查询", notes = "详细指标查询", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/metricsDetail",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = RunningEvaluateMetricsDetailVO.class),
    })
    public JsonViewObject schemeProblems(@RequestBody MetricsDetailBO bo) {
        RunningEvaluateMetricsDetailVO res = runningEvaluateService.metricsDetail(bo);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "运行评价热力图", notes = "运行评价热力图", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/heatMap",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = RunningEvaluateHeatMapVO.class),
    })
    public JsonViewObject heatMap(@RequestBody HeatMapBO bo) {
        List<RunningEvaluateHeatMapVO> res = runningEvaluateService.heatMap(bo);
        return JsonViewObject.newInstance().success(res);
    }

}