package net.wanji.opt.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.opt.bo.CrossSchemeListBO;
import net.wanji.opt.bo.CurveChartBO;
import net.wanji.opt.bo.ProblemSchemeBO;
import net.wanji.opt.bo.SchemeDetailOverallBO;
import net.wanji.opt.service.impl.SchemeEvaluateServiceImpl;
import net.wanji.opt.vo.*;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.core.MediaType;
import java.text.ParseException;
import java.util.List;

/**
 * 方案评价
 *
 * @author Kent HAN
 */
@Api(value = "SchemeEvaluateController", description = "方案评价")
@RequestMapping("/schemeEvaluate")
@RestController
public class SchemeEvaluateController {
    private final SchemeEvaluateServiceImpl schemeEvaluateService;

    public SchemeEvaluateController(SchemeEvaluateServiceImpl schemeEvaluateService) {
        this.schemeEvaluateService = schemeEvaluateService;
    }

    @ApiOperation(value = "问题方案列表", notes = "问题方案列表", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/problemSchemeList",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SchemeEvaluateProblemSchemeVO.class),
    })
    public JsonViewObject problemSchemeList(@RequestBody ProblemSchemeBO problemSchemeBO) {
        SchemeEvaluateProblemSchemeVO res = schemeEvaluateService.problemSchemeList(problemSchemeBO);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "策略-评价指标下拉菜单", notes = "策略-评价指标下拉菜单", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/strategyMetricMenu",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SchemeEvaluateStrategyMetricMenuVO.class),
    })
    public JsonViewObject strategyMetricMenu(@RequestBody CrossIdBO crossIdBO) {
        List<SchemeEvaluateStrategyMetricMenuVO> res = schemeEvaluateService.strategyMetricMenu(crossIdBO);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "路口方案列表", notes = "路口方案列表", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/crossSchemeList",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SchemeEvaluateCrossSchemeListVO.class),
    })
    public JsonViewObject crossSchemeList(@RequestBody CrossSchemeListBO crossSchemeListBO) {
        List<SchemeEvaluateCrossSchemeListVO> res = schemeEvaluateService.crossSchemeList(crossSchemeListBO);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "方案详情-总体评价", notes = "方案详情-总体评价", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/schemeDetailOverall",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SchemeEvaluateSchemeDetailOverallVO.class),
    })
    public JsonViewObject schemeDetailOverall(@RequestBody SchemeDetailOverallBO schemeDetailOverallBO)
            throws Exception {
        SchemeEvaluateSchemeDetailOverallVO res = schemeEvaluateService.schemeDetailOverall(schemeDetailOverallBO);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "方案详情-详细问题", notes = "方案详情-详细问题", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/detailedProblem",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SchemeEvaluateSchemeDetailedProblemVO.class),
    })
    public JsonViewObject detailedProblem(@RequestBody SchemeDetailOverallBO schemeDetailOverallBO)
            throws ParseException {
        SchemeEvaluateSchemeDetailedProblemVO res = schemeEvaluateService.detailedProblem(schemeDetailOverallBO);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "方案详情-曲线图", notes = "方案详情-曲线图", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/curveChart",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SchemeEvaluateCurveChartVO.class),
    })
    public JsonViewObject curveChart(@RequestBody CurveChartBO curveChartBO)
            throws ParseException, NoSuchFieldException, IllegalAccessException {
        List<SchemeEvaluateCurveChartVO> res = schemeEvaluateService.curveChart(curveChartBO);
        return JsonViewObject.newInstance().success(res);
    }

}