package net.wanji.opt.controller.signalcontrol;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.common.annotation.aspect.AspectLog;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.opt.service.signalcontrol.FeignProxyService;
import net.wanji.opt.vo.SchemeOptSendVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotBlank;
import javax.ws.rs.core.MediaType;

/**
 * @author duanruiming
 * @date 2023/03/01 20:44
 */
@Api(value = "SignalCommandOptController", description = "信号优化")
@RequestMapping("/signalControl")
@RestController
public class SignalCommandOptController {

    @Autowired
    private FeignProxyService feignProxyService;

    @AspectLog(description = "诊断优化手动优化方案下发", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "手动优化方案下发", notes = "优化方案下发", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/schemeOptSend",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({@ApiResponse(code = 200, message = "OK", response = JsonViewObject.class)})
    public JsonViewObject schemeOptSend(@RequestBody @Validated SchemeOptSendVO schemeOptSendVO) throws Exception{
        JsonViewObject jsonViewObject = feignProxyService.schemeOptSend(schemeOptSendVO);
        return jsonViewObject;
    }

    @AspectLog(description = "诊断优化手动优化方案恢复", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "手动优化方案恢复", notes = "优化方案恢复", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/schemeOptRestore",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({@ApiResponse(code = 200, message = "OK", response = JsonViewObject.class)})
    public JsonViewObject schemeOptRestore(@RequestBody @NotBlank String crossId) throws Exception{
        JsonViewObject jsonViewObject = feignProxyService.schemeOptRestore(crossId);
        return jsonViewObject;
    }

}
