package net.wanji.opt.kafka;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.opt.dao.mapper.SynthesisOptimizeLogInfoMapper;
import net.wanji.opt.synthesis.pojo.SynthesisOptimizeLogEntity;
import net.wanji.opt.synthesis.pojo.SynthesisOptimizeLogInfo;
import org.apache.kafka.clients.consumer.Consumer;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.beans.BeanUtils;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.listener.KafkaListenerErrorHandler;
import org.springframework.kafka.listener.ListenerExecutionFailedException;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * @author duanruiming
 * @date 2024/11/05 20:45
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class SynthesisConsumerHandler  implements KafkaListenerErrorHandler {

    @Resource
    private SynthesisOptimizeLogInfoMapper synthesisOptimizeLogInfoMapper;

    private String str = "{\n" +
            "    \"cross_id\":\"cross0001\", \n" +
            "\t\"control_id\":\"123456\", \n" +
            "\t\"provider\":\"hisense\", \n" +
            "\t\"plan_no\":4, \n" +
            "\t\"cycle\":100, \n" +
            "\t\"offset\":10, \n" +
            "\t\"ring_cnt\":3, \n" +
            "\t\"ring_set\":[[[1,2],[3,4]],[[5,6],[7,8]],[[9,10],[11,12]]],\n" +
            "\t\"seq_id\":[1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16], \n" +
            "\t\"red_time\":[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],\n" +
            "\t\"yellow_time\":[3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3],\n" +
            "\t\"green_time\":[20,0,10,20,20,30,60,0,0,0,0,0,0,0,0,0],\n" +
            "\t\"green_flash\":[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],\n" +
            "\t\"person_flash\":[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0],\n" +
            "\t\"min_green\":[3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3], \n" +
            "\t\"max_green\":[100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100],\n" +
            "\t\"direction\":[[[1,2],[3,4]],[[5,6],[7,8]],[[9,10],[11,12]]],\n" +
            "\t\"timestamp\":\"2024-11-10 10:10:10\",\n" +
            "\t\"strategy\":0,\n" +
            "\t\"greenbelt_id\": \"road1\"\n" +
            "}";

    @Transactional
    @KafkaListener(topics = {"t_synthesis_optimize_log_info"})
    public void crossSchemeOptLogSave(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        try {
            String data = String.valueOf(record.value());
            if (StringUtils.isNotBlank(data)) {
                ObjectMapper instance = JacksonUtils.getInstance();
                SynthesisOptimizeLogInfo synthesisOptimizeLogInfo = instance.readValue(data, SynthesisOptimizeLogInfo.class);
                SynthesisOptimizeLogEntity synthesisOptimizeLogEntity = new SynthesisOptimizeLogEntity();
                BeanUtils.copyProperties(synthesisOptimizeLogInfo, synthesisOptimizeLogEntity);
                synthesisOptimizeLogEntity.setRingSet(instance.writeValueAsString(synthesisOptimizeLogInfo.getRingSet()));
                synthesisOptimizeLogEntity.setDirection(instance.writeValueAsString(synthesisOptimizeLogInfo.getDirection()));
                synthesisOptimizeLogEntity.setSeqId(instance.writeValueAsString(synthesisOptimizeLogInfo.getSeqId()));
                synthesisOptimizeLogEntity.setRedTime(instance.writeValueAsString(synthesisOptimizeLogInfo.getRedTime()));
                synthesisOptimizeLogEntity.setYellowTime(instance.writeValueAsString(synthesisOptimizeLogInfo.getYellowTime()));
                synthesisOptimizeLogEntity.setGreenTime(instance.writeValueAsString(synthesisOptimizeLogInfo.getGreenTime()));
                synthesisOptimizeLogEntity.setGreenFlash(instance.writeValueAsString(synthesisOptimizeLogInfo.getGreenFlash()));
                synthesisOptimizeLogEntity.setPersonFlash(instance.writeValueAsString(synthesisOptimizeLogInfo.getPersonFlash()));
                synthesisOptimizeLogEntity.setMinGreen(instance.writeValueAsString(synthesisOptimizeLogInfo.getMinGreen()));
                synthesisOptimizeLogEntity.setMaxGreen(instance.writeValueAsString(synthesisOptimizeLogInfo.getMaxGreen()));
                synthesisOptimizeLogInfoMapper.insert(synthesisOptimizeLogEntity);
            }
            acknowledgment.acknowledge();
        } catch (Exception e) {
            log.error("消费神思方案优化历史数据异常：", e);
            throw new Exception(e);
        }
    }


    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException e) {
        return new Object();
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException exception, Consumer<?, ?> consumer) {
        String errorMessage = String.format("监听主题：%s，消费者详情：%s，异常信息：%s，消息详情：%s",
                consumer.listTopics(), consumer.groupMetadata(), exception, message);
        log.error(errorMessage);
        return KafkaListenerErrorHandler.super.handleError(message, exception, consumer);
    }

    
}