package net.wanji.opt.service.impl;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.BeanListUtils;
import net.wanji.databus.dao.entity.BaseCrossSchedulesPlanPO;
import net.wanji.databus.dao.mapper.BaseCrossSchedulesMapper;
import net.wanji.opt.dto.CrossSchedulesDTO;
import net.wanji.opt.dto.CrossSectionDTO;
import net.wanji.opt.service.CrossSchedulesService;
import net.wanji.opt.service.CrossSectionService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author hfx
 * @date 2023/1/17 15:58
 * @desc CrossSchedulesServiceImpl
 */
@Service
@Slf4j
public class CrossSchedulesServiceImpl implements CrossSchedulesService {

    @Resource
    BaseCrossSchedulesMapper baseCrossSchedulesMapper;

    @Resource
    CrossSectionService crossSectionService;

    @Override
    public List<CrossSchedulesDTO> listCrossSchedulesDetails(String crossId, Integer status) {

        List<CrossSchedulesDTO> schedulesDTOList = new ArrayList<>();
        // 获取调度计划信息
        List<BaseCrossSchedulesPlanPO> schedulesPlanPOList = baseCrossSchedulesMapper.listSchedulesPlan(crossId, status);
        schedulesDTOList = BeanListUtils.populateList(schedulesPlanPOList, schedulesDTOList, CrossSchedulesDTO.class);
        // 获取日计划信息
        List<CrossSectionDTO> sectionPOList = crossSectionService.listCrossSectionInfo(crossId, null);
        // 处理日计划数据，封装成map：key是计划ID，value时段集合
        Map<Integer, List<CrossSectionDTO>> sectionMap = sectionPOList.stream().collect(Collectors.groupingBy(CrossSectionDTO::getPlanId));
        for(CrossSchedulesDTO schedules : schedulesDTOList) {
            schedules.setSectionInfos(sectionMap.get(schedules.getPlanId()));
        }
        return schedulesDTOList;
    }
}
