package net.wanji.opt.service.impl;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.BeanListUtils;
import net.wanji.databus.dao.entity.BaseCrossSchemePO;
import net.wanji.databus.dao.mapper.BaseCrossSchemeMapper;
import net.wanji.opt.dto.CrossPhaseDTO;
import net.wanji.opt.dto.CrossSchemeDTO;
import net.wanji.opt.service.CrossPhaseService;
import net.wanji.opt.service.CrossSchemeService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author hfx
 * @date 2023/1/17 15:05
 * @desc CrossSchemeServiceImpl
 */
@Service
@Slf4j
public class CrossSchemeServiceImpl implements CrossSchemeService {

    @Resource
    BaseCrossSchemeMapper baseCrossSchemeMapper;

    @Resource
    CrossPhaseService crossPhaseService;

    @Override
    public List<CrossSchemeDTO> listCrossSchemeInfo(String crossId) {

        List<BaseCrossSchemePO> schemePOList = baseCrossSchemeMapper.listCrossSchemeInfo(crossId, "", null);
        return BeanListUtils.populateList(schemePOList, new ArrayList<>(), CrossSchemeDTO.class);
    }

    @Override
    public List<CrossSchemeDTO> listCrossSchemeDetails(String crossId) {

        // 获取方案基础信息
        List<CrossSchemeDTO> schemeDTOList = listCrossSchemeInfo(crossId);
        // 获取相位详情信息
        List<CrossPhaseDTO> phaseDTOList = crossPhaseService.listPhaseDetails();
        // 处理相位数据，封装成map：key是方案ID，value相位信息
        Map<Integer, List<CrossPhaseDTO>> phaseMap = phaseDTOList.stream().collect(Collectors.groupingBy(CrossPhaseDTO::getSchemeId));
        for(CrossSchemeDTO scheme : schemeDTOList) {
            // 获取方案对应的相位列表
            scheme.setPhaseInfos(phaseMap.get(scheme.getId()));
        }
        return schemeDTOList;
    }

}
