package net.wanji.opt.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.enums.CrossInOutEnum;
import net.wanji.common.enums.StrategyAndMetricsEnum;
import net.wanji.common.enums.TurnConvertEnum;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.common.utils.tool.TimeArrayUtil;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.dao.entity.BaseCrossSchedulesPO;
import net.wanji.databus.dao.entity.BaseCrossSchedulesPlanPO;
import net.wanji.databus.dao.entity.BaseCrossSchemePO;
import net.wanji.databus.dao.entity.CrossSectionPO;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.*;
import net.wanji.opt.bo.CrossSchemeListBO;
import net.wanji.opt.bo.CurveChartBO;
import net.wanji.opt.bo.CurveChartBO.DirTurn;
import net.wanji.opt.bo.ProblemSchemeBO;
import net.wanji.opt.bo.SchemeDetailOverallBO;
import net.wanji.opt.dao.mapper.HoloEventMapper;
import net.wanji.opt.dao.mapper.strategy.SceneMapper;
import net.wanji.opt.dao.mapper.strategy.SceneStrategyMapper;
import net.wanji.opt.dao.mapper.strategy.StrategyMapper;
import net.wanji.opt.po.strategy.ScenePO;
import net.wanji.opt.po.strategy.SceneStrategyPO;
import net.wanji.opt.po.strategy.StrategyPO;
import net.wanji.opt.service.SchemeEvaluateService;
import net.wanji.opt.vo.*;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/8/25 16:09
 */
@Slf4j
@Service
public class SchemeEvaluateServiceImpl implements SchemeEvaluateService {
    private final CrossDataHistMapper crossDataHistMapper;
    private final RunningEvaluateServiceImpl runningEvaluateService;
    private final BaseCrossSectionMapper baseCrossSectionMapper;
    private final BaseCrossSchemeMapper baseCrossSchemeMapper;
    private final SceneMapper sceneMapper;
    private final SceneStrategyMapper sceneStrategyMapper;
    private final StrategyMapper strategyMapper;
    private final BaseCrossSchedulesMapper baseCrossSchedulesMapper;
    private final BaseCrossSchedulesPlanMapper baseCrossSchedulesPlanMapper;
    private final CrossDataRealtimeMapper crossDataRealtimeMapper;
    private final CrossDirDataHistMapper crossDirDataHistMapper;
    private final BaseCrossDirInfoMapper baseCrossDirInfoMapper;
    private final CrossTurnDataHistMapper crossTurnDataHistMapper;
    private final BaseCrossTurnInfoMapper baseCrossTurnInfoMapper;
    private final HoloEventMapper holoEventMapper;

    SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
    SimpleDateFormat timeFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");

    public SchemeEvaluateServiceImpl(CrossDataHistMapper crossDataHistMapper,
                                     RunningEvaluateServiceImpl runningEvaluateService,
                                     @Qualifier("baseCrossSectionMapper") BaseCrossSectionMapper baseCrossSectionMapper,
                                     @Qualifier("baseCrossSchemeMapper") BaseCrossSchemeMapper baseCrossSchemeMapper,
                                     @Qualifier("sceneMapper") SceneMapper sceneMapper,
                                     @Qualifier("sceneStrategyMapper") SceneStrategyMapper sceneStrategyMapper,
                                     @Qualifier("strategyMapper") StrategyMapper strategyMapper,
                                     @Qualifier("baseCrossSchedulesMapper") BaseCrossSchedulesMapper baseCrossSchedulesMapper,
                                     @Qualifier("baseCrossSchedulesPlanMapper") BaseCrossSchedulesPlanMapper baseCrossSchedulesPlanMapper,
                                     CrossDataRealtimeMapper crossDataRealtimeMapper, CrossDirDataHistMapper crossDirDataHistMapper,
                                     @Qualifier("baseCrossDirInfoMapper") BaseCrossDirInfoMapper baseCrossDirInfoMapper,
                                     CrossTurnDataHistMapper crossTurnDataHistMapper,
                                     @Qualifier("baseCrossTurnInfoMapper") BaseCrossTurnInfoMapper baseCrossTurnInfoMapper, @Qualifier("holoEventMapper") HoloEventMapper holoEventMapper) {
        this.crossDataHistMapper = crossDataHistMapper;
        this.runningEvaluateService = runningEvaluateService;
        this.baseCrossSectionMapper = baseCrossSectionMapper;
        this.baseCrossSchemeMapper = baseCrossSchemeMapper;
        this.sceneMapper = sceneMapper;
        this.sceneStrategyMapper = sceneStrategyMapper;
        this.strategyMapper = strategyMapper;
        this.baseCrossSchedulesMapper = baseCrossSchedulesMapper;
        this.baseCrossSchedulesPlanMapper = baseCrossSchedulesPlanMapper;
        this.crossDataRealtimeMapper = crossDataRealtimeMapper;
        this.crossDirDataHistMapper = crossDirDataHistMapper;
        this.baseCrossDirInfoMapper = baseCrossDirInfoMapper;
        this.crossTurnDataHistMapper = crossTurnDataHistMapper;
        this.baseCrossTurnInfoMapper = baseCrossTurnInfoMapper;
        this.holoEventMapper = holoEventMapper;
    }

    @Override
    public SchemeEvaluateProblemSchemeVO problemSchemeList(ProblemSchemeBO problemSchemeBO) {
        SchemeEvaluateProblemSchemeVO res = new SchemeEvaluateProblemSchemeVO();
        String crossId = problemSchemeBO.getCrossId();
        Date startDate = problemSchemeBO.getStartDate();
        Date endDate = problemSchemeBO.getEndDate();
        endDate = DateUtil.offsetDay(endDate, 1); // 包含最后一天
        int startDateStamp = (int) (startDate.getTime() / 1000); // 10位时间戳
        int endDateStamp = (int) (endDate.getTime() / 1000);
        String startTime = problemSchemeBO.getStartTime();
        String endTime = problemSchemeBO.getEndTime();
        List<CrossDataHistPO> crossDataHistPOList =
                crossDataHistMapper.selectByCrossIdAndStartEnd(crossId, startDateStamp, endDateStamp);
        // 构造时间-问题结果集
        Map<String, List<CrossDataHistPO>> timeProblemMap = getTimeProblemMap(
                crossDataHistPOList, endTime, startTime);
        // 构造结果
        int problemCounts = 0; // 问题次数
        if (CollectionUtil.isNotEmpty(timeProblemMap)) {
            for (List<CrossDataHistPO> list : timeProblemMap.values()) {
                problemCounts += list.size();
            }
        }
        res.setProblemCounts(problemCounts); // 问题方案数量
        // 构造问题方案列表（根据日期）
        Map<Date, List<CrossDataHistPO>> dateProblemsMap = buildDateProblemsMap(timeProblemMap); // 构造日期-问题结果集

        List<CrossSectionPO> crossSectionPOList = baseCrossSectionMapper.selectAll();
        List<BaseCrossSchemePO> crossSchemePOList = baseCrossSchemeMapper.selectAll();
        Map<Date, List<RunningEvaluateMetricsDetailVO.ProblemScheme>> dateSchemesMap =
                buildDateSchemesMap(dateProblemsMap, crossId, crossSectionPOList, crossSchemePOList); // 构造日期-问题方案结果集

        List<ScenePO> scenePOList = sceneMapper.selectAll();
        List<SchemeEvaluateProblemSchemeVO.ByDateVO> byDateVOList = buildProblemSchemeListByDate(
                dateProblemsMap, dateSchemesMap, scenePOList);
        res.setProblemSchemeListByDate(byDateVOList);
        // 构造计数
        int schemeCounts = calcSchemeCounts(dateSchemesMap);
        res.setSchemeCounts(schemeCounts);
        res.setEfficiencyProblemCounts(calcProblemCounts(
                byDateVOList, SchemeEvaluateProblemSchemeVO.ByDateVO::getEfficiencyProblemCounts));
        res.setBalanceProblemCounts(calcProblemCounts(
                byDateVOList, SchemeEvaluateProblemSchemeVO.ByDateVO::getBalanceProblemCounts));
        res.setSecurityProblemCounts(calcProblemCounts(
                byDateVOList, SchemeEvaluateProblemSchemeVO.ByDateVO::getSecurityProblemCounts));
        // 构造问题方案列表（根据方案）
        res.setProblemSchemeListByScheme(buildProblemSchemeListByScheme(dateSchemesMap));

        return res;
    }

    @Override
    public List<SchemeEvaluateStrategyMetricMenuVO> strategyMetricMenu(CrossIdBO crossIdBO) {
        List<SchemeEvaluateStrategyMetricMenuVO> voList = new ArrayList<>();

        for (Map.Entry<StrategyAndMetricsEnum.Strategy, List<StrategyAndMetricsEnum.Metrics>> entry
                : StrategyAndMetricsEnum.STRATEGY_METRICS_MAP.entrySet()) {
            SchemeEvaluateStrategyMetricMenuVO vo = new SchemeEvaluateStrategyMetricMenuVO();
            vo.setStrategyCode(entry.getKey().getCode());
            vo.setStrategyName(entry.getKey().getMsg());
            List<SchemeEvaluateStrategyMetricMenuVO.EvaluateMetric> metricInfoList = buildEvaluateMetrics(entry);
            vo.setMetricList(metricInfoList);
            voList.add(vo);
        }
        return voList;
    }

    private List<SchemeEvaluateStrategyMetricMenuVO.EvaluateMetric> buildEvaluateMetrics(
            Map.Entry<StrategyAndMetricsEnum.Strategy, List<StrategyAndMetricsEnum.Metrics>> entry) {
        List<SchemeEvaluateStrategyMetricMenuVO.EvaluateMetric> metricInfoList = new ArrayList<>();
        for (StrategyAndMetricsEnum.Metrics metric : entry.getValue()) {
            SchemeEvaluateStrategyMetricMenuVO.EvaluateMetric evaluateMetric =
                    new SchemeEvaluateStrategyMetricMenuVO.EvaluateMetric();
            String metricCode = metric.getCode();
            evaluateMetric.setMetricCode(metricCode);
            evaluateMetric.setMetricName(metric.getDescription());
            if (Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.LOAD_BALANCE.getCode())
                    || Objects.equals(metricCode, StrategyAndMetricsEnum.Metrics.CLEAR_RATE.getCode())) {
                evaluateMetric.setIsShownInDetail(0);
            }
            metricInfoList.add(evaluateMetric);
        }
        return metricInfoList;
    }

    @Override
    public List<SchemeEvaluateCrossSchemeListVO> crossSchemeList(CrossSchemeListBO crossSchemeListBO) {
        String crossId = crossSchemeListBO.getCrossId();
        String schemeNoOrName = crossSchemeListBO.getSchemeNoOrName();
        String strategyCode = crossSchemeListBO.getStrategyCode();
        // 查找方案
        List<BaseCrossSchemePO> baseCrossSchemePOList = new ArrayList<>();
        if (ObjectUtil.isNotEmpty(schemeNoOrName)) {
            if (isNumeric(schemeNoOrName)) { // 方案号
                String schemeNo = schemeNoOrName;
                BaseCrossSchemePO baseCrossSchemePO = baseCrossSchemeMapper.selectByCrossIdAndSchemeNo(
                        crossId, Integer.parseInt(schemeNo));
                baseCrossSchemePOList.add(baseCrossSchemePO);
            } else { // 方案名模糊搜索
                String schemeName = schemeNoOrName;
                baseCrossSchemePOList = baseCrossSchemeMapper.selectByCrossIdAndPartialSchemeName(
                        crossId, schemeName);
            }
        } else { // 全部方案
            baseCrossSchemePOList = baseCrossSchemeMapper.selectByCrossId(crossId);
        }

        Integer weekDay = crossSchemeListBO.getWeekDay();
        // 找出当前路口已执行的调度，在某天对应的计划
        BaseCrossSchedulesPO baseCrossSchedulesPO = baseCrossSchedulesMapper.selectExecByCrossId(crossId);
        if (ObjectUtil.isEmpty(baseCrossSchedulesPO)) {
            throw new RuntimeException("该路口没有正在执行的调度");
        }
        Integer scheduleId = baseCrossSchedulesPO.getId();
        List<BaseCrossSchedulesPlanPO> baseCrossSchedulesPlanPOList =
                baseCrossSchedulesPlanMapper.selectByCrossIdAndSchedulesId(crossId, scheduleId);
        List<BaseCrossSchedulesPlanPO> filteredList = baseCrossSchedulesPlanPOList.stream()
                .filter(po -> po.getWeek().equals(weekDay))
                .collect(Collectors.toList());

        List<SchemeEvaluateCrossSchemeListVO> res = new ArrayList<>();
        // 根据计划 ID 和方案 ID，获取时段信息
        for (BaseCrossSchedulesPlanPO baseCrossSchedulesPlanPO : filteredList) {
            Integer planId = baseCrossSchedulesPlanPO.getPlanId();
            for (BaseCrossSchemePO baseCrossSchemePO : baseCrossSchemePOList) {
                Integer schemeId = baseCrossSchemePO.getId();
                List<CrossSectionPO> crossSectionPOList = baseCrossSectionMapper.selectByCrossIdPlanIdAndSchemeId(
                        crossId, planId, schemeId);
                // todo 按控制模式和策略入参筛选结果
                for (CrossSectionPO crossSectionPO : crossSectionPOList) {
                    SchemeEvaluateCrossSchemeListVO vo =
                            buildSchemeEvaluateCrossSchemeListVO(baseCrossSchemePO, crossSectionPO);
                    res.add(vo);
                }
            }
        }
        // 特殊日期根据方案 ID 筛选重复方案
        if (weekDay == 0) {
            List<SchemeEvaluateCrossSchemeListVO> sortedUniqueList = res.stream()
                    .collect(Collectors.toMap(
                            SchemeEvaluateCrossSchemeListVO::getSchemeId,
                            vo -> vo,
                            (existing, replacement) -> existing))
                    .values()
                    .stream()
                    .sorted(Comparator.comparing(SchemeEvaluateCrossSchemeListVO::getStartTime))
                    .collect(Collectors.toList());
            return sortedUniqueList;
        }
        // 按照 startTime 排序
        List<SchemeEvaluateCrossSchemeListVO> sortedList = res.stream()
                .sorted(Comparator.comparing(SchemeEvaluateCrossSchemeListVO::getStartTime))
                .collect(Collectors.toList());
        return sortedList;
    }

    @Override
    public SchemeEvaluateSchemeDetailOverallVO schemeDetailOverall(SchemeDetailOverallBO schemeDetailOverallBO)
            throws ParseException {
        String crossId = schemeDetailOverallBO.getCrossId();
        String strategyCode = schemeDetailOverallBO.getStrategyCode();
        Date problemDate = schemeDetailOverallBO.getProblemDate();
        String startTime = schemeDetailOverallBO.getStartTime();
        String endTime = schemeDetailOverallBO.getEndTime();

        SchemeEvaluateSchemeDetailOverallVO vo = new SchemeEvaluateSchemeDetailOverallVO();
        // 查询起止时间内历史数据
        String dateStr = dateFormat.format(problemDate);
        String concatenatedStartTime = dateStr + " " + startTime;
        String concatenatedEndTime = dateStr + " " + endTime;
        int startTimeStamp = (int) (timeFormat.parse(concatenatedStartTime).getTime() / 1000);
        int endTimeStamp = (int) (timeFormat.parse(concatenatedEndTime).getTime() / 1000);
        List<CrossDataHistPO> crossDataHistPOList = crossDataHistMapper.selectByCrossIdAndStartEnd(
                crossId, startTimeStamp, endTimeStamp);
        List<CrossDirDataHistPO> crossDirDataHistPOList = crossDirDataHistMapper.selectByCrossIdAndStartEnd(
                crossId, startTimeStamp, endTimeStamp);
        // 服务水平
        double sumA = crossDataHistPOList.stream()
                .filter(item -> item.getBatchTime() >= startTimeStamp && item.getBatchTime() <= endTimeStamp)
                .mapToDouble(item -> item.getFlow() * item.getSturation())
                .sum();
        double sumB = crossDataHistPOList.stream()
                .filter(item -> item.getBatchTime() >= startTimeStamp && item.getBatchTime() <= endTimeStamp)
                .mapToDouble(CrossDataHistPO::getFlow)
                .sum();
        double saturation = sumA / sumB;
        String serviceLevel = CrossUtil.getServiceLevel(saturation);
        vo.setServiceLevel(serviceLevel);

        if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.EFFICIENCY.getCode())) {
            buildEfficiencyVO(crossDataHistPOList, crossDirDataHistPOList, vo, crossId, startTimeStamp, endTimeStamp);
        } else if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.BALANCE.getCode())) {
            buildBalanceVO(crossDataHistPOList, crossDirDataHistPOList, vo, crossId, startTimeStamp, endTimeStamp);
        } else if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.SECURITY.getCode())) {
            buildSecurityVO(crossDataHistPOList, crossDirDataHistPOList, vo, crossId, startTimeStamp, endTimeStamp);
        }

        return vo;
    }

    @Override
    public SchemeEvaluateSchemeDetailedProblemVO detailedProblem(SchemeDetailOverallBO schemeDetailOverallBO)
            throws ParseException {
        String crossId = schemeDetailOverallBO.getCrossId();
        Date problemDate = schemeDetailOverallBO.getProblemDate();
        String startTime = schemeDetailOverallBO.getStartTime();
        String endTime = schemeDetailOverallBO.getEndTime();
        String strategyCode = schemeDetailOverallBO.getStrategyCode();

        // 查询起止时间内历史数据
        String dateStr = dateFormat.format(problemDate);
        String concatenatedStartTime = dateStr + " " + startTime;
        String concatenatedEndTime = dateStr + " " + endTime;
        int startTimeStamp = (int) (timeFormat.parse(concatenatedStartTime).getTime() / 1000);
        int endTimeStamp = (int) (timeFormat.parse(concatenatedEndTime).getTime() / 1000);
        List<CrossDirDataHistPO> crossDirDataHistPOList = crossDirDataHistMapper.selectByCrossIdAndStartEnd(
                crossId, startTimeStamp, endTimeStamp);
        List<CrossTurnDataHistPO> crossTurnDataHistPOList =
                crossTurnDataHistMapper.selectByCrossId(crossId, endTimeStamp, startTimeStamp);

        SchemeEvaluateSchemeDetailedProblemVO vo = new SchemeEvaluateSchemeDetailedProblemVO();

        if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.EFFICIENCY.getCode())) {
            buildEfficiencyDetailVO(
                    crossDirDataHistPOList, crossTurnDataHistPOList, vo, crossId);
        } else if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.BALANCE.getCode())) {
            buildBalanceDetailVO(
                    crossDirDataHistPOList, crossTurnDataHistPOList, vo, crossId);
        } else if (Objects.equals(strategyCode, StrategyAndMetricsEnum.Strategy.SECURITY.getCode())) {
            buildSecurityDetailVO(
                    crossDirDataHistPOList, crossTurnDataHistPOList, vo, crossId, startTimeStamp, endTimeStamp);
        }

        return vo;
    }

    @Override
    public List<SchemeEvaluateCurveChartVO> curveChart(CurveChartBO curveChartBO)
            throws ParseException, NoSuchFieldException, IllegalAccessException {
        String crossId = curveChartBO.getCrossId();
        Integer minutes = curveChartBO.getMinutes();

        Date problemDate = curveChartBO.getProblemDate();
        String startTime = curveChartBO.getStartTime();
        String endTime = curveChartBO.getEndTime();
        String dateStr = dateFormat.format(problemDate);
        String concatenatedStartTime = dateStr + " " + startTime;
        String concatenatedEndTime = dateStr + " " + endTime;
        int startTimeStamp = (int) (timeFormat.parse(concatenatedStartTime).getTime() / 1000);
        int endTimeStamp = (int) (timeFormat.parse(concatenatedEndTime).getTime() / 1000);


        String metricCode = curveChartBO.getMetricCode();

        List<DirTurn> dirTurnList = curveChartBO.getDirTurnList();

        List<SchemeEvaluateCurveChartVO> res = new ArrayList<>();
        List<String> minuteSectionArray = TimeArrayUtil.getCustomTimeIntervals(startTime, endTime, minutes);

        for (DirTurn dirTurn : dirTurnList) {
            String turnType = dirTurn.getTurn();
            String dirCode = dirTurn.getDir();
            int dirInt = Integer.parseInt(dirCode);
            if (ObjectUtil.isEmpty(turnType)) {
                // 查询该方向所有的车道ID
                List<String> laneIds = baseCrossDirInfoMapper.selectLaneIds(crossId, dirInt);
                // 查询该方向，该时段内全量数据
                List<CrossDirDataHistPOExt> crossDirDataHistPOList = crossDirDataHistMapper.selectByMetrics(
                        crossId, dirInt, startTimeStamp, endTimeStamp);
                Integer emergencyCount = holoEventMapper.selectEmergencyCountWithLaneIds(
                        crossId, startTimeStamp, endTimeStamp, laneIds);
                for (CrossDirDataHistPOExt po : crossDirDataHistPOList) {
                    po.setEmergencyCount(emergencyCount);
                }

                // 按时间段分组
                Map<String, List<CrossDirDataHistPOExt>> groupedByTime = crossDirDataHistPOList.stream()
                        .collect(Collectors.groupingBy(po -> {
                            long timestampInMillSeconds = po.getBatchTime() * 1000L;
                            Date date = new Date(timestampInMillSeconds);
                            SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
                            int minuteOfDay = date.getMinutes() + date.getHours() * 60;
                            int roundedMinute = (minuteOfDay / minutes) * minutes;
                            date.setHours(roundedMinute / 60);
                            date.setMinutes(roundedMinute % 60);
                            return sdf.format(date);
                        }));

                // 按照时间粒度聚合
                for (String section : minuteSectionArray) {
                    SchemeEvaluateCurveChartVO vo = new SchemeEvaluateCurveChartVO();
                    vo.setMetricTime(section);
                    vo.setDir(dirCode);
                    vo.setTurn(turnType);
                    List<CrossDirDataHistPOExt> poList = groupedByTime.get(section);

                    String metricField = StrategyAndMetricsEnum.Metrics.getFieldByCode(metricCode);
                    Field declaredField = CrossDirDataHistPOExt.class.getField(metricField);
                    declaredField.setAccessible(true);

                    if (CollectionUtil.isNotEmpty(poList)) {
                        double metricSum = 0;

                        for (CrossDirDataHistPOExt crossDirDataHistPOExt : poList) {
                            Double doubleValue = getDirDouble(crossDirDataHistPOExt, declaredField);
                            metricSum += doubleValue;
                        }
                        int size = poList.size();
                        double v = metricSum / size;
                        BigDecimal bd = new BigDecimal(v);
                        bd = bd.setScale(2, RoundingMode.HALF_UP);
                        double roundedValue = bd.doubleValue();
                        vo.setMetricValue(roundedValue);
                    }
                    res.add(vo);
                }
            } else {
                // 查询该转向所有的车道ID
                Integer intTurnCode = BaseEnum.TurnTypeEnum.getIntCodeByStrCode(turnType);
                List<String> laneIds = baseCrossTurnInfoMapper.selectLaneIds(crossId, dirInt, intTurnCode);
                // 查询该方向，该转向，该时段内全量数据
                List<CrossTurnDataHistPOExt> crossTurnDataHistPOList = crossTurnDataHistMapper.selectByMetrics(
                        crossId, dirInt, turnType, startTimeStamp, endTimeStamp);
                Integer emergencyCount = holoEventMapper.selectEmergencyCountWithLaneIds(
                        crossId, startTimeStamp, endTimeStamp, laneIds);
                for (CrossTurnDataHistPOExt crossTurnDataHistPOExt : crossTurnDataHistPOList) {
                    crossTurnDataHistPOExt.setEmergencyCount(emergencyCount);
                }

                // 按时间段分组
                Map<String, List<CrossTurnDataHistPOExt>> groupedByTime = crossTurnDataHistPOList.stream()
                        .collect(Collectors.groupingBy(po -> {
                            long timestampInMillSeconds = po.getBatchTime() * 1000L;
                            Date date = new Date(timestampInMillSeconds);
                            SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
                            int minuteOfDay = date.getMinutes() + date.getHours() * 60;
                            int roundedMinute = (minuteOfDay / minutes) * minutes;
                            date.setHours(roundedMinute / 60);
                            date.setMinutes(roundedMinute % 60);
                            return sdf.format(date);
                        }));

                // 按照时间粒度聚合
                for (String section : minuteSectionArray) {
                    SchemeEvaluateCurveChartVO vo = new SchemeEvaluateCurveChartVO();
                    vo.setMetricTime(section);
                    vo.setDir(dirCode);
                    vo.setTurn(turnType);
                    List<CrossTurnDataHistPOExt> poList = groupedByTime.get(section);

                    String metricField = StrategyAndMetricsEnum.Metrics.getFieldByCode(metricCode);
                    Field declaredField = CrossTurnDataHistPOExt.class.getField(metricField);
                    declaredField.setAccessible(true);

                    if (CollectionUtil.isNotEmpty(poList)) {
                        double metricSum = 0;

                        for (CrossTurnDataHistPOExt crossTurnDataHistPOExt : poList) {
                            Double doubleValue = getTurnDouble(crossTurnDataHistPOExt, declaredField);
                            metricSum += doubleValue;
                        }
                        int size = poList.size();
                        double v = metricSum / size;
                        BigDecimal bd = new BigDecimal(v);
                        bd = bd.setScale(2, RoundingMode.HALF_UP);
                        double roundedValue = bd.doubleValue();
                        vo.setMetricValue(roundedValue);
                    }
                    res.add(vo);
                }
            }
        }

        return res;
    }

    private static final Set<String> SPECIAL_METRICS = new HashSet<>();

    static {
        SPECIAL_METRICS.add(StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getField());
        SPECIAL_METRICS.add(StrategyAndMetricsEnum.Metrics.STOP_RATE.getField());
        SPECIAL_METRICS.add(StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getField());
        SPECIAL_METRICS.add(StrategyAndMetricsEnum.Metrics.EFFUSION_RATE.getField());
        SPECIAL_METRICS.add(StrategyAndMetricsEnum.Metrics.CLEAR_RATE.getField());
    }

    private Double getDoubleFromObject(Object object, Field declaredField) throws IllegalAccessException {
        Object objValue = declaredField.get(object);
        if (ObjectUtil.isEmpty(objValue)) {
            objValue = 0;
        }
        String fieldName = declaredField.getName();
        if (SPECIAL_METRICS.contains(fieldName) && objValue instanceof Double) {
            objValue = (double) Math.round((Double) objValue * 100);
        }
        if (objValue instanceof Integer) {
            return (double) objValue;
        } else if (objValue instanceof Double) {
            BigDecimal bd = BigDecimal.valueOf((double) objValue);
            bd = bd.setScale(2, RoundingMode.HALF_UP);
            return bd.doubleValue();
        } else {
            throw new IllegalArgumentException("字段值不是 int 或 double");
        }
    }

    private Double getTurnDouble(CrossTurnDataHistPOExt crossTurnDataHistPO, Field declaredField)
            throws IllegalAccessException {
        return getDoubleFromObject(crossTurnDataHistPO, declaredField);
    }

    private Double getDirDouble(CrossDirDataHistPOExt crossDirDataHistPO, Field declaredField)
            throws IllegalAccessException {
        return getDoubleFromObject(crossDirDataHistPO, declaredField);
    }

    private void buildSecurityDetailVO(List<CrossDirDataHistPO> crossDirDataHistPOList,
                                       List<CrossTurnDataHistPO> crossTurnDataHistPOList,
                                       SchemeEvaluateSchemeDetailedProblemVO vo, String crossId,
                                       int startTimeStamp, int endTimeStamp) {
        List<String> problemList = new ArrayList<>();
        List<SchemeEvaluateSchemeDetailedProblemVO.TableContent> tableContentList = new ArrayList<>();
        // 获取路口方向
        List<Integer> dirCodes = baseCrossDirInfoMapper.selectInDirsByCrossId(crossId);
        for (Integer dirCode : dirCodes) {
            Map<String, String> metricsMap = new HashMap<>();

            SchemeEvaluateSchemeDetailedProblemVO.TableContent tableContent =
                    new SchemeEvaluateSchemeDetailedProblemVO.TableContent();
            tableContent.setPosition(BaseEnum.SignalDirectionEnum.getNameByCode(dirCode) + "进口");
            tableContent.setPositionCode(dirCode.toString());

            // 流量加总
            int flow = crossDirDataHistPOList.stream()
                    .filter(item -> Objects.equals(item.getDirType(), dirCode)
                            && Objects.equals(item.getInOutType(), CrossInOutEnum.IN.getCode()))
                    .mapToInt(CrossDirDataHistPO::getFlow)
                    .sum();
            tableContent.setFlow(flow);
            // 溢流率取最大
            double effusionRateMax = crossDirDataHistPOList.stream()
                    .filter(item -> Objects.equals(item.getDirType(), dirCode)
                            && Objects.equals(item.getInOutType(), CrossInOutEnum.IN.getCode()))
                    .mapToDouble(CrossDirDataHistPO::getEffusionRate)
                    .max()
                    .orElse(0.0);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.EFFUSION_RATE.getCode(),
                    effusionRateMax * 100 + StrategyAndMetricsEnum.Metrics.EFFUSION_RATE.getUnit());
            // 方向级别三急一速数量
            // 查询该方向所有的车道ID
            List<String> laneIds = baseCrossDirInfoMapper.selectLaneIds(crossId, dirCode);
            // 加总该方向的三急一速
            Integer emergencyCount = holoEventMapper.selectEmergencyCountWithLaneIds(
                    crossId, startTimeStamp, endTimeStamp, laneIds);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.EMERGENCY_COUNT.getCode(),
                    emergencyCount + StrategyAndMetricsEnum.Metrics.EMERGENCY_COUNT.getUnit());

            // 筛选当前方向的转向数据
            List<CrossTurnDataHistPO> dirTurnPOList = crossTurnDataHistPOList.stream()
                    .filter(item -> Objects.equals(item.getInDir(), dirCode))
                    .collect(Collectors.toList());
            tableContent.setSubList(buildSecuritySublist(dirTurnPOList, dirCode, crossId,
                    startTimeStamp, endTimeStamp));
            tableContent.setMetricsMap(metricsMap);

            for (SchemeEvaluateSchemeDetailedProblemVO.SubTableContent content : tableContent.getSubList()) {
                if (content.getHasProblem() == 1) {
                    tableContent.setHasProblem(1);
                }
            }

            tableContentList.add(tableContent);
        }

        vo.setTableContentList(tableContentList);

        // 计算指标是否合格
        List<CrossDataHistPO> crossDataHistPOList = crossDataHistMapper.selectByCrossIdAndStartEnd(
                crossId, startTimeStamp, endTimeStamp);
        Double maxCrossSaturation = crossDataHistMapper.selectMaxSaturation(crossId, startTimeStamp, endTimeStamp);
        // 绿灯间隔清空率取最大
        OptionalDouble clearRateMax = crossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getClearRate)
                .max();
        boolean isQualified = calcSecurity(maxCrossSaturation, clearRateMax);
        if (!isQualified) {
            problemList.add("路口存在安全隐患");
        } else {
            problemList.add("路口满足安全保障需求");
        }
        vo.setProblems(problemList);
    }

    private List<SchemeEvaluateSchemeDetailedProblemVO.SubTableContent> buildSecuritySublist(
            List<CrossTurnDataHistPO> currentDirTurnPOList, Integer dirCode, String crossId,
            int startTimeStamp, int endTimeStamp) {
        List<SchemeEvaluateSchemeDetailedProblemVO.SubTableContent> tableContentList = new ArrayList<>();
        // 获取该方向的每个转向
        List<String> turnTypes = currentDirTurnPOList.stream()
                .map(CrossTurnDataHistPO::getTurnType)
                .distinct()
                .collect(Collectors.toList());

        for (String turnType : turnTypes) {
            SchemeEvaluateSchemeDetailedProblemVO.SubTableContent tableContent =
                    new SchemeEvaluateSchemeDetailedProblemVO.SubTableContent();

            tableContent.setPosition(TurnConvertEnum.getDescByCode(turnType));
            tableContent.setPositionCode(turnType);
            tableContent.setDirCode(dirCode.toString());
            Map<String, String> metricsMap = new HashMap<>();

            // 流量加总
            int flow = currentDirTurnPOList.stream()
                    .filter(item -> Objects.equals(item.getTurnType(), turnType))
                    .mapToInt(CrossTurnDataHistPO::getFlow)
                    .sum();
            tableContent.setFlow(flow);
            // 溢流率取最大
            double effusionRateMax = currentDirTurnPOList.stream()
                    .filter(item -> Objects.equals(item.getTurnType(), turnType))
                    .mapToDouble(CrossTurnDataHistPO::getEffusionRate)
                    .max()
                    .orElse(0.0);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.EFFUSION_RATE.getCode(),
                    effusionRateMax * 100 + StrategyAndMetricsEnum.Metrics.EFFUSION_RATE.getUnit());
            // 转向级别三急一速数量
            // 查询该转向所有的车道ID
            Integer intTurnCode = BaseEnum.TurnTypeEnum.getIntCodeByStrCode(turnType);
            List<String> laneIds = baseCrossTurnInfoMapper.selectLaneIds(crossId, dirCode, intTurnCode);
            // 加总该转向的三急一速
            Integer emergencyCount = holoEventMapper.selectEmergencyCountWithLaneIds(
                    crossId, startTimeStamp, endTimeStamp, laneIds);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.EMERGENCY_COUNT.getCode(),
                    emergencyCount + StrategyAndMetricsEnum.Metrics.EMERGENCY_COUNT.getUnit());

            tableContent.setMetricsMap(metricsMap);
            tableContentList.add(tableContent);

        }
        return tableContentList;
    }

    private void buildBalanceDetailVO(List<CrossDirDataHistPO> crossDirDataHistPOList,
                                      List<CrossTurnDataHistPO> crossTurnDataHistPOList,
                                      SchemeEvaluateSchemeDetailedProblemVO vo, String crossId) {
        List<String> problemList = new ArrayList<>();
        List<SchemeEvaluateSchemeDetailedProblemVO.TableContent> tableContentList = new ArrayList<>();
        // 获取路口方向
        List<Integer> dirCodes = baseCrossDirInfoMapper.selectInDirsByCrossId(crossId);
        for (Integer dirCode : dirCodes) {
            Map<String, String> metricsMap = new HashMap<>();

            SchemeEvaluateSchemeDetailedProblemVO.TableContent tableContent =
                    new SchemeEvaluateSchemeDetailedProblemVO.TableContent();
            tableContent.setPosition(BaseEnum.SignalDirectionEnum.getNameByCode(dirCode) + "进口");
            tableContent.setPositionCode(dirCode.toString());

            // 流量加总
            int flow = crossDirDataHistPOList.stream()
                    .filter(item -> Objects.equals(item.getDirType(), dirCode)
                            && Objects.equals(item.getInOutType(), CrossInOutEnum.IN.getCode()))
                    .mapToInt(CrossDirDataHistPO::getFlow)
                    .sum();
            tableContent.setFlow(flow);
            // 绿灯有效利用率取最大
            double greenLightEfficiencyMax = crossDirDataHistPOList.stream()
                    .filter(item -> Objects.equals(item.getDirType(), dirCode)
                            && Objects.equals(item.getInOutType(), CrossInOutEnum.IN.getCode()))
                    .mapToDouble(CrossDirDataHistPO::getGreenLightEfficiency)
                    .max()
                    .orElse(0.0);
            long greenLightEfficiencyMaxLong = Math.round(greenLightEfficiencyMax * 100);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getCode(),
                    greenLightEfficiencyMaxLong + StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getUnit());
            // 饱和度取最大
            double maxSaturation = crossDirDataHistPOList.stream()
                    .filter(item -> Objects.equals(item.getDirType(), dirCode)
                            && Objects.equals(item.getInOutType(), CrossInOutEnum.IN.getCode()))
                    .mapToDouble(CrossDirDataHistPO::getSturation)
                    .max()
                    .orElse(0.0);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.SATURATION.getCode(),
                    maxSaturation + StrategyAndMetricsEnum.Metrics.SATURATION.getUnit());

            // 筛选当前方向的转向数据
            List<CrossTurnDataHistPO> dirTurnPOList = crossTurnDataHistPOList.stream()
                    .filter(item -> Objects.equals(item.getInDir(), dirCode))
                    .collect(Collectors.toList());
            tableContent.setSubList(buildBalanceSublist(problemList, dirTurnPOList, dirCode));
            tableContent.setMetricsMap(metricsMap);

            for (SchemeEvaluateSchemeDetailedProblemVO.SubTableContent content : tableContent.getSubList()) {
                if (content.getHasProblem() == 1) {
                    tableContent.setHasProblem(1);
                }
            }

            tableContentList.add(tableContent);
        }

        vo.setTableContentList(tableContentList);
        if (CollectionUtil.isNotEmpty(problemList)) {
            problemList.add("其他方向均衡状况良好");
        } else {
            problemList.add("各方向均衡状况良好");
        }
        vo.setProblems(problemList);
    }

    private List<SchemeEvaluateSchemeDetailedProblemVO.SubTableContent> buildBalanceSublist(
            List<String> problemList, List<CrossTurnDataHistPO> currentDirTurnPOList, Integer dirCode) {
        List<SchemeEvaluateSchemeDetailedProblemVO.SubTableContent> tableContentList = new ArrayList<>();
        // 获取该方向的每个转向
        List<String> turnTypes = currentDirTurnPOList.stream()
                .map(CrossTurnDataHistPO::getTurnType)
                .distinct()
                .collect(Collectors.toList());

        for (String turnType : turnTypes) {
            SchemeEvaluateSchemeDetailedProblemVO.SubTableContent tableContent =
                    new SchemeEvaluateSchemeDetailedProblemVO.SubTableContent();

            tableContent.setPosition(TurnConvertEnum.getDescByCode(turnType));
            tableContent.setPositionCode(turnType);
            tableContent.setDirCode(dirCode.toString());

            Map<String, String> metricsMap = new HashMap<>();

            // 流量加总
            int flow = currentDirTurnPOList.stream()
                    .filter(item -> Objects.equals(item.getTurnType(), turnType))
                    .mapToInt(CrossTurnDataHistPO::getFlow)
                    .sum();
            tableContent.setFlow(flow);
            // 绿灯有效利用率取最大
            double greenLightEfficiencyMax = currentDirTurnPOList.stream()
                    .filter(item -> Objects.equals(item.getTurnType(), turnType))
                    .mapToDouble(CrossTurnDataHistPO::getGreenLightEfficiency)
                    .max()
                    .orElse(0.0);
            long greenLightEfficiencyMaxLong = Math.round(greenLightEfficiencyMax * 100);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getCode(),
                    greenLightEfficiencyMaxLong + StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getUnit());
            // 饱和度取最大
            double maxSaturation = currentDirTurnPOList.stream()
                    .filter(item -> Objects.equals(item.getTurnType(), turnType))
                    .mapToDouble(CrossTurnDataHistPO::getSturation)
                    .max()
                    .orElse(0);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.SATURATION.getCode(),
                    maxSaturation + StrategyAndMetricsEnum.Metrics.SATURATION.getUnit());

            // 计算指标是否合格
            boolean isQualified = calcBalance(maxSaturation, greenLightEfficiencyMax);
            if (!isQualified) {
                tableContent.setHasProblem(1);
                problemList.add(BaseEnum.SignalDirectionEnum.getNameByCode(dirCode) + "进口"
                        + TurnConvertEnum.getDescByCode(turnType) + "失衡");
            }

            tableContent.setMetricsMap(metricsMap);
            tableContentList.add(tableContent);

        }
        return tableContentList;
    }

    private void buildEfficiencyDetailVO(List<CrossDirDataHistPO> crossDirDataHistPOList,
                                         List<CrossTurnDataHistPO> crossTurnDataHistPOList,
                                         SchemeEvaluateSchemeDetailedProblemVO vo, String crossId) {
        List<String> problemList = new ArrayList<>();
        List<SchemeEvaluateSchemeDetailedProblemVO.TableContent> tableContentList = new ArrayList<>();

        // 获取路口方向
        List<Integer> dirCodes = baseCrossDirInfoMapper.selectInDirsByCrossId(crossId);
        for (Integer dirCode : dirCodes) {
            Map<String, String> metricsMap = new HashMap<>();

            SchemeEvaluateSchemeDetailedProblemVO.TableContent tableContent =
                    new SchemeEvaluateSchemeDetailedProblemVO.TableContent();
            tableContent.setPosition(BaseEnum.SignalDirectionEnum.getNameByCode(dirCode) + "进口");
            tableContent.setPositionCode(dirCode.toString());

            // 流量加总
            int flow = crossDirDataHistPOList.stream()
                    .filter(item -> Objects.equals(item.getDirType(), dirCode)
                            && Objects.equals(item.getInOutType(), CrossInOutEnum.IN.getCode()))
                    .mapToInt(CrossDirDataHistPO::getFlow)
                    .sum();
            tableContent.setFlow(flow);
            // 速度取平均
            double speed = crossDirDataHistPOList.stream()
                    .filter(item -> Objects.equals(item.getDirType(), dirCode)
                            && Objects.equals(item.getInOutType(), CrossInOutEnum.IN.getCode()))
                    .mapToDouble(CrossDirDataHistPO::getSpeed)
                    .average()
                    .orElse(0.0);
            long speedLong = Math.round(speed);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getCode(),
                    speedLong + StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getUnit());
            // 延误取最大
            int maxDelayTime = crossDirDataHistPOList.stream()
                    .filter(item -> Objects.equals(item.getDirType(), dirCode)
                            && Objects.equals(item.getInOutType(), CrossInOutEnum.IN.getCode()))
                    .mapToInt(CrossDirDataHistPO::getDelayTime)
                    .max()
                    .orElse(0);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode(),
                    maxDelayTime + StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getUnit());
            // 排队长度取最大
            double maxQueueLength = crossDirDataHistPOList.stream()
                    .filter(item -> Objects.equals(item.getDirType(), dirCode)
                            && Objects.equals(item.getInOutType(), CrossInOutEnum.IN.getCode()))
                    .mapToDouble(CrossDirDataHistPO::getQueueLength)
                    .max()
                    .orElse(0.0);
            long queueLengthLong = Math.round(maxQueueLength);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode(),
                    queueLengthLong + StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getUnit());
            // 不停车通过率取平均
            double noStopRate = crossDirDataHistPOList.stream()
                    .filter(item -> Objects.equals(item.getDirType(), dirCode)
                            && Objects.equals(item.getInOutType(), CrossInOutEnum.IN.getCode()))
                    .mapToDouble(CrossDirDataHistPO::getNoStopRate)
                    .average()
                    .orElse(0.0);
            long noStopRateLong = Math.round(noStopRate * 100);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getCode(),
                    noStopRateLong + StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getUnit());
            // 一/二/三次及以上停车通过率取最大
            double stopRate = crossDirDataHistPOList.stream()
                    .filter(item -> Objects.equals(item.getDirType(), dirCode)
                            && Objects.equals(item.getInOutType(), CrossInOutEnum.IN.getCode()))
                    .mapToDouble(CrossDirDataHistPO::getOneStopRate)
                    .max()
                    .orElse(0.0);
            long stopRateLong = Math.round(stopRate * 100);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.STOP_RATE.getCode(),
                    stopRateLong + StrategyAndMetricsEnum.Metrics.STOP_RATE.getUnit());
            // 停车次数取最大
            double stopTimes = crossDirDataHistPOList.stream()
                    .filter(item -> Objects.equals(item.getDirType(), dirCode)
                            && Objects.equals(item.getInOutType(), CrossInOutEnum.IN.getCode()))
                    .mapToDouble(CrossDirDataHistPO::getStopTimes)
                    .max()
                    .orElse(0.0);
            long stopTimesLong = Math.round(stopTimes);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode(),
                    stopTimesLong + StrategyAndMetricsEnum.Metrics.STOP_TIMES.getUnit());

            // 筛选当前方向的转向数据
            List<CrossTurnDataHistPO> dirTurnPOList = crossTurnDataHistPOList.stream()
                    .filter(item -> Objects.equals(item.getInDir(), dirCode))
                    .collect(Collectors.toList());
            tableContent.setSubList(buildEfficiencySublist(problemList, dirTurnPOList, dirCode));
            tableContent.setMetricsMap(metricsMap);

            for (SchemeEvaluateSchemeDetailedProblemVO.SubTableContent content : tableContent.getSubList()) {
                if (content.getHasProblem() == 1) {
                    tableContent.setHasProblem(1);
                }
            }

            tableContentList.add(tableContent);
        }

        vo.setTableContentList(tableContentList);
        if (CollectionUtil.isNotEmpty(problemList)) {
            problemList.add("其他方向运行效率良好");
        } else {
            problemList.add("各方向运行效率良好");
        }
        vo.setProblems(problemList);
    }

    private List<SchemeEvaluateSchemeDetailedProblemVO.SubTableContent> buildEfficiencySublist(
            List<String> problemList, List<CrossTurnDataHistPO> currentDirTurnPOList, Integer dirCode) {
        List<SchemeEvaluateSchemeDetailedProblemVO.SubTableContent> tableContentList = new ArrayList<>();
        // 获取该方向的每个转向
        List<String> turnTypes = currentDirTurnPOList.stream()
                .map(CrossTurnDataHistPO::getTurnType)
                .distinct()
                .collect(Collectors.toList());

        for (String turnType : turnTypes) {
            SchemeEvaluateSchemeDetailedProblemVO.SubTableContent tableContent =
                    new SchemeEvaluateSchemeDetailedProblemVO.SubTableContent();

            tableContent.setPosition(TurnConvertEnum.getDescByCode(turnType));
            tableContent.setPositionCode(turnType);
            tableContent.setDirCode(dirCode.toString());

            Map<String, String> metricsMap = new HashMap<>();

            // 流量加总
            int flow = currentDirTurnPOList.stream()
                    .filter(item -> Objects.equals(item.getTurnType(), turnType))
                    .mapToInt(CrossTurnDataHistPO::getFlow)
                    .sum();
            tableContent.setFlow(flow);
            // 速度取平均
            double speed = currentDirTurnPOList.stream()
                    .filter(item -> Objects.equals(item.getTurnType(), turnType))
                    .mapToDouble(CrossTurnDataHistPO::getSpeed)
                    .average()
                    .orElse(0.0);
            long speedLong = Math.round(speed);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getCode(),
                    speedLong + StrategyAndMetricsEnum.Metrics.AVERAGE_SPEED.getUnit());
            // 延误取最大
            int maxDelayTime = currentDirTurnPOList.stream()
                    .filter(item -> Objects.equals(item.getTurnType(), turnType))
                    .mapToInt(CrossTurnDataHistPO::getDelayTime)
                    .max()
                    .orElse(0);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode(),
                    maxDelayTime + StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getUnit());
            // 排队长度取最大
            double maxQueueLength = currentDirTurnPOList.stream()
                    .filter(item -> Objects.equals(item.getTurnType(), turnType))
                    .mapToDouble(CrossTurnDataHistPO::getQueueLength)
                    .max()
                    .orElse(0.0);
            long queueLengthLong = Math.round(maxQueueLength);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode(),
                    queueLengthLong + StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getUnit());
            // 不停车通过率取平均
            double noStopRate = currentDirTurnPOList.stream()
                    .filter(item -> Objects.equals(item.getTurnType(), turnType))
                    .mapToDouble(CrossTurnDataHistPO::getNoStopRate)
                    .average()
                    .orElse(0.0);
            long noStopRateLong = Math.round(noStopRate * 100);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getCode(),
                    noStopRateLong + StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getUnit());
            // 一/二/三次及以上停车通过率取最大
            double stopRate = currentDirTurnPOList.stream()
                    .filter(item -> Objects.equals(item.getTurnType(), turnType))
                    .mapToDouble(CrossTurnDataHistPO::getOneStopRate)
                    .max()
                    .orElse(0.0);
            long stopRateLong = Math.round(stopRate * 100);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.STOP_RATE.getCode(),
                    stopRateLong + StrategyAndMetricsEnum.Metrics.STOP_RATE.getUnit());
            // 停车次数取最大
            double stopTimes = currentDirTurnPOList.stream()
                    .filter(item -> Objects.equals(item.getTurnType(), turnType))
                    .mapToDouble(CrossTurnDataHistPO::getStopTimes)
                    .max()
                    .orElse(0.0);
            long stopTimesLong = Math.round(stopTimes);
            metricsMap.put(StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode(),
                    stopTimesLong + StrategyAndMetricsEnum.Metrics.STOP_TIMES.getUnit());

            // 计算指标是否合格
            int level = CrossUtil.calcEfficiencyLevel(maxDelayTime, maxQueueLength);
            if (level == 3) {
                tableContent.setHasProblem(1);
                problemList.add(BaseEnum.SignalDirectionEnum.getNameByCode(dirCode) + "进口"
                        + TurnConvertEnum.getDescByCode(turnType) + "延误较高效率较低");
            } else if (level == 4) {
                tableContent.setHasProblem(1);
                problemList.add(BaseEnum.SignalDirectionEnum.getNameByCode(dirCode) + "进口"
                        + TurnConvertEnum.getDescByCode(turnType) + "延误高效率低");
            }

            tableContent.setMetricsMap(metricsMap);
            tableContentList.add(tableContent);

        }
        return tableContentList;
    }

    private void buildSecurityVO(List<CrossDataHistPO> crossDataHistPOList,
                                 List<CrossDirDataHistPO> crossDirDataHistPOList,
                                 SchemeEvaluateSchemeDetailOverallVO vo, String crossId,
                                 int startTimeStamp, int endTimeStamp) {
        List<SchemeEvaluateSchemeDetailOverallVO.OverallMetrics> overallMetricsList = new ArrayList<>();
        // 溢流率取最大
        OptionalDouble effusionMax = crossDirDataHistPOList.stream()
                .mapToDouble(CrossDirDataHistPO::getEffusionRate)
                .max();
        if (effusionMax.isPresent()) {
            int intMax = (int) Math.round(effusionMax.getAsDouble() * 100);
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildEffusionOverallMetrics(intMax);
            overallMetricsList.add(overallMetrics);
        } else {
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildEffusionOverallMetrics(0);
            overallMetricsList.add(overallMetrics);
//            throw new RuntimeException("该时段无溢流率数据");
        }
        // 绿灯间隔清空率取最大
        OptionalDouble clearRateMax = crossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getClearRate)
                .max();
        if (clearRateMax.isPresent()) {
            int intMax = (int) Math.round(clearRateMax.getAsDouble());
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildClearRateOverallMetrics(intMax);
            overallMetricsList.add(overallMetrics);
        } else {
//            throw new RuntimeException("该时段无绿灯间隔清空率数据");
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildClearRateOverallMetrics(0);
            overallMetricsList.add(overallMetrics);
        }
        // 路口级别三急一速数量
        Integer emergencyCount = holoEventMapper.selectCrossEmergencyCount(crossId, startTimeStamp, endTimeStamp);
        SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                buildEmergencyCountOverallMetrics(emergencyCount);
        overallMetricsList.add(overallMetrics);
        vo.setOverallMetricsList(overallMetricsList);
        // 查询路口指标数值（都取最大），并与评价标准比较：如果饱和度<0.8，则需绿灯间隔清空率大于70%；如果饱和度>0.8，则需绿灯间隔清空率大于60%
        // 达到指标要求，显示路口满足安全保障策略需求；未达到指标，显示路口存在安全隐患
        List<String> strategyEvaluateList = new ArrayList<>();
        boolean isQualified = false;
        Double maxSaturation = crossDataHistMapper.selectMaxSaturation(crossId, startTimeStamp, endTimeStamp);
        if (clearRateMax.isPresent()) {
            isQualified = calcSecurity(maxSaturation, clearRateMax);
        }
        if (!isQualified) {
            strategyEvaluateList.add("路口存在安全隐患");
        }
        if (CollectionUtil.isEmpty(strategyEvaluateList)) {
            strategyEvaluateList.add("路口满足安全保障策略需求");
        }
        vo.setStrategyEvaluateList(strategyEvaluateList);
    }

    private boolean calcSecurity(Double maxSaturation, OptionalDouble clearRateMax) {
        double clearRateMaxDouble = clearRateMax.getAsDouble();
        // 如果饱和度<0.8，则需绿灯间隔清空率大于70%；如果饱和度>0.8，则需绿灯间隔清空率大于60%
        if (maxSaturation < 0.8 && clearRateMaxDouble > 0.7) {
            return true;
        } else if (maxSaturation >= 0.8 && clearRateMaxDouble > 0.6) {
            return true;
        }
        return false;
    }

    private SchemeEvaluateSchemeDetailOverallVO.OverallMetrics buildEmergencyCountOverallMetrics(
            Integer emergencyCount) {
        SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                new SchemeEvaluateSchemeDetailOverallVO.OverallMetrics();
        overallMetrics.setMetricCode(StrategyAndMetricsEnum.Metrics.EMERGENCY_COUNT.getCode());
        overallMetrics.setMetricName(StrategyAndMetricsEnum.Metrics.EMERGENCY_COUNT.getDescription());
        overallMetrics.setMetricValue(emergencyCount);
        overallMetrics.setMetricUnit(StrategyAndMetricsEnum.Metrics.EMERGENCY_COUNT.getUnit());
        return overallMetrics;
    }

    private void buildBalanceVO(List<CrossDataHistPO> crossDataHistPOList,
                                List<CrossDirDataHistPO> crossDirDataHistPOList,
                                SchemeEvaluateSchemeDetailOverallVO vo, String crossId,
                                int startTimeStamp, int endTimeStamp) {
        List<SchemeEvaluateSchemeDetailOverallVO.OverallMetrics> overallMetricsList = new ArrayList<>();
        // 绿灯有效利用率取最大
        OptionalDouble greenLightEfficiencyMax = crossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getGreenLightEfficiency)
                .filter(Objects::nonNull)
                .max();
        if (greenLightEfficiencyMax.isPresent()) {
            int intGreenLightEfficiencyMax = (int) Math.round(greenLightEfficiencyMax.getAsDouble() * 100);
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildGreenLightUseOverallMetrics(intGreenLightEfficiencyMax);
            overallMetricsList.add(overallMetrics);
        } else {
//            throw new RuntimeException("该时段无绿灯有效利用率数据");
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildGreenLightUseOverallMetrics(0);
            overallMetricsList.add(overallMetrics);
        }
        // 饱和度取最大
        OptionalDouble saturationMax = crossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getSturation)
                .filter(Objects::nonNull)
                .max();
        if (saturationMax.isPresent()) {
            int intMax = (int) Math.round(saturationMax.getAsDouble());
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildSaturationMaxOverallMetrics(intMax);
            overallMetricsList.add(overallMetrics);
        } else {
//            throw new RuntimeException("该时段无饱和度数据");
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildSaturationMaxOverallMetrics(0);
            overallMetricsList.add(overallMetrics);
        }
        // 负载均衡度取最大
        OptionalDouble loadBalanceMax = crossDataHistPOList.stream()
                .map(CrossDataHistPO::getLoadBalance)
                .filter(Objects::nonNull)
                .mapToDouble(Double::doubleValue)  // 将Double转换为double
                .max();
        if (loadBalanceMax.isPresent()) {
            int intMax = (int) Math.round(loadBalanceMax.getAsDouble());
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildLoadBalanceOverallMetrics(intMax);
            overallMetricsList.add(overallMetrics);
        } else {
//            throw new RuntimeException("该时段无负载均衡度数据");
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildLoadBalanceOverallMetrics(0);
            overallMetricsList.add(overallMetrics);
        }
        vo.setOverallMetricsList(overallMetricsList);
        // 查询方向指标数值（都取最大），并与评价标准比较：如果饱和度<0.8，则需绿灯利用率大于50%；如果饱和度>0.8，则需绿灯利用率大于80%
        // 未达到指标，显示xx方向失衡；达到指标要求，显示路口满足均衡调控策略需求
        // 获取路口方向
        List<Integer> dirCodes = baseCrossDirInfoMapper.selectInDirsByCrossId(crossId);
        List<String> strategyEvaluateList = new ArrayList<>();
        for (Integer dirCode : dirCodes) {
            Double maxSaturation = crossDirDataHistMapper.selectMaxSaturation(
                    crossId, dirCode, CrossInOutEnum.IN.getCode(), startTimeStamp, endTimeStamp);
            Double maxGreenLightUse = crossDirDataHistMapper.selectMaxGreenLightEfficiency(
                    crossId, dirCode, CrossInOutEnum.IN.getCode(), startTimeStamp, endTimeStamp);
            boolean isQualified = calcBalance(maxSaturation, maxGreenLightUse);
            if (!isQualified) {
                strategyEvaluateList.add(BaseEnum.SignalDirectionEnum.getNameByCode(dirCode) + "进口方向失衡");
            }
        }
        if (CollectionUtil.isEmpty(strategyEvaluateList)) {
            strategyEvaluateList.add("路口满足均衡调控策略需求");
        }
        vo.setStrategyEvaluateList(strategyEvaluateList);
    }

    private SchemeEvaluateSchemeDetailOverallVO.OverallMetrics buildClearRateOverallMetrics(int intMax) {
        SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                new SchemeEvaluateSchemeDetailOverallVO.OverallMetrics();
        overallMetrics.setMetricCode(StrategyAndMetricsEnum.Metrics.CLEAR_RATE.getCode());
        overallMetrics.setMetricName(StrategyAndMetricsEnum.Metrics.CLEAR_RATE.getDescription());
        overallMetrics.setMetricValue(intMax);
        overallMetrics.setMetricUnit(StrategyAndMetricsEnum.Metrics.CLEAR_RATE.getUnit());
        return overallMetrics;
    }

    private SchemeEvaluateSchemeDetailOverallVO.OverallMetrics buildEffusionOverallMetrics(int intMax) {
        SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                new SchemeEvaluateSchemeDetailOverallVO.OverallMetrics();
        overallMetrics.setMetricCode(StrategyAndMetricsEnum.Metrics.EFFUSION_RATE.getCode());
        overallMetrics.setMetricName(StrategyAndMetricsEnum.Metrics.EFFUSION_RATE.getDescription());
        overallMetrics.setMetricValue(intMax);
        overallMetrics.setMetricUnit(StrategyAndMetricsEnum.Metrics.EFFUSION_RATE.getUnit());
        return overallMetrics;
    }

    private boolean calcBalance(Double maxSaturation, Double greenLightEfficiencyMax) {
        // 如果饱和度<0.8，则需绿灯利用率大于50%；如果饱和度>0.8，则需绿灯利用率大于80%
        if (Objects.nonNull(maxSaturation) && Objects.nonNull(greenLightEfficiencyMax)) {
            if (maxSaturation < 0.8 && greenLightEfficiencyMax > 0.5) {
                return true;
            } else if (maxSaturation >= 0.8 && greenLightEfficiencyMax > 0.8) {
                return true;
            }
        }
        return false;
    }

    private SchemeEvaluateSchemeDetailOverallVO.OverallMetrics buildLoadBalanceOverallMetrics(
            int intMax) {
        SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                new SchemeEvaluateSchemeDetailOverallVO.OverallMetrics();
        overallMetrics.setMetricCode(StrategyAndMetricsEnum.Metrics.LOAD_BALANCE.getCode());
        overallMetrics.setMetricName(StrategyAndMetricsEnum.Metrics.LOAD_BALANCE.getDescription());
        overallMetrics.setMetricValue(intMax);
        overallMetrics.setMetricUnit(StrategyAndMetricsEnum.Metrics.LOAD_BALANCE.getUnit());
        return overallMetrics;
    }

    private SchemeEvaluateSchemeDetailOverallVO.OverallMetrics buildSaturationMaxOverallMetrics(
            int intMax) {
        SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                new SchemeEvaluateSchemeDetailOverallVO.OverallMetrics();
        overallMetrics.setMetricCode(StrategyAndMetricsEnum.Metrics.SATURATION.getCode());
        overallMetrics.setMetricName(StrategyAndMetricsEnum.Metrics.SATURATION.getDescription());
        overallMetrics.setMetricValue(intMax);
        overallMetrics.setMetricUnit(StrategyAndMetricsEnum.Metrics.SATURATION.getUnit());
        return overallMetrics;
    }

    private SchemeEvaluateSchemeDetailOverallVO.OverallMetrics buildGreenLightUseOverallMetrics(
            int intGreenLightEfficiencyMax) {
        SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                new SchemeEvaluateSchemeDetailOverallVO.OverallMetrics();
        overallMetrics.setMetricCode(StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getCode());
        overallMetrics.setMetricName(StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getDescription());
        overallMetrics.setMetricValue(intGreenLightEfficiencyMax);
        overallMetrics.setMetricUnit(StrategyAndMetricsEnum.Metrics.GREEN_LIGHT_EFFICIENCY.getUnit());
        return overallMetrics;
    }

    private void buildEfficiencyVO(List<CrossDataHistPO> crossDataHistPOList,
                                   List<CrossDirDataHistPO> crossDirDataHistPOList,
                                   SchemeEvaluateSchemeDetailOverallVO vo, String crossId,
                                   int startTimeStamp, int endTimeStamp) {
        List<SchemeEvaluateSchemeDetailOverallVO.OverallMetrics> overallMetricsList = new ArrayList<>();
        // 排队长度取最大
        OptionalDouble queueLengthMax = crossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getQueueLength)
                .max();
        if (queueLengthMax.isPresent()) {
            // 四舍五入并转换为 int 类型
            int intQueueLengthAverage = (int) Math.round(queueLengthMax.getAsDouble());
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildQueueLengthOverallMetrics(intQueueLengthAverage);
            overallMetricsList.add(overallMetrics);
        } else {
//            throw new RuntimeException("该时段无最大排队数据");
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildQueueLengthOverallMetrics(0);
            overallMetricsList.add(overallMetrics);
        }
        // 延误取最大
        OptionalInt delayTimeMax = crossDataHistPOList.stream()
                .mapToInt(CrossDataHistPO::getDelayTime)
                .max();
        if (delayTimeMax.isPresent()) {
            int intDelayTimeAverage = delayTimeMax.getAsInt();
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildDelayTimeOverallMetrics(intDelayTimeAverage);
            overallMetricsList.add(overallMetrics);
        } else {
//            throw new RuntimeException("该时段无延误数据");
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildDelayTimeOverallMetrics(0);
            overallMetricsList.add(overallMetrics);
        }
        // 停车次数取最大
        OptionalDouble stopTimesMax = crossDataHistPOList.stream()
                .mapToDouble(CrossDataHistPO::getStopTimes)
                .max();
        if (stopTimesMax.isPresent()) {
            // 四舍五入并转换为 int 类型
            int intStopTimesAverage = (int) Math.round(stopTimesMax.getAsDouble());
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildStopTimesOverallMetrics(intStopTimesAverage);
            overallMetricsList.add(overallMetrics);
        } else {
//            throw new RuntimeException("该时段无停车次数数据");
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildStopTimesOverallMetrics(0);
            overallMetricsList.add(overallMetrics);
        }

        // 不停车通过率取最大
        OptionalDouble noStopRateMax = crossDirDataHistPOList.stream()
                .mapToDouble(CrossDirDataHistPO::getNoStopRate)
                .max();
        if (noStopRateMax.isPresent()) {
            // 先乘以100，再四舍五入并转换为 int 类型
            int intStopTimesAverage = (int) Math.round(noStopRateMax.getAsDouble() * 100);
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildNoStopRateOverallMetrics(intStopTimesAverage);
            overallMetricsList.add(overallMetrics);
        } else {
//            throw new RuntimeException("该时段无不停车通过率数据");
            SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                    buildNoStopRateOverallMetrics(0);
            overallMetricsList.add(overallMetrics);
        }
        vo.setOverallMetricsList(overallMetricsList);
        // 查询方向指标数值（都取最大），并与评价标准比较
        // 获取路口方向
        List<Integer> dirCodes = baseCrossDirInfoMapper.selectInDirsByCrossId(crossId);
        List<String> strategyEvaluateList = new ArrayList<>();
        for (Integer dirCode : dirCodes) {
            Integer maxDelayTime = crossDirDataHistMapper.selectMaxDelayTime(
                    crossId, dirCode, CrossInOutEnum.IN.getCode(), startTimeStamp, endTimeStamp);
            Double maxQueueLength = crossDirDataHistMapper.selectMaxQueueLength(
                    crossId, dirCode, CrossInOutEnum.IN.getCode(), startTimeStamp, endTimeStamp);
            int level = CrossUtil.calcEfficiencyLevel(maxDelayTime, maxQueueLength);
            if (level == 3) {
                strategyEvaluateList.add(BaseEnum.SignalDirectionEnum.getNameByCode(dirCode) + "进口延误较高效率较低");
            } else if (level == 4) {
                strategyEvaluateList.add(BaseEnum.SignalDirectionEnum.getNameByCode(dirCode) + "进口延误高效率低");
            }
        }
        if (CollectionUtil.isEmpty(strategyEvaluateList)) {
            strategyEvaluateList.add("路口满足效率提升策略需求");
        }
        vo.setStrategyEvaluateList(strategyEvaluateList);
    }

    private SchemeEvaluateSchemeDetailOverallVO.OverallMetrics buildNoStopRateOverallMetrics(
            int intStopTimesAverage) {
        SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                new SchemeEvaluateSchemeDetailOverallVO.OverallMetrics();
        overallMetrics.setMetricCode(StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getCode());
        overallMetrics.setMetricName(StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getDescription());
        overallMetrics.setMetricValue(intStopTimesAverage);
        overallMetrics.setMetricUnit(StrategyAndMetricsEnum.Metrics.NO_STOP_RATE.getUnit());
        return overallMetrics;
    }

    private SchemeEvaluateSchemeDetailOverallVO.OverallMetrics buildStopTimesOverallMetrics(
            int intStopTimesAverage) {
        SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                new SchemeEvaluateSchemeDetailOverallVO.OverallMetrics();
        overallMetrics.setMetricCode(StrategyAndMetricsEnum.Metrics.STOP_TIMES.getCode());
        overallMetrics.setMetricName(StrategyAndMetricsEnum.Metrics.STOP_TIMES.getDescription());
        overallMetrics.setMetricValue(intStopTimesAverage);
        overallMetrics.setMetricUnit(StrategyAndMetricsEnum.Metrics.STOP_TIMES.getUnit());
        return overallMetrics;
    }

    private SchemeEvaluateSchemeDetailOverallVO.OverallMetrics buildDelayTimeOverallMetrics(
            int intDelayTimeAverage) {
        SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                new SchemeEvaluateSchemeDetailOverallVO.OverallMetrics();
        overallMetrics.setMetricCode(StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getCode());
        overallMetrics.setMetricName(StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getDescription());
        overallMetrics.setMetricValue(intDelayTimeAverage);
        overallMetrics.setMetricUnit(StrategyAndMetricsEnum.Metrics.AVERAGE_DELAY.getUnit());
        return overallMetrics;
    }

    @NotNull
    private static SchemeEvaluateSchemeDetailOverallVO.OverallMetrics buildQueueLengthOverallMetrics(
            int intQueueLengthAverage) {
        SchemeEvaluateSchemeDetailOverallVO.OverallMetrics overallMetrics =
                new SchemeEvaluateSchemeDetailOverallVO.OverallMetrics();
        overallMetrics.setMetricCode(StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getCode());
        overallMetrics.setMetricName(StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getDescription());
        overallMetrics.setMetricValue(intQueueLengthAverage);
        overallMetrics.setMetricUnit(StrategyAndMetricsEnum.Metrics.MAX_QUEUE_LENGTH.getUnit());
        return overallMetrics;
    }

    @NotNull
    private static SchemeEvaluateCrossSchemeListVO buildSchemeEvaluateCrossSchemeListVO
            (BaseCrossSchemePO baseCrossSchemePO, CrossSectionPO crossSectionPO) {
        SchemeEvaluateCrossSchemeListVO vo = new SchemeEvaluateCrossSchemeListVO();
        vo.setSchemeName(baseCrossSchemePO.getName());
        vo.setSchemeId(baseCrossSchemePO.getId());
        vo.setStartTime(crossSectionPO.getStartTime());
        vo.setEndTime(crossSectionPO.getEndTime());
        vo.setSectionId(crossSectionPO.getId());
        vo.setSchemeNo(baseCrossSchemePO.getSchemeNo());
        vo.setCycle(baseCrossSchemePO.getCycle());
        vo.setNote("");
        return vo;
    }

    private boolean isNumeric(String str) {
        return str != null && str.matches("[0-9]+");
    }


    private List<SchemeEvaluateProblemSchemeVO.BySchemeVO> buildProblemSchemeListByScheme(
            Map<Date, List<RunningEvaluateMetricsDetailVO.ProblemScheme>> dateSchemesMap) {
        // 使用一个Map来临时存储按方案名称分组的数据
        Map<String, List<SchemeEvaluateProblemSchemeVO.SchemeDatetime>> tempMap = new HashMap<>();

        for (Map.Entry<Date, List<RunningEvaluateMetricsDetailVO.ProblemScheme>> entry : dateSchemesMap.entrySet()) {
            Date date = entry.getKey();
            List<RunningEvaluateMetricsDetailVO.ProblemScheme> problemSchemes = entry.getValue();

            for (RunningEvaluateMetricsDetailVO.ProblemScheme problemScheme : problemSchemes) {
                Integer schemeId = problemScheme.getSchemeId();
                String schemeName = problemScheme.getSchemeName();
                String schemeIdName = schemeId + "_" + schemeName;
                SchemeEvaluateProblemSchemeVO.SchemeDatetime schemeDatetime =
                        new SchemeEvaluateProblemSchemeVO.SchemeDatetime();
                schemeDatetime.setYear(date);
                schemeDatetime.setWeek(DateUtil.dayOfWeek(date));
                schemeDatetime.setProblemDate(date);
                schemeDatetime.setStartTime(problemScheme.getStartTime());
                schemeDatetime.setEndTime(problemScheme.getEndTime());

                tempMap.computeIfAbsent(schemeIdName, k -> new ArrayList<>()).add(schemeDatetime);
            }
        }
        // 构建最终的 List<BySchemeVO>
        List<SchemeEvaluateProblemSchemeVO.BySchemeVO> bySchemeVOList = tempMap.entrySet().stream()
                .map(entry -> {
                    SchemeEvaluateProblemSchemeVO.BySchemeVO bySchemeVO = new SchemeEvaluateProblemSchemeVO.BySchemeVO();
                    String schemeIdName = entry.getKey();
                    String[] s = schemeIdName.split("_");
                    bySchemeVO.setSchemeId(Integer.parseInt(s[0]));
                    bySchemeVO.setSchemeName(s[1]);
                    bySchemeVO.setSchemeDatetimeList(entry.getValue());
                    return bySchemeVO;
                })
                .collect(Collectors.toList());
        return bySchemeVOList;
    }

    private int calcProblemCounts(List<SchemeEvaluateProblemSchemeVO.ByDateVO> byDateVOList,
                                  Function<SchemeEvaluateProblemSchemeVO.ByDateVO, Integer> getCount) {
        return byDateVOList.stream().mapToInt(getCount::apply).sum();
    }

    private int calcSchemeCounts(Map<Date, List<RunningEvaluateMetricsDetailVO.ProblemScheme>> dateSchemesMap) {
        Set<Integer> problemIdSet = new HashSet<>();
        Collection<List<RunningEvaluateMetricsDetailVO.ProblemScheme>> values = dateSchemesMap.values();

        for (List<RunningEvaluateMetricsDetailVO.ProblemScheme> list : values) {
            for (RunningEvaluateMetricsDetailVO.ProblemScheme problemScheme : list) {
                Integer schemeId = problemScheme.getSchemeId();
                problemIdSet.add(schemeId);
            }
        }
        return problemIdSet.size();
    }

    @NotNull
    private List<SchemeEvaluateProblemSchemeVO.ByDateVO> buildProblemSchemeListByDate(
            Map<Date, List<CrossDataHistPO>> dateProblemsMap,
            Map<Date, List<RunningEvaluateMetricsDetailVO.ProblemScheme>> dateSchemesMap, List<ScenePO> scenePOList) {

        List<SchemeEvaluateProblemSchemeVO.ByDateVO> byDateVOList = new ArrayList<>();
        List<SceneStrategyPO> sceneStrategyPOList = sceneStrategyMapper.selectAll();
        List<StrategyPO> strategyPOList = strategyMapper.selectAll();

        for (Map.Entry<Date, List<CrossDataHistPO>> entry : dateProblemsMap.entrySet()) {
            Date key = entry.getKey();
            SchemeEvaluateProblemSchemeVO.ByDateVO byDateVO = new SchemeEvaluateProblemSchemeVO.ByDateVO();
            byDateVO.setProblemDate(key);
            List<RunningEvaluateMetricsDetailVO.ProblemScheme> problemSchemeList = dateSchemesMap.get(key);
            byDateVO.setSectionSechemeList(problemSchemeList);
            List<CrossDataHistPO> value = entry.getValue();

            int efficiencyProblemCounts = 0;
            int balanceProblemCounts = 0;
            int securityProblemCounts = 0;
            for (CrossDataHistPO crossDataHistPO : value) {
                Integer status = crossDataHistPO.getStatus();
                // 根据场景查询策略
                Optional<ScenePO> first = scenePOList.stream()
                        .filter(po -> status.equals(po.getSceneNum()))
                        .findFirst();
                ScenePO scenePO = first.get();
                Integer sceneId = scenePO.getId();

                List<SceneStrategyPO> sceneStrategyList = sceneStrategyPOList.stream()
                        .filter(po -> sceneId.equals(po.getSceneId()))
                        .collect(Collectors.toList());
                for (SceneStrategyPO sceneStrategyPO : sceneStrategyList) {
                    Integer strategyId = sceneStrategyPO.getStrategyId();

                    Optional<StrategyPO> first1 = strategyPOList.stream()
                            .filter(po -> strategyId.equals(po.getId()))
                            .findFirst();
                    StrategyPO strategyPO = first1.get();
                    String strategyCode = strategyPO.getStrategyCode();
                    if (strategyCode.equals(StrategyAndMetricsEnum.Strategy.EFFICIENCY.getCode())) {
                        efficiencyProblemCounts += 1;
                    } else if (strategyCode.equals(StrategyAndMetricsEnum.Strategy.BALANCE.getCode())) {
                        balanceProblemCounts += 1;
                    } else if (strategyCode.equals(StrategyAndMetricsEnum.Strategy.SECURITY.getCode())) {
                        securityProblemCounts += 1;
                    }
                }

            }
            byDateVO.setEfficiencyProblemCounts(efficiencyProblemCounts);
            byDateVO.setBalanceProblemCounts(balanceProblemCounts);
            byDateVO.setSecurityProblemCounts(securityProblemCounts);
            byDateVOList.add(byDateVO);
        }
        return byDateVOList;
    }

    private Map<Date, List<RunningEvaluateMetricsDetailVO.ProblemScheme>> buildDateSchemesMap(
            Map<Date, List<CrossDataHistPO>> dateProblemsMap, String crossId, List<CrossSectionPO> crossSectionPOList,
            List<BaseCrossSchemePO> crossSchemePOList) {

        Map<Date, List<RunningEvaluateMetricsDetailVO.ProblemScheme>> dateSchemesMap = new HashMap<>();
        // 获取当前计划ID
        for (Map.Entry<Date, List<CrossDataHistPO>> dateListEntry : dateProblemsMap.entrySet()) {
            List<RunningEvaluateMetricsDetailVO.ProblemScheme> problemSchemeList = new ArrayList<>();
            Date datetime = dateListEntry.getKey();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String dateStr = sdf.format(datetime);
            Integer planId = runningEvaluateService.findPlanId(datetime, dateStr, crossId);

            List<CrossSectionPO> filteredList = crossSectionPOList.stream()
                    .filter(crossSection -> crossId.equals(crossSection.getCrossId()) &&
                            planId.equals(crossSection.getPlanId()))
                    .collect(Collectors.toList());

            for (CrossSectionPO crossSectionPO : filteredList) {
                String sectionStartTime = crossSectionPO.getStartTime();
                String sectionEndTime = crossSectionPO.getEndTime();
                List<CrossDataHistPO> value = dateListEntry.getValue();
                for (CrossDataHistPO crossDataHistPO : value) {
                    SimpleDateFormat sdf2 = new SimpleDateFormat("HH:mm");
                    Date abnormalEventStartTime = crossDataHistPO.getStartTime();
                    Integer duration = crossDataHistPO.getDuration();
                    Date abnormalEventEndTime = DateUtil.offsetMinute(abnormalEventStartTime, duration);
                    String eventStartTime = sdf2.format(abnormalEventStartTime);
                    String eventEndTime = sdf2.format(abnormalEventEndTime);
                    // 比较两组时间段是否相交
                    boolean isIntersecting = runningEvaluateService.isTimeIntersecting(
                            eventStartTime, eventEndTime, sectionStartTime, sectionEndTime);
                    runningEvaluateService.buildResIfIntersecting(problemSchemeList, crossSectionPO, sectionStartTime,
                            sectionEndTime, isIntersecting, crossSchemePOList);
                }
            }
            // 去除相同方案
            List<RunningEvaluateMetricsDetailVO.ProblemScheme> uniqueRes = problemSchemeList.stream()
                    .distinct()
                    .collect(Collectors.toList());
            dateSchemesMap.put(datetime, uniqueRes);
        }
        return dateSchemesMap;
    }

    private Map<Date, List<CrossDataHistPO>> buildDateProblemsMap(Map<String, List<CrossDataHistPO>> timeProblemMap) {
        // 创建一个新的Map，用于存储以Date为Key，List<CrossDataHistPO>为Value的数据
        Map<Date, List<CrossDataHistPO>> dateMap = new HashMap<>();
        // 创建一个SimpleDateFormat对象，用于将10位时间戳转换为Date对象
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        // 遍历timeProblemMap的所有entries
        for (List<CrossDataHistPO> list : timeProblemMap.values()) {
            for (CrossDataHistPO po : list) {
                long timestamp = po.getBatchTime() * 1000L; // 将10位时间戳转换为13位
                Date date = new Date(timestamp);
                String formattedDate = sdf.format(date); // 格式化日期为"yyyy-MM-dd"
                try {
                    Date keyDate = sdf.parse(formattedDate); // 将格式化后的日期字符串转回为Date对象
                    // 将CrossDataHistPO对象添加到相应日期的List中
                    dateMap.computeIfAbsent(keyDate, k -> new ArrayList<>()).add(po);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        return dateMap;
    }

    @NotNull
    private static Map<String, List<CrossDataHistPO>> getTimeProblemMap(
            List<CrossDataHistPO> crossDataHistPOList, String endTime, String startTime) {
        // 过滤有问题的记录
        List<CrossDataHistPO> filteredList = crossDataHistPOList.stream()
                .filter(po -> po.getStatus() != null && po.getStatus() != 0)
                .collect(Collectors.toList());
        // 保留 duration 最大的一条
        List<CrossDataHistPO> result = filteredList.stream()
                .collect(Collectors.groupingBy(CrossDataHistPO::getStartTime))
                .values().stream()
                .map(list -> list.stream().max(Comparator.comparingInt(CrossDataHistPO::getDuration)).orElse(null))
                .collect(Collectors.toList());
        // 按照 startTime 和 endTime 的 "HH:mm" 的时间区间分组
        Map<String, List<CrossDataHistPO>> groupedByTimeInterval = result.stream()
                .collect(Collectors.groupingBy(po -> {
                    Date startTime1 = po.getStartTime();
                    Date endTime1 = new Date(startTime1.getTime() + po.getDuration() * 60 * 1000);
                    SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
                    return sdf.format(startTime1) + " - " + sdf.format(endTime1);
                }));
        // 过滤 groupedByTimeInterval，保留与 startTime 和 endTime 有交叉的条目
        Map<String, List<CrossDataHistPO>> filteredGroupedByTimeInterval = groupedByTimeInterval.entrySet().stream()
                .filter(entry -> {
                    String[] times = entry.getKey().split(" - ");
                    String groupStartTime = times[0];
                    String groupEndTime = times[1];
                    return (groupStartTime.compareTo(endTime) < 0 || groupStartTime.equals(endTime))
                            && (groupEndTime.compareTo(startTime) > 0 || groupEndTime.equals(startTime));
                })
                .collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
        return filteredGroupedByTimeInterval;
    }
}
