package net.wanji.opt.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.util.DateUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.*;
import net.wanji.common.framework.Constants;
import net.wanji.common.utils.tool.CrossUtil;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.dto.MetricHistDTO;
import net.wanji.databus.po.*;
import net.wanji.databus.vo.AbnormalCrossListVO;
import net.wanji.databus.vo.GreenwaveListVO;
import net.wanji.opt.bo.*;
import net.wanji.opt.cache.BaseCrossInfoCache;
import net.wanji.opt.common.ExcelExportUtils;
import net.wanji.opt.common.KafkaConsumerUtil;
import net.wanji.opt.common.RedisUtils;
import net.wanji.opt.common.exception.OptServiceException;
import net.wanji.opt.config.DirectionMappingsConfig;
import net.wanji.opt.dao.mapper.CrossSchemeOptLogMapper;
import net.wanji.opt.dao.mapper.HoloEventMapper;
import net.wanji.opt.dao.mapper.strategy.SceneMapper;
import net.wanji.opt.dao.mapper.strategy.SceneStrategyIdeaMapper;
import net.wanji.opt.dao.mapper.strategy.SceneStrategyMapper;
import net.wanji.opt.dao.mapper.strategy.StrategyMapper;
import net.wanji.opt.dao.mapper.trend.AnalysisRidTurnIndicatorsMapper;
import net.wanji.opt.dao.mapper.trend.EventAlarmMapper;
import net.wanji.opt.dto.CrossEventDTO;
import net.wanji.opt.dto.CrossLaneSnapshotDataDTO;
import net.wanji.opt.dto.LineCongestion;
import net.wanji.opt.dto.LineSchemeDTO;
import net.wanji.opt.dto.trend.AbnormalCrossListDTO;
import net.wanji.opt.dto.trend.GreenwaveListDTO;
import net.wanji.opt.po.base.CrossSchemeOptLogPO;
import net.wanji.opt.po.strategy.ScenePO;
import net.wanji.opt.po.strategy.StrategyPO;
import net.wanji.opt.po.trend.AnalysisRidTurnIndicators;
import net.wanji.opt.po.trend.EventAlarmPO;
import net.wanji.opt.po.trend.HoloEventInfoPO;
import net.wanji.opt.service.TrendService;
import net.wanji.opt.service.es.LaneSnapshotDataQueryService;
import net.wanji.opt.vo.*;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.function.Function;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author Kent HAN
 * @date 2023/2/8 10:31
 */
@Service
@Slf4j
public class TrendServiceImpl implements TrendService {
    private final GreenwaveInfoMapper greenwaveInfoMapper;
    private final BaseCrossInfoMapper baseCrossInfoMapper;
    private final CrossDataRealtimeMapper crossDataRealtimeMapper;
    private final CrossTurnDataRealtimeMapper crossTurnDataRealtimeMapper;
    private final CrossDirDataHistMapper crossDirDataHistMapper;
    private final EventAlarmMapper eventAlarmMapper;
    private final RidInfoMapper ridInfoMapper;
    private final BaseCrossDirInfoMapper baseCrossDirInfoMapper;
    private final CrossSchemeOptLogMapper crossSchemeOptLogMapper;
    private final GreenwaveCrossMapper greenwaveCrossMapper;
    private final CrossDirDataRealtimeMapper crossDirDataRealtimeMapper;
    private final GreenwaveHistMapper greenwaveHistMapper;
    private final GreenwaveRealtimeMapper greenwaveRealtimeMapper;
    private final StrategyMapper strategyMapper;
    private final BaseCrossSchedulesMapper baseCrossSchedulesMapper;
    private final BaseCrossSchedulesPlanMapper baseCrossSchedulesPlanMapper;
    private final BaseCrossSectionMapper baseCrossSectionMapper;
    private final BaseCrossSchemeMapper baseCrossSchemeMapper;
    private final BaseCrossPhaseMapper baseCrossPhaseMapper;
    private final BaseCrossPhaseLightsMapper crossPhaseLightsMapper;
    private final BaseCrossLaneLightsMapper baseCrossLaneLightsMapper;
    private final CrossBaseLaneInfoMapper crossBaseLaneInfoMapper;
    private final CrossTurnDataHistMapper crossTurnDataHistMapper;
    private final GreenwaveSceneMapper greenwaveSceneMapper;
    private final SceneStrategyMapper sceneStrategyMapper;
    private final SceneMapper sceneMapper;
    private final BaseCrossPlanMapper baseCrossPlanMapper;
    private final MainlineEvaluateServiceImpl mainlineEvaluateServiceImpl;
    private final CrossLaneDataHistMapper crossLaneDataHistMapper;
    private final LaneSnapshotDataQueryService laneSnapshotDataQueryService;
    private final DirectionMappingsConfig directionMappingsConfig;
    private final CrossDataHistMapper crossDataHistMapper;
    private final RunningEvaluateServiceImpl runningEvaluateService;
    private final AnalysisRidTurnIndicatorsMapper ridTurnIndicatorsMapper;
    private final HoloEventMapper holoEventMapper;
    private final BaseCrossInfoCache crossInfoCache;
    @Value("${spring.kafka.bootstrap-servers}")
    private String bootstrapServers;

    // 用于计算路口状态,key为方向，value为状态
    private Map<Integer, Integer> preStatus = new HashMap<Integer, Integer>() {{
        put(1, 1);
        put(2, 1);
        put(3, 1);
        put(4, 1);
        put(5, 1);
        put(6, 1);
        put(7, 1);
        put(8, 1);
    }};

    // 表格实时推送开始时间
    private Date tableRealTimeFirstDate;
    // 表格周期推送开始时间
    private Date tableCycleFirstDate;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Autowired
    RedisUtils redisUtils;

    public TrendServiceImpl(GreenwaveInfoMapper greenwaveInfoMapper, BaseCrossInfoMapper baseCrossInfoMapper,
                            CrossDataRealtimeMapper crossDataRealtimeMapper,
                            CrossTurnDataRealtimeMapper crossTurnDataRealtimeMapper,
                            CrossDirDataHistMapper crossDirDataHistMapper, EventAlarmMapper eventAlarmMapper,
                            RidInfoMapper ridInfoMapper, BaseCrossDirInfoMapper baseCrossDirInfoMapper,
                            CrossSchemeOptLogMapper crossSchemeOptLogMapper, GreenwaveCrossMapper greenwaveCrossMapper, CrossDirDataRealtimeMapper crossDirDataRealtimeMapper, GreenwaveHistMapper greenwaveHistMapper, GreenwaveRealtimeMapper greenwaveRealtimeMapper, SceneStrategyIdeaMapper strategyIdeaMapper, StrategyMapper strategyMapper, BaseCrossSchedulesMapper baseCrossSchedulesMapper, BaseCrossSchedulesPlanMapper baseCrossSchedulesPlanMapper, BaseCrossSectionMapper baseCrossSectionMapper, BaseCrossSchemeMapper crossSchemeMapper, BaseCrossSchemeMapper baseCrossSchemeMapper, BaseCrossPhaseMapper baseCrossPhaseMapper, BaseCrossPhaseLightsMapper crossPhaseLightsMapper, BaseCrossLaneLightsMapper baseCrossLaneLightsMapper, CrossBaseLaneInfoMapper crossBaseLaneInfoMapper, CrossTurnDataHistMapper crossTurnDataHistMapper, @Qualifier("greenwaveSceneMapper") GreenwaveSceneMapper greenwaveSceneMapper, @Qualifier("sceneStrategyMapper") SceneStrategyMapper sceneStrategyMapper, @Qualifier("sceneMapper") SceneMapper sceneMapper, @Qualifier("baseCrossPlanMapper") BaseCrossPlanMapper baseCrossPlanMapper, MainlineEvaluateServiceImpl mainlineEvaluateServiceImpl, CrossLaneDataHistMapper crossLaneDataHistMapper, LaneSnapshotDataQueryService laneSnapshotDataQueryService, DirectionMappingsConfig directionMappingsConfig,
                            CrossDataHistMapper crossDataHistMapper, RunningEvaluateServiceImpl runningEvaluateService,
                            AnalysisRidTurnIndicatorsMapper ridTurnIndicatorsMapper, HoloEventMapper holoEventMapper,
                            BaseCrossInfoCache crossInfoCache) {
        this.greenwaveInfoMapper = greenwaveInfoMapper;
        this.baseCrossInfoMapper = baseCrossInfoMapper;
        this.crossDataRealtimeMapper = crossDataRealtimeMapper;
        this.crossTurnDataRealtimeMapper = crossTurnDataRealtimeMapper;
        this.crossDirDataHistMapper = crossDirDataHistMapper;
        this.eventAlarmMapper = eventAlarmMapper;
        this.ridInfoMapper = ridInfoMapper;
        this.baseCrossDirInfoMapper = baseCrossDirInfoMapper;
        this.crossSchemeOptLogMapper = crossSchemeOptLogMapper;
        this.greenwaveCrossMapper = greenwaveCrossMapper;
        this.crossDirDataRealtimeMapper = crossDirDataRealtimeMapper;
        this.greenwaveHistMapper = greenwaveHistMapper;
        this.greenwaveRealtimeMapper = greenwaveRealtimeMapper;
        this.strategyMapper = strategyMapper;
        this.baseCrossSchedulesMapper = baseCrossSchedulesMapper;
        this.baseCrossSchedulesPlanMapper = baseCrossSchedulesPlanMapper;
        this.baseCrossSectionMapper = baseCrossSectionMapper;
        this.baseCrossSchemeMapper = baseCrossSchemeMapper;
        this.baseCrossPhaseMapper = baseCrossPhaseMapper;
        this.crossPhaseLightsMapper = crossPhaseLightsMapper;
        this.baseCrossLaneLightsMapper = baseCrossLaneLightsMapper;
        this.crossBaseLaneInfoMapper = crossBaseLaneInfoMapper;
        this.crossTurnDataHistMapper = crossTurnDataHistMapper;
        this.greenwaveSceneMapper = greenwaveSceneMapper;
        this.sceneStrategyMapper = sceneStrategyMapper;
        this.sceneMapper = sceneMapper;
        this.baseCrossPlanMapper = baseCrossPlanMapper;
        this.mainlineEvaluateServiceImpl = mainlineEvaluateServiceImpl;
        this.crossLaneDataHistMapper = crossLaneDataHistMapper;
        this.laneSnapshotDataQueryService = laneSnapshotDataQueryService;
        this.directionMappingsConfig = directionMappingsConfig;
        this.crossDataHistMapper = crossDataHistMapper;
        this.runningEvaluateService = runningEvaluateService;
        this.ridTurnIndicatorsMapper = ridTurnIndicatorsMapper;
        this.holoEventMapper = holoEventMapper;
        this.crossInfoCache = crossInfoCache;
    }

    @Override
    public List<GreenwaveListVO> greenwaveList(GreenwaveListDTO greenwaveListDTO) {
        Integer status = greenwaveListDTO.getStatus();
        String name = greenwaveListDTO.getName();
        Integer type = greenwaveListDTO.getType();
        List<GreenwaveListVO> greenwaveListVOList = greenwaveInfoMapper.listGreenwave(status, name, type);
        // 3、4都算拥堵
        if (status != null && status == 3) {
            List<GreenwaveListVO> extraList = greenwaveInfoMapper.listGreenwave(4, name, type);
            greenwaveListVOList.addAll(extraList);
        }
        changeStatus(greenwaveListVOList);
        for (GreenwaveListVO greenwaveListVO : greenwaveListVOList) {
            // 计算持续时间
            buildDuration(greenwaveListVO);
            // 构造路口列表
            Integer greenwaveId = greenwaveListVO.getId();
            List<GreenwaveListVO.CrossListElement> crossList = buildCrossList(greenwaveId);
            greenwaveListVO.setCrossList(crossList);
        }
        return greenwaveListVOList;
    }

    @Override
    public GreenwaveStats buildGreenwaveStats(List<GreenwaveListVO> greenwaveListVOList) {
        GreenwaveStats greenwaveStats = new GreenwaveStats();
        Integer smooth = greenwaveStats.getSmooth();
        Integer slow = greenwaveStats.getSlow();
        Integer congestion = greenwaveStats.getCongestion();
        for (GreenwaveListVO greenwaveListVO : greenwaveListVOList) {
            Integer realtimeStatus = greenwaveListVO.getRealtimeStatus();
            if (realtimeStatus == 1) {
                smooth += 1;
            } else if (realtimeStatus == 2) {
                slow += 1;
            } else if (realtimeStatus == 3) {
                congestion += 1;
            }
        }
        greenwaveStats.setSmooth(smooth);
        greenwaveStats.setSlow(slow);
        greenwaveStats.setCongestion(congestion);
        return greenwaveStats;
    }

    @Override
    public AbnormalCrossVO abnormalCrossList(AbnormalCrossListDTO abnormalCrossListDTO) {
        AbnormalCrossVO abnormalCrossVO = new AbnormalCrossVO();
        // 构建问题路口列表
        Integer status = abnormalCrossListDTO.getStatus();
        String name = abnormalCrossListDTO.getName();
        Integer type = abnormalCrossListDTO.getType();
        List<AbnormalCrossListVO> abnormalCrossListVOList =
                crossDataRealtimeMapper.selectAbnormalCross(status, name, type);
        // 添加优化状态
        for (AbnormalCrossListVO abnormalCrossListVO : abnormalCrossListVOList) {
            abnormalCrossListVO.setOptStatus(2); // 2未优化
            String crossId = abnormalCrossListVO.getId();
            // 查询优化日志
            List<CrossSchemeOptLogPO> crossSchemeOptLogPOS = crossSchemeOptLogMapper.selectMaxByCrossId(crossId);
            if (CollectionUtil.isNotEmpty(crossSchemeOptLogPOS)) {
                CrossSchemeOptLogPO crossSchemeOptLogPO = crossSchemeOptLogPOS.get(0);
                Date endTime = crossSchemeOptLogPO.getEndTime();
                if (ObjectUtil.isEmpty(endTime)) {
                    abnormalCrossListVO.setOptStatus(1); // 1优化中
                }
            }
        }
        // 坐标格式转换
        for (AbnormalCrossListVO abnormalCrossListVO : abnormalCrossListVOList) {
            String locationStr = abnormalCrossListVO.getLocationStr();
            double[] lonLat = CrossUtil.getLonLat(locationStr);
            List<Double> location = new ArrayList<>();
            location.add(lonLat[0]);
            location.add(lonLat[1]);
            abnormalCrossListVO.setLocation(location);
            // 关联路口信息
            Integer realtimeStatus = abnormalCrossListVO.getRealtimeStatus();
            String endCrossId = abnormalCrossListVO.getId();
            if (Objects.equals(realtimeStatus, CrossStatusEnum.SPILLOVER.getCode())) {
                // 溢出只返回溢出方向的关联路口
                String spilloverDirs = abnormalCrossListVO.getSpilloverDirs();
                String[] split = spilloverDirs.split(",");
                List<Integer> spilloverDirList = new ArrayList<>();
                for (String s : split) {
                    spilloverDirList.add(Integer.parseInt(s));
                }
                buildRidData(abnormalCrossListVO, endCrossId, spilloverDirList);
            } else if (Objects.equals(realtimeStatus, CrossStatusEnum.CONGESTION.getCode())) {
                // 拥堵返回所有方向关联路口
                List<Integer> inDirs = baseCrossDirInfoMapper.selectInDirsByCrossId(endCrossId);
                buildRidData(abnormalCrossListVO, endCrossId, inDirs);
            }
        }
        CrossComparator crossComparator = new CrossComparator();
        abnormalCrossListVOList.sort(crossComparator);

        // 设置拥堵指数同比 环比
        for (AbnormalCrossListVO abnormalCrossListVO : abnormalCrossListVOList) {
            String crossId = abnormalCrossListVO.getId();
            Integer batchTime = abnormalCrossListVO.getBatchTime();

            double roundedTrafficIndex = Math.round(abnormalCrossListVO.getCongestionIndex() * 100.0) / 100.0;
            abnormalCrossListVO.setCongestionIndex(roundedTrafficIndex);

            Double congestionIndex = abnormalCrossListVO.getCongestionIndex();
            double lastWeekIndex = getIndex(congestionIndex, crossId, batchTime - 604800);
            double lastPeriodIndex = getIndex(congestionIndex, crossId, batchTime - 300);
            abnormalCrossListVO.setLastWeekIndex(Math.floor(lastWeekIndex));
            abnormalCrossListVO.setLastPeriodIndex(Math.floor(lastPeriodIndex));

            // 常发性偶发性 一个月内超过三次
            int frequent = getFrequent(crossId, batchTime);

            abnormalCrossListVO.setFrequent(frequent);
        }


        abnormalCrossVO.setAbnormalCrossList(abnormalCrossListVOList);
        // 构造统计信息
        List<AbnormalCrossListVO> listForStats =
                crossDataRealtimeMapper.selectAbnormalCross(null, null, null);
        buildAbnormalCrossStats(abnormalCrossVO, listForStats);

        return abnormalCrossVO;
    }

    private int getFrequent(String crossId, Integer batchTime) {
        LambdaQueryWrapper<CrossDataHistPO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(CrossDataHistPO::getCrossId, crossId);
        queryWrapper.between(CrossDataHistPO::getBatchTime, batchTime - 30 * 86400, batchTime);
        queryWrapper.eq(CrossDataHistPO::getIsCongestion, 1);
        List<CrossDataHistPO> crossDataHistPOS = crossDataHistMapper.selectList(queryWrapper);
        List<CrossDataHistPO> result = new ArrayList<>();
        for (CrossDataHistPO crossDataHistPO : crossDataHistPOS) {
            Date startTime = crossDataHistPO.getStartTime();
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("HH:mm");
            LocalTime localTime = startTime.toInstant().atZone(ZoneId.of("Asia/Shanghai")).toLocalTime();
            if ((localTime.isAfter(LocalTime.parse("06:30", formatter))
                    && localTime.isBefore(LocalTime.parse("09:30", formatter))
                    && StringUtils.equalsIgnoreCase(crossId, crossDataHistPO.getCrossId())) ||
                    (localTime.isAfter(LocalTime.parse("16:30", formatter))
                            && localTime.isBefore(LocalTime.parse("19:30", formatter))
                            && StringUtils.equalsIgnoreCase(crossId, crossDataHistPO.getCrossId()))) {
                result.add(crossDataHistPO);
            }
        }
        return result.size() > 3 ? 1 : 0;
    }

    private double getIndex(Double congestionIndex, String crossId, Integer batchTime) {
        LambdaQueryWrapper<CrossDataHistPO> lastWeekQuery = new LambdaQueryWrapper<>();
        lastWeekQuery.eq(CrossDataHistPO::getCrossId, crossId);
        lastWeekQuery.eq(CrossDataHistPO::getBatchTime, batchTime);
        CrossDataHistPO lastHistPO = crossDataHistMapper.selectOne(lastWeekQuery);
        if (Objects.isNull(lastHistPO)) {
            return 0.0;
        }
        Double lastCongestionIndex = lastHistPO.getCongestionIndex();
        return (congestionIndex - lastCongestionIndex) / lastCongestionIndex * 100;
    }

    private static class CrossComparator implements Comparator<AbnormalCrossListVO> {
        @Override
        public int compare(AbnormalCrossListVO o1, AbnormalCrossListVO o2) {
            // 先按路口状态排序
            Integer o1status = o1.getRealtimeStatus();
            Integer o2status = o2.getRealtimeStatus();
            int flag = o2status - o1status;
            if (flag == 0) {
                // 再按拥堵时长排序
                Long o1duration = o1.getDuration();
                Long o2Duration = o2.getDuration();
                long res = o2Duration - o1duration;
                return (int) res;
            }
            return flag;
        }
    }

    private void buildRidData(AbnormalCrossListVO abnormalCrossListVO, String endCrossId,
                              List<Integer> spilloverDirList) {
        List<List<Double>> ridLocationList = new ArrayList<>();
        List<String> ridWktList = new ArrayList<>();
        for (Integer spilloverDir : spilloverDirList) {
            RidInfoEntity ridInfo = ridInfoMapper.selectByEndInDir(endCrossId, spilloverDir);
            if (ridInfo != null) {
                // 关联路口坐标
                String startCrossId = ridInfo.getStartCrossId();
                TBaseCrossInfo crossInfoPO = baseCrossInfoMapper.selectByPrimaryKey(startCrossId);
                if (crossInfoPO != null) {
                    String startLocationStr = crossInfoPO.getLocation();
                    double[] startLonLat = CrossUtil.getLonLat(startLocationStr);
                    List<Double> startLocation = new ArrayList<>();
                    startLocation.add(startLonLat[0]);
                    startLocation.add(startLonLat[1]);
                    ridLocationList.add(startLocation);
                }
                // 关联路口wkt
                String wkt = ridInfo.getWkt();
                ridWktList.add(wkt);
            }
        }
        abnormalCrossListVO.setRidLocationList(ridLocationList);
        abnormalCrossListVO.setRidWktList(ridWktList);
    }

    @Override
    public AbnormalCrossDetailVO abnormalCrossDetail(CrossIdBO crossIdBO) {
        String crossId = crossIdBO.getCrossId();
        AbnormalCrossDetailVO abnormalCrossDetailVO = new AbnormalCrossDetailVO();

        // 构造路口实时数据
        CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(crossId);
        abnormalCrossDetailVO.setCrossStatus(crossDataRealtimePO.getStatus());
        abnormalCrossDetailVO.setCrossQueueLength(crossDataRealtimePO.getQueueLength());
        abnormalCrossDetailVO.setCrossFlowRate((double) crossDataRealtimePO.getFlow());
        abnormalCrossDetailVO.setCrossStopTimes(crossDataRealtimePO.getStopTimes());
        abnormalCrossDetailVO.setSpeed(crossDataRealtimePO.getSpeed());
        Double sturation = crossDataRealtimePO.getSturation();
        abnormalCrossDetailVO.setSturation((double) Math.round(sturation) * 100);
        String serviceLevel = CrossUtil.getServiceLevel(sturation);
        abnormalCrossDetailVO.setServiceLevel(serviceLevel);
        abnormalCrossDetailVO.setHourFlow(crossDataRealtimePO.getFlow() * 12);

        BigDecimal bigDecimal = BigDecimal.valueOf(crossDataRealtimePO.getCongestionIndex()).setScale(2, RoundingMode.HALF_UP);
        abnormalCrossDetailVO.setCongestionIndex(bigDecimal);
        // 构造转向实时数据
        List<AbnormalCrossDetailVO.TurnDataElement> turnData = buildTurnData(crossId);
        abnormalCrossDetailVO.setTurnData(turnData);
        // 构造方向历史数据
        List<AbnormalCrossDetailVO.DirDataElement> dirData = buildDirData(crossId);
        abnormalCrossDetailVO.setDirData(dirData);

        return abnormalCrossDetailVO;
    }

    @Override
    public List<EventAlarmVO> eventAlarm(Date todayTime) throws ParseException {
        List<EventAlarmPO> eventAlarmPOList = eventAlarmMapper.selectByTodayTime(todayTime);
        List<EventAlarmVO> eventAlarmOutVOList = new ArrayList<>();
        for (EventAlarmPO eventAlarmPO : eventAlarmPOList) {
            EventAlarmVO eventAlarmOutVO = new EventAlarmVO();
            eventAlarmOutVO.setEventDesc(eventAlarmPO.getEventDesc());
            // 事件类型
            Integer typeCode = eventAlarmPO.getType();
            String msgByCode = EventAlarmEnum.getMsgByCode(typeCode);
            eventAlarmOutVO.setType(msgByCode);

            Date endTime = eventAlarmPO.getEndTime();
            if (typeCode == 1 || typeCode == 2) {
                // 发生时间
                Date startTime = eventAlarmPO.getStartTime();
                String startTimeStr = sdf.format(startTime);
                String substring = startTimeStr.substring(11, 16);
                eventAlarmOutVO.setStartTime(substring);
                // 时长
                if (endTime == null) {
                    Date nowTime = DateUtil.date();
                    long between = DateUtil.between(startTime, nowTime, DateUnit.MINUTE);
                    int betweenInt = (int) between;
                    eventAlarmOutVO.setDuration(betweenInt);
                } else {
                    long between = DateUtil.between(startTime, endTime, DateUnit.MINUTE);
                    int betweenInt = (int) between;
                    eventAlarmOutVO.setDuration(betweenInt);
                }
            }
            if (typeCode == 3 || typeCode == 4) {
                // 时间范围
                Date startTime = eventAlarmPO.getStartTime();
                String startTimeStr = sdf.format(startTime);
                String startTimeSubStr = startTimeStr.substring(0, 16); // 截取到分钟
                String endTimeStr = sdf.format(endTime);
                String endTimeSubStr = endTimeStr.substring(0, 16); // 截取到分钟
                eventAlarmOutVO.setTimeSpan(startTimeSubStr + "~" + endTimeSubStr);
            }

            Integer sourceCode = eventAlarmPO.getSource();
            String sourceStr = EventAlarmSourceEnum.getMsgByCode(sourceCode);
            eventAlarmOutVO.setSource(sourceStr);
            eventAlarmOutVO.setCoordinate(eventAlarmPO.getWkt());

            eventAlarmOutVOList.add(eventAlarmOutVO);

        }
        return eventAlarmOutVOList;
    }

    @Override
    public List<GreenWaveCrossMonitorVO> greenwaveCrossMonitor(GreenwaveIdBO greenwaveIdBO) {
        Integer greenwaveId = greenwaveIdBO.getGreenwaveId();
        List<GreenwaveCrossPO> greenwaveCrossPOList = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);
        List<GreenWaveCrossMonitorVO> res = buildRes(greenwaveCrossPOList);
        return res;
    }

    @Override
    public List<GreenwaveRunMonitorVO> greenwaveRunMonitor(GreenwaveIdBO greenwaveIdBO) {
        List<GreenwaveRunMonitorVO> res = new ArrayList<>();
        Date nowTime = new Date();
        List<GreenwaveHistPOExt> extList = greenwaveHistMapper.selectRunMonitor(nowTime);
        for (GreenwaveHistPOExt greenwaveHistPOExt : extList) {
            GreenwaveRunMonitorVO greenWaveRunMonitorVO = new GreenwaveRunMonitorVO();
            String hour = greenwaveHistPOExt.getHour();
            String hourSub = hour.substring(11, 16); // 截取小时
            greenWaveRunMonitorVO.setHour(hourSub);
            greenWaveRunMonitorVO.setAvgSpeed(greenwaveHistPOExt.getAvgSpeed());
            greenWaveRunMonitorVO.setCongestionIndex(greenwaveHistPOExt.getCongestionIndex());
            res.add(greenWaveRunMonitorVO);
        }
        return res;
    }

    @Override
    public GreenwaveDetailVO greenwaveDetail(GreenwaveDetailBO greenwaveDetailBO) throws Exception {
        String greenwaveName = greenwaveDetailBO.getGreenwaveName();
        String startDate = greenwaveDetailBO.getStartDate();
        String endDate = greenwaveDetailBO.getEndDate();

        LineSchemeDTO lineSchemeDTO = MainlineEvaluateServiceImpl.lineSchemeBuffer.get(greenwaveName);
        Integer greenwaveId = lineSchemeDTO.getGreenwaveId();

        GreenwaveDetailVO res = new GreenwaveDetailVO();
        GreenwaveRealtimePO greenwaveRealtimePO = greenwaveRealtimeMapper.selectById(greenwaveId);
        GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectById(greenwaveId);
        res.setName(greenwaveInfoPO.getName());
        res.setStatus(greenwaveRealtimePO.getStatus());

        // 绿波带宽和设计速度
        List<GreenwaveCrossPOExt> greenwaveCrossPOExtList = greenwaveCrossMapper.selectAllWidthAndSpeed();
        String[] split = greenwaveName.split(" "); // 旅游路 东向西 转山西路至霞景路路段:正向绿波 工作日 19:00-20:00
        String baseGreenwaveName = split[0] + " " + split[1] + " " + split[2]; // 旅游路 东向西 转山西路至霞景路路段:正向绿波
        String planName = split[3]; // 工作日
        String timeSpan = split[4]; // 19:00-20:00
        String[] split1 = timeSpan.split("-");
        String startTime = split1[0];

        Optional<GreenwaveCrossPOExt> result = greenwaveCrossPOExtList.stream()
                .filter(item -> baseGreenwaveName.equals(item.getGreenwaveName()) &&
                        planName.equals(item.getPlanName()) &&
                        startTime.equals(item.getStartTime()))
                .findFirst();
        GreenwaveCrossPOExt foundItem = result.get();

        double width = foundItem.getWidth();
        double designSpeed = foundItem.getDesignSpeed() / 3.6; // 换算成米/秒
        int widthSeconds = (int) (width / designSpeed);
        Integer dir = greenwaveInfoPO.getDir();
        if (dir == 0) { // 正向
            res.setForwardWidth(widthSeconds);
        } else if (dir == 1) { // 反向
            res.setReverseWidth(widthSeconds);
        } else if (dir == 2) { // 双向
            res.setForwardWidth(widthSeconds);
            res.setReverseWidth(widthSeconds);
        }

        double uncoordinatePhaseQueue = greenwaveRealtimePO.getUncoordinatePhaseQueue();
        res.setUncoordinatePhaseQueue((int) uncoordinatePhaseQueue);
        double noparkPassRate = greenwaveRealtimePO.getNoparkPassRate();
        res.setNoparkPassRate((int) (noparkPassRate * 100));
        res.setTrvalTime(greenwaveRealtimePO.getTrvalTime());

        // 策略列表
        List<String> strategyList = strategyMapper.selectGreenwaveAllNames();
        res.setStrategyList(strategyList);
        List<StrategyPO> strategyPOList = strategyMapper.selectByGreenwaveId(greenwaveId);
        List<String> collect = strategyPOList.stream()
                .map(StrategyPO::getStrategyName)
                .collect(Collectors.toList());
        res.setStrategySelected(collect);
        // 绿波路口信息
        List<GreenwaveDetailVO.GreenwaveCross> greenwaveCrossList = buildGreenwaveCrossList(
                greenwaveId, greenwaveName, startDate, endDate);
        res.setGreenwaveCrossList(greenwaveCrossList);

        // 周期时长
        GreenwaveDetailVO.GreenwaveCross greenwaveCross = greenwaveCrossList.get(0);
        List<GreenwaveDetailVO.GreenwavePhase> phaseList = greenwaveCross.getGreenwavePhaseList();
        int cycle = phaseList.stream()
                .mapToInt(GreenwaveDetailVO.GreenwavePhase::getSeconds)
                .sum();
        res.setCycle(cycle);

        // 指针所在秒数
        res.setPointerSecond(calcPointerSecond(startTime, cycle));

        // 绿波带
        if (dir == 0) { // 正向
            List<GreenwaveDetailVO.GreenwavePoint> greenwavePoints =
                    buildForwardGreenwaveList(greenwaveCrossList, greenwaveInfoPO);
            addStartOffset(greenwavePoints, greenwaveCrossList);
            res.setForwardGreenwaveList(greenwavePoints);
        } else if (dir == 1) { // 反向
            List<GreenwaveDetailVO.GreenwavePoint> greenwavePoints =
                    buildReverseGreenwaveList(greenwaveCrossList, greenwaveInfoPO);
            addStartOffset(greenwavePoints, greenwaveCrossList);
            res.setReverseGreenwaveList(greenwavePoints);
        } else if (dir == 2) { // 双向
            List<GreenwaveDetailVO.GreenwavePoint> greenwavePoints =
                    buildForwardGreenwaveList(greenwaveCrossList, greenwaveInfoPO);
            addStartOffset(greenwavePoints, greenwaveCrossList);
            res.setForwardGreenwaveList(greenwavePoints);
            List<GreenwaveDetailVO.GreenwavePoint> greenwavePoints1 =
                    buildReverseGreenwaveList(greenwaveCrossList, greenwaveInfoPO);
            addStartOffset(greenwavePoints1, greenwaveCrossList);
            res.setReverseGreenwaveList(greenwavePoints1);
        }

        return res;
    }

    private void addStartOffset(List<GreenwaveDetailVO.GreenwavePoint> greenwavePoints,
                                List<GreenwaveDetailVO.GreenwaveCross> greenwaveCrossList) {
        for (GreenwaveDetailVO.GreenwavePoint greenwavePoint : greenwavePoints) {
            String crossId = greenwavePoint.getCrossId();
            for (GreenwaveDetailVO.GreenwaveCross greenwaveCross : greenwaveCrossList) {
                String crossId1 = greenwaveCross.getCrossId();
                if (Objects.equals(crossId1, crossId)) {
                    Integer coordOffset = greenwavePoint.getCoordOffset();
                    Integer seconds = calcTimeFromStart(greenwaveCross);
                    greenwavePoint.setOffsetToStart(coordOffset + seconds);
                }
            }
        }
    }

    private Integer calcTimeFromStart(GreenwaveDetailVO.GreenwaveCross greenwaveCross) {
        int res = 0;
        List<GreenwaveDetailVO.GreenwavePhase> phaseList = greenwaveCross.getGreenwavePhaseList();
        for (GreenwaveDetailVO.GreenwavePhase greenwavePhase : phaseList) {
            Integer seconds = greenwavePhase.getSeconds();
            Integer isCoordinate = greenwavePhase.getIsCoordinate();
            if (1 != isCoordinate) {
                res += seconds;
            } else {
                break;
            }
        }
        return res;
    }

    private Integer calcPointerSecond(String startTime, int cycle) {
        // 计算当前小时分钟数与录波开始时间相差的秒数
        int currentSecondDiff = calcCurrentSecondDiff(startTime);
        // 计算相差秒数除以相位周期的余数
        int res = currentSecondDiff % cycle;
        if (res == 0) {
            res = cycle;
        }
        return res;
    }

    private int calcCurrentSecondDiff(String startTime) {
        LocalTime now = LocalTime.now();
        LocalTime startTimeObj = LocalTime.parse(startTime);
        Duration duration = Duration.between(startTimeObj, now);
        long seconds = duration.getSeconds();
        return (int) seconds;
    }

    private List<GreenwaveDetailVO.GreenwavePoint> buildReverseGreenwaveList(
            List<GreenwaveDetailVO.GreenwaveCross> greenwaveCrossList, GreenwaveInfoPO greenwaveInfoPO) {
        List<GreenwaveDetailVO.GreenwavePoint> res = new ArrayList<>();
        // 找关键路口
        GreenwaveDetailVO.GreenwaveCross keyCross = null;
        Integer keyCrossBeginSeconds = null;
        for (GreenwaveDetailVO.GreenwaveCross greenwaveCross : greenwaveCrossList) {
            if (1 == greenwaveCross.getIsKeyRoute()) {
                keyCross = greenwaveCross;
                GreenwaveDetailVO.GreenwavePoint greenwavePoint = buildGreenwavePoint(greenwaveCross);
                Integer coordSeconds = greenwavePoint.getCoordSeconds();
                greenwavePoint.setCoordOffset(coordSeconds);
                res.add(greenwavePoint);
                keyCrossBeginSeconds = calcCoordBeginSeconds(greenwaveCross) + coordSeconds;
            }
        }
        Integer sort = keyCross.getSort();
        // 从关键路口向上
        int upSize = sort - 2;
        Integer preCrossBeginSeconds = keyCrossBeginSeconds;
        for (int i = upSize; i >= 0; i--) {
            GreenwaveDetailVO.GreenwaveCross greenwaveCross = greenwaveCrossList.get(i);
            GreenwaveDetailVO.GreenwavePoint greenwavePoint = buildGreenwavePoint(greenwaveCross);
            // 计算行程时间
            int distanceToNextCross = greenwaveCross.getDistanceToNextCross();
            double designSpeed = greenwaveInfoPO.getDesignSpeed() / 3.6; // 换算成米/秒
            int travalTime = (int) (distanceToNextCross / designSpeed);
            // 计算协调相位绿灯开始的绝对时间
            int absoluteSecondsBegin = calcCoordBeginSeconds(greenwaveCross);
            // 计算绿波点位的绝对时间
            int absoluteSecondsPoint = preCrossBeginSeconds + travalTime;
            greenwavePoint.setCoordOffset(absoluteSecondsPoint - absoluteSecondsBegin);

            res.add(greenwavePoint);
            preCrossBeginSeconds = absoluteSecondsPoint;
        }
        // 从关键路口向下
        preCrossBeginSeconds = keyCrossBeginSeconds;
        for (int i = sort; i < greenwaveCrossList.size(); i++) {
            GreenwaveDetailVO.GreenwaveCross greenwaveCross = greenwaveCrossList.get(i);
            GreenwaveDetailVO.GreenwaveCross greenwaveCrossAbove = greenwaveCrossList.get(i - 1);
            GreenwaveDetailVO.GreenwavePoint greenwavePoint = buildGreenwavePoint(greenwaveCross);
            // 计算行程时间
            int distanceToNextCross = greenwaveCrossAbove.getDistanceToNextCross();
            double designSpeed = greenwaveInfoPO.getDesignSpeed() / 3.6; // 换算成米/秒
            int travalTime = (int) (distanceToNextCross / designSpeed);
            // 计算协调相位绿灯开始的绝对时间
            int absoluteSecondsBegin = calcCoordBeginSeconds(greenwaveCross);
            // 计算绿波点位的绝对时间
            int absoluteSecondsPoint = preCrossBeginSeconds - travalTime;
            greenwavePoint.setCoordOffset(absoluteSecondsPoint - absoluteSecondsBegin);

            res.add(greenwavePoint);
            preCrossBeginSeconds = absoluteSecondsPoint;
        }
        List<GreenwaveDetailVO.GreenwavePoint> sorted = res.stream()
                .sorted(Comparator.comparing(GreenwaveDetailVO.GreenwavePoint::getSort))
                .collect(Collectors.toList());
        return sorted;
    }

    private List<GreenwaveDetailVO.GreenwavePoint> buildForwardGreenwaveList(
            List<GreenwaveDetailVO.GreenwaveCross> greenwaveCrossList, GreenwaveInfoPO greenwaveInfoPO) {
        List<GreenwaveDetailVO.GreenwavePoint> res = new ArrayList<>();
        // 找关键路口
        GreenwaveDetailVO.GreenwaveCross keyCross = null;
        Integer keyCrossBeginSeconds = null;
        for (GreenwaveDetailVO.GreenwaveCross greenwaveCross : greenwaveCrossList) {
            if (1 == greenwaveCross.getIsKeyRoute()) {
                keyCross = greenwaveCross;
                GreenwaveDetailVO.GreenwavePoint greenwavePoint = buildGreenwavePoint(greenwaveCross);
                greenwavePoint.setCoordOffset(0);
                res.add(greenwavePoint);
                keyCrossBeginSeconds = calcCoordBeginSeconds(greenwaveCross);
            }
        }
        Integer sort = keyCross.getSort();
        // 从关键路口向上
        int upSize = sort - 2;
        Integer preCrossBeginSeconds = keyCrossBeginSeconds;
        for (int i = upSize; i >= 0; i--) {
            GreenwaveDetailVO.GreenwaveCross greenwaveCross = greenwaveCrossList.get(i);
            GreenwaveDetailVO.GreenwavePoint greenwavePoint = buildGreenwavePoint(greenwaveCross);
            // 计算行程时间
            int distanceToNextCross = greenwaveCross.getDistanceToNextCross();
            double designSpeed = greenwaveInfoPO.getDesignSpeed() / 3.6; // 换算成米/秒
            int travalTime = (int) (distanceToNextCross / designSpeed);
            // 计算协调相位绿灯开始的绝对时间
            int absoluteSecondsBegin = calcCoordBeginSeconds(greenwaveCross);
            // 计算绿波点位的绝对时间
            int absoluteSecondsPoint = preCrossBeginSeconds - travalTime;
            greenwavePoint.setCoordOffset(absoluteSecondsPoint - absoluteSecondsBegin);

            res.add(greenwavePoint);
            preCrossBeginSeconds = absoluteSecondsPoint;
        }
        // 从关键路口向下
        preCrossBeginSeconds = keyCrossBeginSeconds;
        for (int i = sort; i < greenwaveCrossList.size(); i++) {
            GreenwaveDetailVO.GreenwaveCross greenwaveCross = greenwaveCrossList.get(i);
            GreenwaveDetailVO.GreenwaveCross greenwaveCrossAbove = greenwaveCrossList.get(i - 1);
            GreenwaveDetailVO.GreenwavePoint greenwavePoint = buildGreenwavePoint(greenwaveCross);
            // 计算行程时间
            int distanceToNextCross = greenwaveCrossAbove.getDistanceToNextCross();
            double designSpeed = greenwaveInfoPO.getDesignSpeed() / 3.6; // 换算成米/秒
            int travalTime = (int) (distanceToNextCross / designSpeed);
            // 计算协调相位绿灯开始的绝对时间
            int absoluteSecondsBegin = calcCoordBeginSeconds(greenwaveCross);
            // 计算绿波点位的绝对时间
            int absoluteSecondsPoint = preCrossBeginSeconds + travalTime;
            greenwavePoint.setCoordOffset(absoluteSecondsPoint - absoluteSecondsBegin);

            res.add(greenwavePoint);
            preCrossBeginSeconds = absoluteSecondsPoint;
        }
        List<GreenwaveDetailVO.GreenwavePoint> sorted = res.stream()
                .sorted(Comparator.comparing(GreenwaveDetailVO.GreenwavePoint::getSort))
                .collect(Collectors.toList());
        return sorted;
    }

    private int calcCoordBeginSeconds(GreenwaveDetailVO.GreenwaveCross greenwaveCross) {
        int res = 0;
        List<GreenwaveDetailVO.GreenwavePhase> greenwavePhaseList = greenwaveCross.getGreenwavePhaseList();
        for (GreenwaveDetailVO.GreenwavePhase greenwavePhase : greenwavePhaseList) {
            int seconds = greenwavePhase.getSeconds();
            int isCoordinate = greenwavePhase.getIsCoordinate();
            if (1 != isCoordinate) {
                res += seconds;
            } else {
                break;
            }
        }
        return res;
    }

    private GreenwaveDetailVO.GreenwavePoint buildGreenwavePoint(GreenwaveDetailVO.GreenwaveCross greenwaveCross) {
        GreenwaveDetailVO.GreenwavePoint greenwavePoint = new GreenwaveDetailVO.GreenwavePoint();
        greenwavePoint.setSort(greenwaveCross.getSort());
        greenwavePoint.setCrossId(greenwaveCross.getCrossId());
        greenwavePoint.setCrossName(greenwaveCross.getCrossName());
        List<GreenwaveDetailVO.GreenwavePhase> greenwavePhaseList = greenwaveCross.getGreenwavePhaseList();
        for (GreenwaveDetailVO.GreenwavePhase greenwavePhase : greenwavePhaseList) {
            if (1 == greenwavePhase.getIsCoordinate()) {
                greenwavePoint.setCoordSeconds(greenwavePhase.getSeconds());
            }
        }
        return greenwavePoint;
    }

    private List<GreenwaveDetailVO.GreenwaveCross> buildGreenwaveCrossList(
            Integer greenwaveId, String greenwaveName, String startDate, String endDate) throws Exception {

        String[] split = greenwaveName.split(" "); // 旅游路 东向西 转山西路至霞景路路段:正向绿波 工作日 19:00-20:00
        String s = split[4]; // 19:00-20:00
        String[] split1 = s.split("-");
        String startTime = split1[0];
        String endTime = split1[1];
        String planName = split[3];

        List<GreenwaveDetailVO.GreenwaveCross> res = new ArrayList<>();
        List<GreenwaveCrossPO> greenwaveCrossList = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);

        // 获取绿波的路口ID列表
        List<String> crossList = greenwaveCrossList.stream()
                .map(GreenwaveCrossPO::getCrossId)
                .distinct()
                .collect(Collectors.toList());

        // 查询路口时段表基础数据
        List<CrossSectionPOExt> crossSectionPoExtList =
                baseCrossSectionMapper.selectByCrossIdsAndStartTime(startTime, crossList);

        // 根据方案名和开始时间过滤
        List<Integer> sectionIdList = crossSectionPoExtList.stream()
                .filter(poExt -> planName.equals(poExt.getPlanName()) && startTime.equals(poExt.getStartTime()))
                .map(CrossSectionPOExt::getId)
                .collect(Collectors.toList());

        // 根据时段ID过滤数据
        List<GreenwaveCrossPO> filteredList = greenwaveCrossList.stream()
                .filter(greenwaveCross -> sectionIdList.contains(greenwaveCross.getSectionId()))
                .collect(Collectors.toList());
        filteredList.sort(Comparator.comparing(GreenwaveCrossPO::getSort));

        // 相对相位差列表
        List<Integer> relativeOffsets = filteredList.stream()
                .map(GreenwaveCrossPO::getOffset)
                .collect(Collectors.toList());
        // 绝对相位差列表
        List<Integer> absoluteOffsets = buildAbsoluteOffsets(relativeOffsets);

        int size = filteredList.size();
        for (int i = 0; i < size; i++) {
            GreenwaveCrossPO greenwaveCrossPO = filteredList.get(i);
            GreenwaveDetailVO.GreenwaveCross greenwaveCross = new GreenwaveDetailVO.GreenwaveCross();
            String crossId = greenwaveCrossPO.getCrossId();
            greenwaveCross.setCrossId(crossId);
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            if (baseCrossInfoPO == null) {
                throw new OptServiceException("无此路口基础信息");
            }
            greenwaveCross.setCrossName(baseCrossInfoPO.getName());
            greenwaveCross.setIsKeyRoute(greenwaveCrossPO.getIsKeyRoute());
            greenwaveCross.setSort(greenwaveCrossPO.getSort());
            // 到下一个路口距离和速度
            Integer distanceToNextCross = null;
            Double speed = null;
            if (i < size - 1) { // 不是最后一个路口
                double nextCrossLen = greenwaveCrossPO.getNextCrossLen();
                distanceToNextCross = (int) nextCrossLen;

                GreenwaveCrossPO nextCrossPO = filteredList.get(i + 1);
                String nextCrossId = nextCrossPO.getCrossId();
                speed = calcSpeed(greenwaveName, nextCrossId, startTime, endTime, startDate, endDate);
            }
            greenwaveCross.setDistanceToNextCross(distanceToNextCross);
            greenwaveCross.setSpeed(speed);
            // 相位差
            greenwaveCross.setOffset(greenwaveCrossPO.getOffset());

            // 获取当前方案ID
            BaseCrossPlanPO baseCrossPlanPO = baseCrossPlanMapper.selectByCrossIdAndName(crossId, planName);
            Integer planId = baseCrossPlanPO.getId();
            CrossSectionPO crossSectionPO = baseCrossSectionMapper.
                    selectbyStartTimeCrossIdPlanId(startTime, crossId, planId);
            if (crossSectionPO == null) {
                throw new OptServiceException("无当前时段方案信息");
            }
            Integer currentSchemeId = crossSectionPO.getSchemeId();
            BaseCrossSchemePO baseCrossSchemePO = baseCrossSchemeMapper.selectById(currentSchemeId);

            // 构造相位时差图
            Integer absoluteOffset = absoluteOffsets.get(i);
            Integer crossSort = greenwaveCrossPO.getSort();
            List<GreenwaveDetailVO.GreenwavePhase> greenwavePhaseList =
                    getGreenwavePhaseList(baseCrossSchemePO, absoluteOffset, crossSort);
            greenwaveCross.setGreenwavePhaseList(greenwavePhaseList);

            res.add(greenwaveCross);
        }
        return res;
    }

    private List<Integer> buildAbsoluteOffsets(List<Integer> relativeOffsets) {
        List<Integer> res = new ArrayList<>();
        for (int i = 0; i < relativeOffsets.size(); i++) {
            Integer subSum = 0;
            for (int j = 0; j <= i; j++) {
                subSum += relativeOffsets.get(j);
            }
            res.add(subSum);
        }
        return res;
    }

    private List<GreenwaveDetailVO.GreenwavePhase> getGreenwavePhaseList(
            BaseCrossSchemePO baseCrossSchemePO, Integer absoluteOffset, Integer crossSort) {

        List<GreenwaveDetailVO.GreenwavePhase> res = new ArrayList<>();
        Integer schemeId = baseCrossSchemePO.getId();
        String crossId = baseCrossSchemePO.getCrossId();
        Integer coordPhaseId = baseCrossSchemePO.getCoordPhase();
        // 获取方案相位并按相位序排序
        List<CrossPhasePO> crossPhasePOList = baseCrossPhaseMapper.selectByCrossIdAndSchemeId(crossId, schemeId);
        if (1 == crossSort) { // 第一个路口没有相位差
            int size = crossPhasePOList.size();
            for (int i = 0; i < size; i++) {
                CrossPhasePO crossPhasePO = crossPhasePOList.get(i);
                GreenwaveDetailVO.GreenwavePhase greenwavePhase = new GreenwaveDetailVO.GreenwavePhase();

                greenwavePhase.setSort(i + 1);
                greenwavePhase.setPhaseNo(crossPhasePO.getPhaseNo());
                greenwavePhase.setSeconds(crossPhasePO.getPhaseTime());
                Integer phaseId = crossPhasePO.getId();
                greenwavePhase.setIsCoordinate(ObjectUtil.equals(phaseId, coordPhaseId) ? 1 : 0);

                res.add(greenwavePhase);
            }
            return res;
        } else {
            // 其余路口，首先根据绝对相位差构造第一个相位，与最后一个相位首位相接
            GreenwaveDetailVO.GreenwavePhase firstGreenwavePhase = new GreenwaveDetailVO.GreenwavePhase();
            int size = crossPhasePOList.size();
            CrossPhasePO endCrossPhasePO = crossPhasePOList.get(size - 1);
            firstGreenwavePhase.setSort(1);
            firstGreenwavePhase.setPhaseNo(endCrossPhasePO.getPhaseNo());
            firstGreenwavePhase.setSeconds(absoluteOffset);
            Integer firstPhaseId = endCrossPhasePO.getId();
            firstGreenwavePhase.setIsCoordinate(ObjectUtil.equals(firstPhaseId, coordPhaseId) ? 1 : 0);
            res.add(firstGreenwavePhase);

            for (int i = 0; i < size; i++) {
                CrossPhasePO crossPhasePO = crossPhasePOList.get(i);
                GreenwaveDetailVO.GreenwavePhase greenwavePhase = new GreenwaveDetailVO.GreenwavePhase();

                greenwavePhase.setSort(i + 2);
                greenwavePhase.setPhaseNo(crossPhasePO.getPhaseNo());
                if (i == size - 1) {
                    greenwavePhase.setSeconds(crossPhasePO.getPhaseTime() - absoluteOffset);
                } else {
                    greenwavePhase.setSeconds(crossPhasePO.getPhaseTime());
                }
                Integer phaseId = crossPhasePO.getId();
                greenwavePhase.setIsCoordinate(ObjectUtil.equals(phaseId, coordPhaseId) ? 1 : 0);

                res.add(greenwavePhase);
            }
            return res;
        }
    }

    private int convertWeekDay(int i) {
        if (1 == i) {
            return 7;
        }
        return i - 1;
    }

    private Double calcSpeed(String greenwaveName, String crossId,
                             String startTime, String endTime, String startDate, String endDate) throws Exception {
        // 确定协调方向
        String[] split = greenwaveName.split(" ");
        String dirStr = split[1];
        String[] split1 = dirStr.split("向");
        String lineName = split[0] + " " + split[1] + " " + split[2];
        GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectByName(lineName);
        Integer dir = greenwaveInfoPO.getDir();

        List<Integer> coordDirCodeList = new ArrayList<>();
        if (dir == 0) { // 正向
            String dirStr1 = split1[0];
            Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr1);
            dirCode = convertDirCode(dirStr1, crossId, dirCode);
            coordDirCodeList.add(dirCode);
        } else if (dir == 1) { // 反向
            String dirStr1 = split1[1];
            Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr1);
            dirCode = convertDirCode(dirStr1, crossId, dirCode);
            coordDirCodeList.add(dirCode);
        } else { // 双向
            String dirStr1 = split1[0];
            Integer dirCode1 = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr1);
            dirCode1 = convertDirCode(dirStr1, crossId, dirCode1);
            coordDirCodeList.add(dirCode1);
            String dirStr2 = split1[1];
            Integer dirCode2 = BaseEnum.SignalDirectionEnum.getCodeByName(dirStr2);
            dirCode2 = convertDirCode(dirStr2, crossId, dirCode2);
            coordDirCodeList.add(dirCode2);
        }

        if (ObjectUtil.isEmpty(startDate)) { // 查询实时数据
            CrossDirDataRealtimePO crossDirDataRealtimePO =
                    crossDirDataRealtimeMapper.selectByCrossIdAndDirs(crossId, coordDirCodeList);
            return crossDirDataRealtimePO.getSpeed();
        } else { // 查询历史数据
            List<CrossDirDataHistPO> crossDirDataHistPOList = crossDirDataHistMapper.selectByCrossDirsAndTimeSection(
                    crossId, coordDirCodeList, startDate, endDate, startTime, endTime);

            double v = crossDirDataHistPOList.stream()
                    .mapToDouble(CrossDirDataHistPO::getSpeed)
                    .average().orElse(0.0);
            BigDecimal formattedAverage = new BigDecimal(v).setScale(2, RoundingMode.HALF_UP);
            return formattedAverage.doubleValue();
        }
    }

    @Override
    public List<GreenwaveCrossMetricsVO> greenwaveCrossMetrics(GreenwaveIdAndTimeStampBO greenwaveIdAndTimeStampBO) throws Exception {
        List<GreenwaveCrossMetricsVO> res = new ArrayList<>();
        Long timeStamp = greenwaveIdAndTimeStampBO.getTimeStamp();

        Integer greenwaveId = greenwaveIdAndTimeStampBO.getGreenwaveId();
        GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectById(greenwaveId);
        String greenwaveName = greenwaveInfoPO.getName(); // 旅游路 东向西 转山西路至霞景路路段:正向绿波
        String[] split = greenwaveName.split(" ");
        String[] split1 = split[1].split("向"); // 东向西
        String dirName1 = split1[0];
        String dirName2 = split1[1];

        List<GreenwaveCrossPO> greenwaveCrossList = greenwaveCrossMapper.selectByGreenwaveId(greenwaveId);
        // 根据路口ID去重，并按路口顺序号排序
        Map<String, GreenwaveCrossPO> uniqueMap = greenwaveCrossList.stream()
                .collect(Collectors.toMap(
                        GreenwaveCrossPO::getCrossId,    // 使用crossId作为键
                        Function.identity(),              // 使用对象本身作为值
                        (existing, replacement) -> existing));  // 如果有重复的key, 保留第一个

        // 从uniqueMap中获取去重后的记录列表
        List<GreenwaveCrossPO> uniqueList = new ArrayList<>(uniqueMap.values());

        Collections.sort(uniqueList, Comparator.comparing(GreenwaveCrossPO::getSort));
        for (GreenwaveCrossPO greenwaveCrossPO : uniqueList) {
            GreenwaveCrossMetricsVO greenwaveCrossMetricsVO = new GreenwaveCrossMetricsVO();
            String crossId = greenwaveCrossPO.getCrossId();
            greenwaveCrossMetricsVO.setCrossId(crossId);
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            if (baseCrossInfoPO != null) {
                greenwaveCrossMetricsVO.setCrossName(baseCrossInfoPO.getName());
            }
            List<Integer> coordDirList = new ArrayList<Integer>();
            int direction = greenwaveInfoPO.getDir();

            if (direction == 0) { // 正向
                Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirName1);
                dirCode = convertDirCode(dirName1, crossId, dirCode);
                coordDirList.add(dirCode);
            } else if (direction == 2) { // 反向
                Integer dirCode = BaseEnum.SignalDirectionEnum.getCodeByName(dirName2);
                dirCode = convertDirCode(dirName2, crossId, dirCode);
                coordDirList.add(dirCode);
            } else { // 双向
                Integer dirCode1 = BaseEnum.SignalDirectionEnum.getCodeByName(dirName1);
                dirCode1 = convertDirCode(dirName1, crossId, dirCode1);
                coordDirList.add(dirCode1);
                Integer dirCode2 = BaseEnum.SignalDirectionEnum.getCodeByName(dirName2);
                dirCode2 = convertDirCode(dirName1, crossId, dirCode2);
                coordDirList.add(dirCode2);
            }

            int currentSeconds = (int) (timeStamp / 1000);
            int preSeconds = (int) (currentSeconds - 60 * 5);
            List<CrossDirDataHistPO> crossDirDataHistPOList =
                    crossDirDataHistMapper.selectByCrossIdAndStartEnd(crossId, preSeconds, currentSeconds);

            greenwaveCrossMetricsVO.setNoparkPassRate(calcNoparkPassRate(crossDirDataHistPOList, coordDirList));
            greenwaveCrossMetricsVO.setUncoordinatePhaseQueue(calcUncoordinatePhaseQueue(crossDirDataHistPOList, coordDirList));
            greenwaveCrossMetricsVO.setTrvalTime(calcTravelTime(greenwaveCrossPO, crossId, coordDirList, timeStamp - 5 * 60 * 1000, timeStamp));

            res.add(greenwaveCrossMetricsVO);
        }
        return res;
    }

    private Integer calcTravelTime(GreenwaveCrossPO greenwaveCrossPO, String crossId, List<Integer> coordDirList, long startTime, long endTime) throws Exception {
        // 从 Kafka 查询绿灯空放时长
        KafkaConsumerUtil kafkaConsumerUtil = new KafkaConsumerUtil(bootstrapServers, "line-travel-time");
        List<LineCongestion> results =
                kafkaConsumerUtil.consumeLineMetricForTimeRange(
                        "line_congestion_sink", 0, startTime, endTime);
        results = results.stream().filter(o -> o.getGreenWaveId().equals(greenwaveCrossPO.getGreenId())).collect(Collectors.toList());

        List<Double> travelTimeList = new ArrayList<>();
        for (LineCongestion dto : results) {
            Map<String, Double> travelMap = dto.getRidTravelTime();
            travelMap.forEach((k, v) -> {
                for (Integer inDir : coordDirList) {
                    String keyPre = crossId + "_" + inDir;
                    if (k.contains(keyPre)) {
                        travelTimeList.add(v);
                    }
                }
            });
        }
        return (int) (travelTimeList.stream().mapToDouble(Double::doubleValue).average().orElse(0));

    }

    private Integer calcTravelTime(String crossId, Integer inDir) throws Exception {
        try {
            // 从 Redis 中获取路段行程时间
            String redisKey = crossId + ":" + inDir;
            Object element = redisUtils.getMaxScoreElement(redisKey);
            ObjectMapper objectMapper = new ObjectMapper();
            if (element != null) {
                CrossEventDTO dto = objectMapper.readValue(element.toString(), CrossEventDTO.class);
                return dto.getTransitTime().intValue();
            }
        } catch (Exception e) {
            log.error("获取绿波旅行时间异常：", e);
            throw new OptServiceException(e);
        }
        return 0;
    }

    private Integer calcUncoordinatePhaseQueue(
            List<CrossDirDataHistPO> crossDirDataHistPOList, List<Integer> coordDirList) {
        double average = crossDirDataHistPOList.stream()
                .filter(po -> po.getInOutType() == 1 && !coordDirList.contains(po.getDirType()))
                .mapToDouble(CrossDirDataHistPO::getQueueLength)
                .average()
                .orElse(0.0);

        return (int) average;
    }

    private Integer calcNoparkPassRate(List<CrossDirDataHistPO> crossDirDataHistPOList, List<Integer> coordDirList) {
        double average = crossDirDataHistPOList.stream()
                .filter(po -> po.getInOutType() == 1 && coordDirList.contains(po.getDirType()))
                .mapToDouble(CrossDirDataHistPO::getNoStopRate)
                .average()
                .orElse(0.0);

        return (int) (average * 100);
    }

    private Integer convertDirCode(String dirStr, String crossId, Integer dirCode) {
        if (ObjectUtil.isNotEmpty(directionMappingsConfig)) {
            List<String> dirMapList = directionMappingsConfig.getMappings().get(crossId); // 如 ["东:2","西:6"]
            if (CollectionUtil.isNotEmpty(dirMapList)) {
                for (String dirMapStr : dirMapList) {
                    String[] split = dirMapStr.split(":");
                    if (dirStr.equals(split[0])) {
                        dirCode = Integer.parseInt(split[1]);
                    }
                }
            }
        }
        return dirCode;
    }

    @Override
    @Transactional
    public void saveGreenwaveStrategy(SaveGreenwaveStrategyBO saveGreenwaveStrategyBO) {
        Integer greenwaveId = saveGreenwaveStrategyBO.getGreenwaveId();
        List<String> strategyNames = saveGreenwaveStrategyBO.getStrategyNames();
        GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectById(greenwaveId);
        Integer dir = greenwaveInfoPO.getDir();
        // 删除旧关联
        greenwaveSceneMapper.deleteByGreenwaveId(greenwaveId);
        for (String strategyName : strategyNames) {
            // 获取策略ID
            StrategyPO strategyPO = strategyMapper.selectByName(strategyName);
            Integer strategyId = strategyPO.getId();
            // 根据策略ID查询场景IDs
            List<Integer> sceneIds = sceneStrategyMapper.selectSceneIdsByStrategyId(strategyId);
            // 插入新关联
            for (Integer sceneId : sceneIds) {
                // 查询场景代码
                ScenePO scenePO = sceneMapper.selectById(sceneId);
                Integer sceneNum = scenePO.getSceneNum();
                Integer sceneCodeByDir = SceneDirEnum.getSceneNumByDir(dir);
                if (Objects.equals(sceneNum, sceneCodeByDir)) {
                    GreenwaveScenePO greenwaveScenePO = new GreenwaveScenePO();
                    greenwaveScenePO.setGreenwaveId(greenwaveId);
                    greenwaveScenePO.setSceneId(sceneId);
                    greenwaveSceneMapper.insertOne(greenwaveScenePO);
                    break;
                }
            }
        }
    }

    @Override
    public GreenwaveDetailVO currentGreenwaveDetail(GreenwaveIdBO bo) throws Exception {

        Integer greenwaveId = bo.getGreenwaveId();
        // 获取绿波名称，调用评价接口
        GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectById(greenwaveId);
        String greenwaveName = greenwaveInfoPO.getName();
        String[] split = greenwaveName.split(":");
        String lineName = split[0];

        MainlineSchemeAnalysisBO mainlineSchemeAnalysisBO = new MainlineSchemeAnalysisBO();
        mainlineSchemeAnalysisBO.setName(lineName);

        // 获取当前日期和时间的 Calendar 对象
        Calendar calendar = Calendar.getInstance();
        calendar.set(2024, Calendar.JANUARY, 15, 10, 0, 0);
        // 设置时、分、秒和毫秒为 0，以获取当天的 00:00:00
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        Date startOfDay = calendar.getTime();

        // 设置时、分、秒和毫秒为 23:59:59.999，以获取当天的 23:59:59
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        Date endOfDay = calendar.getTime();

        mainlineSchemeAnalysisBO.setStartTime(startOfDay);
        mainlineSchemeAnalysisBO.setEndTime(endOfDay);
        MainlineSchemeAnalysisVO mainlineSchemeAnalysisVO = mainlineEvaluateServiceImpl
                .mainlineSchemeAnalysis(mainlineSchemeAnalysisBO);

        // 获取当前时段的干线方案
        List<MainlineSchemeAnalysisVO.GreenwaveData> greenwaveData = mainlineSchemeAnalysisVO.getGreenwaveData();
        MainlineSchemeAnalysisVO.GreenwaveData matchingData = findMatchingData(greenwaveData);
        if (ObjectUtil.isEmpty(matchingData)) {
            throw new OptServiceException("无当天绿波历史数据");
        }
        String lineSchemeName = matchingData.getName();
        GreenwaveDetailBO greenwaveDetailBO = new GreenwaveDetailBO();
        greenwaveDetailBO.setGreenwaveName(lineSchemeName);
        GreenwaveDetailVO greenwaveDetailVO = greenwaveDetail(greenwaveDetailBO);

        return greenwaveDetailVO;
    }

    @Override
    public CountRealTimeVO countRealTime(String crossId) throws Exception {
        CountRealTimeVO countRealTimeVO = new CountRealTimeVO();

        ZoneId zoneId = ZoneId.systemDefault();
        ZonedDateTime startOfDay = LocalDateTime.now().toLocalDate().atStartOfDay(zoneId);
        ZonedDateTime now = ZonedDateTime.now(zoneId);
        int startTimeStamp = (int) startOfDay.toEpochSecond();
        int endTimeStamp = (int) now.toEpochSecond();

        // 统计实时条数
        int counts = laneSnapshotDataQueryService.queryCountsByCrossIdAndTimeSpan(
                crossId, startTimeStamp, endTimeStamp);
        countRealTimeVO.setRealTimeCount(counts);

        // 统计周期条数
        Integer cycleCount = crossLaneDataHistMapper.
                selectCountByCrossIdAndTimeSpan(crossId, startTimeStamp, endTimeStamp);
        countRealTimeVO.setCycleCount(cycleCount);
        return countRealTimeVO;
    }

    @Override
    public TableQueryVO tableQuery(CrossIdAndTimeSpanBO crossIdAndTimeSpanBO) throws Exception {
        String crossId = crossIdAndTimeSpanBO.getCrossId();
        Date startTime = crossIdAndTimeSpanBO.getStartTime();
        Date endTime = crossIdAndTimeSpanBO.getEndTime();
        // 将Date对象转换为10位时间戳
        int startTimeStamp = (int) (startTime.getTime() / 1000);
        int endTimeStamp = (int) (endTime.getTime() / 1000);

        return buildTableQueryVO(crossId, startTimeStamp, endTimeStamp, startTimeStamp);
    }

    private List<TableQueryVO.RealTimeDataElement> buildRealTimeList(List<CrossLaneSnapshotDataDTO> dtoList) {
        List<TableQueryVO.RealTimeDataElement> res = new ArrayList<>();

        for (CrossLaneSnapshotDataDTO dto : dtoList) {
            TableQueryVO.RealTimeDataElement element = new TableQueryVO.RealTimeDataElement();

            String timeStamp = dto.getTimeStamp();
            long timeStampInt = Long.parseLong(timeStamp);
            Date date = new Date(timeStampInt * 1000); // 转换为毫秒
            element.setTime(date);

            element.setDir(dto.getDir());
            element.setDirName(BaseEnum.SignalDirectionEnum.getNameByCode(dto.getDir()));

            String laneId = dto.getLaneId();
            String laneSort = laneId.substring(laneId.length() - 2);
            element.setLaneSort(laneSort);
            element.setLaneId(laneId);

            double staticQueueLength = dto.getStaticQueueLength();
            int staticQueueLengthRound = (int) Math.round(staticQueueLength);
            element.setQueueLength(staticQueueLengthRound);

            double headCarPosition = dto.getHeadCarPosition();
            Integer headCarPositionRound = (int) Math.round(headCarPosition);
            if (headCarPositionRound == -1) {
                headCarPositionRound = null;
            }
            element.setFirstCarPosition(headCarPositionRound);

            double tailCarPosition = dto.getTailCarPosition();
            Integer tailCarPositionRound = (int) Math.round(tailCarPosition);
            if (tailCarPositionRound == -1) {
                tailCarPositionRound = null;
            }
            element.setEndCarPosition(tailCarPositionRound);

            element.setQueueNums(dto.getQueueNums());
            element.setCarNums(dto.getCarNums());
            element.setVehicleLengthRatio(Math.round(dto.getVehicleLengthRatio() * 100.0));
            element.setVehicleNumsRatio(Math.round(dto.getVehicleNumsRatio() * 100.0));
            element.setSpeed(Math.round(dto.getMeanV() * 100.0) / 100.0);
            element.setHeadSpeed(Math.round(dto.getStartV() * 100.0) / 100.0);
            element.setMailSpeed(Math.round(dto.getEndV() * 100.0) / 100.0);
            element.setTeamHeadDistance(Math.round(dto.getTeamHeadDistance() * 100.0) / 100.0);
            element.setTeamTailDistance(Math.round(dto.getTeamTailDistance() * 100.0) / 100.0);
            element.setStdSpaceHeadway(Math.round(dto.getVehicleLengthRatio() * 100.0));
            element.setOverflowNums(dto.getOverflowNums());
            res.add(element);
        }

        return res;
    }

    @Override
    public TableQueryVO tableRealTime(CrossIdAndIsFirstBO crossIdAndIsFirstBO) throws Exception {
        String crossId = crossIdAndIsFirstBO.getCrossId();
        Integer isFirstInvoke = crossIdAndIsFirstBO.getIsFirstInvoke();
        // 如果是初次调用，需初始化开始时间
        if (1 == isFirstInvoke) {
            initFirstDate();
        }
        // 以当前时间为结束时间
        Date endDate = new Date();
        int endTimeStamp = (int) (endDate.getTime() / 1000);
        int realTimeStartTimeStamp = (int) (tableRealTimeFirstDate.getTime() / 1000);
        int cycleStartTimeStamp = (int) (tableCycleFirstDate.getTime() / 1000);

        return buildTableQueryVO(crossId, cycleStartTimeStamp, endTimeStamp, realTimeStartTimeStamp);
    }

    @NotNull
    private TableQueryVO buildTableQueryVO(
            String crossId, int cycleStartTimeStamp, int endTimeStamp, int realTimeStartTimeStamp)
            throws Exception {

        TableQueryVO tableQueryVO = new TableQueryVO();

        // 查询秒级数据
        List<CrossLaneSnapshotDataDTO> dtoList =
                laneSnapshotDataQueryService.queryByCrossIdAndTimeSpan(crossId, realTimeStartTimeStamp,
                        endTimeStamp, 0, null, false);
        tableQueryVO.setRealTimeData(buildRealTimeList(dtoList));

        // 查询周期数据
        List<CrossLaneDataHistPOExt> poExtList = crossLaneDataHistMapper
                .selectByCrossIdAndTimeSpan(crossId, cycleStartTimeStamp, endTimeStamp);
        tableQueryVO.setCycleData(buildCycleData(crossId, poExtList));

        return tableQueryVO;
    }

    @Override
    public OptTypeVO optType(CrossIdBO crossIdBO) {
        String crossId = crossIdBO.getCrossId();
        List<CrossSchemeOptLogPO> poList = crossSchemeOptLogMapper.selectMaxByCrossId(crossId);
        CrossSchemeOptLogPO po = poList.get(0);
        String optType = po.getOptType();
        Integer optTypeInt = Integer.valueOf(optType);

        OptTypeVO optTypeVO = new OptTypeVO();
        optTypeVO.setOptType(optTypeInt);
        return optTypeVO;
    }

    private void initFirstDate() {
        tableRealTimeFirstDate = new Date();
        // 将当前时间向前推10分钟
        LocalDateTime now = LocalDateTime.now();
        LocalDateTime tenMinutesAgo = now.minus(10, ChronoUnit.MINUTES);
        Date resultDate = Date.from(tenMinutesAgo.atZone(ZoneId.systemDefault()).toInstant());
        tableCycleFirstDate = resultDate;
    }

    private List<TableQueryVO.CycleDataElement> buildCycleData(String crossId, List<CrossLaneDataHistPOExt> poExtList) {
        List<TableQueryVO.CycleDataElement> res = new ArrayList<>();

        int pedFlow = 0;
        for (CrossLaneDataHistPOExt po : poExtList) {
            TableQueryVO.CycleDataElement vo = new TableQueryVO.CycleDataElement();

            if (po.getId().contains(crossId.concat(Constants.SystemParam.SEPARATOR_MINUS))) {
                pedFlow += po.getFlow();
            }
            Integer batchTime = po.getBatchTime();
            // 将10位时间戳转换为毫秒
            long millis = batchTime * 1000L;
            // 使用毫秒值创建Date对象
            Date dateFromTimestamp = new Date(millis);
            vo.setTime(dateFromTimestamp);

            vo.setDir(po.getDir());
            vo.setDirName(BaseEnum.SignalDirectionEnum.getNameByCode(po.getDir()));
            vo.setFlow(po.getFlow());

            Double speed = po.getSpeed();
            vo.setSpeed((int) Math.round(speed));
            Double queueLength = po.getQueueLength();
            vo.setQueueLength((int) Math.round(queueLength));

            Integer delayTime = po.getDelayTime();
            vo.setDelayTime(delayTime);

            Double stopTimes = po.getStopTimes();
            vo.setStopTimes((int) Math.round(stopTimes));
            Double vehheadTime = po.getVehheadTime();
            vo.setVehheadTime((int) Math.round(vehheadTime));

            Double timeOccupancyDouble = po.getTimeOccupancy();
            if (timeOccupancyDouble != null) {
                vo.setTimeOccupancy((int) Math.round(timeOccupancyDouble * 100));
            }

            Double vehicleLengthRatioMean = po.getVehicleLengthRatioMean();
            if (vehicleLengthRatioMean != null) {
                vo.setVehicleNumsRatioMean((int) Math.round(vehicleLengthRatioMean * 100));
            }

            vo.setVehheadDist(po.getVehheadDist());
            vo.setTrafficFlowA(po.getTrafficFlowA());
            vo.setTrafficFlowB(po.getTrafficFlowB());
            vo.setTrafficFlowC(po.getTrafficFlowC());
            vo.setNonMotorFlow(po.getNonMotorFlow());
            vo.setV85(Math.round(po.getV85() * 100) / 100);
            vo.setPedFlow(pedFlow);
            vo.setAllFlow(pedFlow + po.getNonMotorFlow() + po.getFlow());
            vo.setAllVehiceleFlow(po.getFlow() + po.getNonMotorFlow());
            vo.setTurn(po.getTurn());

            String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(po.getDir());
            String turnName = TurnConvertEnum.getDescByKey(po.getTurn());
            vo.setLaneSort(String.join("-", dirName, turnName, String.valueOf(po.getSort())));

            res.add(vo);
        }

        List<TableQueryVO.CycleDataElement> sortedRes = res.stream()
                .sorted(Comparator.comparing(TableQueryVO.CycleDataElement::getTime).reversed())
                .collect(Collectors.toList());
        return sortedRes;
    }

    private MainlineSchemeAnalysisVO.GreenwaveData findMatchingData(
            List<MainlineSchemeAnalysisVO.GreenwaveData> greenwaveData) {

        Calendar now = Calendar.getInstance();
        String currentTime = new SimpleDateFormat("HH:mm").format(now.getTime());

        for (MainlineSchemeAnalysisVO.GreenwaveData data : greenwaveData) {
            String name = data.getName();
            // 旅游路 东向西 转山西路至霞景路路段:正向绿波 工作日 07:00-07:40
            Pattern pattern = Pattern.compile(".*?(\\d{2}:\\d{2}-\\d{2}:\\d{2})");
            Matcher matcher = pattern.matcher(name);
            if (matcher.matches()) {
                String timeRange = matcher.group(1);
                String[] times = timeRange.split("-");
                try {
                    Date startTime = new SimpleDateFormat("HH:mm").parse(times[0]);
                    Date endTime = new SimpleDateFormat("HH:mm").parse(times[1]);
                    Date currentTimeDate = new SimpleDateFormat("HH:mm").parse(currentTime);

                    if (currentTimeDate.after(startTime) && currentTimeDate.before(endTime)) {
                        return data;
                    }
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
        }

        return null;
    }

    private List<GreenWaveCrossMonitorVO> buildRes(List<GreenwaveCrossPO> greenwaveCrossPOList) {
        List<GreenWaveCrossMonitorVO> res = new ArrayList<>();

        // 根据路口ID去重，并按路口顺序号排序
        Map<String, GreenwaveCrossPO> uniqueMap = greenwaveCrossPOList.stream()
                .collect(Collectors.toMap(
                        GreenwaveCrossPO::getCrossId,    // 使用crossId作为键
                        Function.identity(),              // 使用对象本身作为值
                        (existing, replacement) -> existing));  // 如果有重复的key, 保留第一个

        // 从uniqueMap中获取去重后的记录列表
        List<GreenwaveCrossPO> uniqueList = new ArrayList<>(uniqueMap.values());

        // 根据sort属性对uniqueList进行排序
        uniqueList.sort(Comparator.comparingInt(GreenwaveCrossPO::getSort));

        for (GreenwaveCrossPO greenwaveCrossPO : uniqueList) {
            GreenWaveCrossMonitorVO greenWaveCrossMonitorVO = new GreenWaveCrossMonitorVO();

            String crossId = greenwaveCrossPO.getCrossId();
            CrossDataRealtimePO crossDataRealtimePO = crossDataRealtimeMapper.selectByCrossId(crossId);
            BaseCrossInfoPO baseCrossInfoPO = baseCrossInfoMapper.selectById(crossId);
            if (crossDataRealtimePO != null && baseCrossInfoPO != null) {
                greenWaveCrossMonitorVO.setCrossId(crossId);
                greenWaveCrossMonitorVO.setCrossName(baseCrossInfoPO.getName());
                greenWaveCrossMonitorVO.setFlow(crossDataRealtimePO.getFlow());

                List<CrossDirDataRealtimePO> crossDirDataRealtimePOList =
                        crossDirDataRealtimeMapper.selectBycrossId(crossId);
                Integer capacity = 0;
                for (CrossDirDataRealtimePO crossDirDataRealtimePO : crossDirDataRealtimePOList) {
                    capacity += crossDirDataRealtimePO.getCapacity();
                }
                greenWaveCrossMonitorVO.setCapacity(capacity);

                greenWaveCrossMonitorVO.setCongestionIndex(crossDataRealtimePO.getCongestionIndex());
                res.add(greenWaveCrossMonitorVO);
            }
        }
        return res;
    }

    private List<AbnormalCrossDetailVO.DirDataElement> buildDirData(String crossId) {
        initPreStatus();
        List<AbnormalCrossDetailVO.DirDataElement> dirData = new ArrayList<>();
        // 获取当前时间之前40分钟的10位时间戳
        long currentSeconds = DateUtil.currentSeconds();
        long preSeconds = currentSeconds - 40 * 60;
        List<CrossDirDataHistPO> crossDirDataHistPOList =
                crossDirDataHistMapper.selectByCrossIdAndTimestamp(crossId, preSeconds);
        List<Integer> dirs = baseCrossDirInfoMapper.selectInDirsByCrossId(crossId);

        Date currentMinute = DateUtil.date(currentSeconds * 1000);
        Date startMinute = DateUtil.offsetMinute(currentMinute, -59);

        for (int i = 1; i <= 60; i++) {
            // 下一分钟
            Date nextMinute = DateUtil.offsetMinute(startMinute, 1);
            for (Integer dir : dirs) {
                AbnormalCrossDetailVO.DirDataElement dirDataElement = new AbnormalCrossDetailVO.DirDataElement();
                dirDataElement.setDir(dir);
                dirDataElement.setTime(startMinute);
                int status = calcStatus(crossDirDataHistPOList, startMinute, nextMinute, dir);
                dirDataElement.setStatus(status);
                dirData.add(dirDataElement);
            }
            startMinute = nextMinute;
        }

        return dirData;
    }

    private void initPreStatus() {
        for (Integer key : preStatus.keySet()) {
            preStatus.put(key, 1);
        }
    }

    /**
     * 计算路口状态
     *
     * @author Kent HAN
     * @date 2023/6/20 9:07
     */
    private Integer calcStatus(List<CrossDirDataHistPO> crossDirDataHistPOList, Date preMinute, Date currentMinute,
                               int dir) {
        for (CrossDirDataHistPO crossDirDataHistPO : crossDirDataHistPOList) {
            int dirType = crossDirDataHistPO.getDirType();
            long batchTime = crossDirDataHistPO.getBatchTime();
            long preSecond = preMinute.getTime() / 1000;
            long currentSecond = currentMinute.getTime() / 1000;
            if (dirType == dir && batchTime <= currentSecond && batchTime >= preSecond) {
                Integer status = crossDirDataHistPO.getStatus();
                // 转换状态，3、4都算拥堵
                if (status == 4) {
                    status = 3;
                }
                preStatus.put(dir, status);
            }
        }
        return preStatus.get(dir);
    }

    private List<AbnormalCrossDetailVO.TurnDataElement> buildTurnData(String crossId) {
        List<AbnormalCrossDetailVO.TurnDataElement> turnData = new ArrayList<>();
        List<CrossTurnDataRealtimePO> crossTurnDataRealtimePOList = crossTurnDataRealtimeMapper.selectByCrossId(crossId);
        Map<Integer, List<CrossTurnDataRealtimePO>> collect = crossTurnDataRealtimePOList.stream()
                .collect(Collectors.groupingBy(CrossTurnDataRealtimePO::getInDir));
        for (Map.Entry<Integer, List<CrossTurnDataRealtimePO>> entry : collect.entrySet()) {
            AbnormalCrossDetailVO.TurnDataElement turnDataElement = new AbnormalCrossDetailVO.TurnDataElement();
            Integer dir = entry.getKey();
            turnDataElement.setDir(dir);
            // 构造转向列表
            List<CrossTurnDataRealtimePO> subList = entry.getValue();
            List<AbnormalCrossDetailVO.TurnDataElement.TurnListElement> turnList = buildTurnList(subList);
            turnDataElement.setTurnList(turnList);

            turnData.add(turnDataElement);
        }
        return turnData;
    }

    private List<AbnormalCrossDetailVO.TurnDataElement.TurnListElement> buildTurnList(
            List<CrossTurnDataRealtimePO> subList) {
        List<AbnormalCrossDetailVO.TurnDataElement.TurnListElement> turnList = new ArrayList<>();

        for (CrossTurnDataRealtimePO crossTurnDataRealtimePO : subList) {
            AbnormalCrossDetailVO.TurnDataElement.TurnListElement turnListElement =
                    new AbnormalCrossDetailVO.TurnDataElement.TurnListElement();

            turnListElement.setTurn(crossTurnDataRealtimePO.getTurnType());
            turnListElement.setTurnQueueLength(crossTurnDataRealtimePO.getQueueLength());
            turnListElement.setTurnStopTimes(crossTurnDataRealtimePO.getStopTimes());
            turnListElement.setFlow(crossTurnDataRealtimePO.getFlow());

            turnList.add(turnListElement);
        }
        return turnList;
    }

    private static void buildAbnormalCrossStats(AbnormalCrossVO abnormalCrossVO,
                                                List<AbnormalCrossListVO> abnormalCrossListVOList) {
        AbnormalCrossStats abnormalCrossStats = new AbnormalCrossStats();
        Integer normal = abnormalCrossStats.getNormal();
        Integer unbalance = abnormalCrossStats.getUnbalance();
        Integer congestion = abnormalCrossStats.getCongestion();
        Integer spillover = abnormalCrossStats.getSpillover();
        Integer phaseEmpty = abnormalCrossStats.getPhaseEmpty();
        for (AbnormalCrossListVO abnormalCrossListVO : abnormalCrossListVOList) {
            Integer realtimeStatus = abnormalCrossListVO.getRealtimeStatus();
            if (realtimeStatus == 0) {
                normal += 1;
            } else if (realtimeStatus == 1) {
                unbalance += 1;
            } else if (realtimeStatus == 2) {
                congestion += 1;
            } else if (realtimeStatus == 3) {
                spillover += 1;
            } else if (realtimeStatus == 5) {
                phaseEmpty += 1;
            }
        }
        abnormalCrossStats.setNormal(normal);
        abnormalCrossStats.setUnbalance(unbalance);
        abnormalCrossStats.setCongestion(congestion);
        abnormalCrossStats.setSpillover(spillover);
        abnormalCrossStats.setPhaseEmpty(phaseEmpty);
        abnormalCrossVO.setAbnormalCrossStats(abnormalCrossStats);
    }

    private static void changeStatus(List<GreenwaveListVO> greenwaveListVOList) {
        if (greenwaveListVOList != null) {
            for (GreenwaveListVO greenwaveListVO : greenwaveListVOList) {
                if (greenwaveListVO.getRealtimeStatus() == 4) {
                    greenwaveListVO.setRealtimeStatus(3);
                }
            }
        }
    }

    private List<GreenwaveListVO.CrossListElement> buildCrossList(Integer greenwaveId) {
        List<GreenwaveListVO.CrossListElement> crossList = new ArrayList<>();
        List<String> crossIdList = greenwaveInfoMapper.selectCrossIdsById(greenwaveId);
        if (CollectionUtil.isNotEmpty(crossIdList)) {
            List<BaseCrossInfoPO> baseCrossInfoPOList = baseCrossInfoMapper.selectByCrossIds(crossIdList);
            for (BaseCrossInfoPO baseCrossInfoPO : baseCrossInfoPOList) {
                GreenwaveListVO.CrossListElement crossListElement = new GreenwaveListVO.CrossListElement();
                crossListElement.setCrossId(baseCrossInfoPO.getId());
                crossListElement.setName(baseCrossInfoPO.getName());
                List<Double> location = buildLocation(baseCrossInfoPO);
                crossListElement.setLocation(location);

                crossList.add(crossListElement);
            }
        }
        return crossList;
    }

    private List<Double> buildLocation(BaseCrossInfoPO baseCrossInfoPO) {
        String location = baseCrossInfoPO.getLocation();
        double[] lonLat = CrossUtil.getLonLat(location);
        List<Double> res = new ArrayList<>();
        res.add(lonLat[0]);
        res.add(lonLat[1]);
        return res;
    }

    private static void buildDuration(GreenwaveListVO greenwaveListVO) {
        Date startTime = greenwaveListVO.getStartTime();
        Date endTime = greenwaveListVO.getEndTime();
        Long duration = null;
        if (endTime != null) {
            duration = DateUtil.between(startTime, endTime, DateUnit.MINUTE);
        } else {
            duration = DateUtil.between(startTime, new Date(), DateUnit.MINUTE);
        }
        greenwaveListVO.setDuration(duration);
    }


    @Override
    public List<HotspotCrossVO> hotspotCross(CommonCrossIdVO commonCrossIdVO) throws Exception {
        try {
            String crossId = commonCrossIdVO.getCrossId();
            List<HotspotCrossVO> hotspotCrossVOS = new ArrayList<>();
            List<AbnormalCrossListVO> signalCrossRealTimeList =
                    crossDataRealtimeMapper.selectAbnormalCross(null, null, null);
            if (!CollectionUtils.isEmpty(signalCrossRealTimeList)) {
                AbnormalCrossListVO abnormalCrossListVO = new AbnormalCrossListVO();
                // 如果路口编号为空，态势监测页面默认展示拥堵指数第一的路口排队长度数据
                if (StringUtils.isBlank(crossId)) {
                    Optional<AbnormalCrossListVO> max = signalCrossRealTimeList.stream().max(Comparator.comparingDouble(AbnormalCrossListVO::getCongestionIndex));
                    if (max.isPresent()) {
                        abnormalCrossListVO = max.get();
                    }
                } else {
                    // 如果路口编号不为空，点击路口查看详细路口信息,并添加转向信息
                    Stream<AbnormalCrossListVO> abnormalCrossListVOStream = signalCrossRealTimeList.stream().filter((po) -> StringUtils.equalsIgnoreCase(crossId, po.getId()));
                    abnormalCrossListVO = abnormalCrossListVOStream.findFirst().orElse(null);
                }
                if (Objects.nonNull(abnormalCrossListVO)) {
                    // 查询当前拥堵指数最大，历史表中一个小时以内数据
                    LambdaQueryWrapper<CrossDirDataHistPO> histQuery = new LambdaQueryWrapper<>();
                    histQuery.eq(CrossDirDataHistPO::getCrossId, abnormalCrossListVO.getId())
                            .between(CrossDirDataHistPO::getBatchTime, abnormalCrossListVO.getBatchTime() - 86400, abnormalCrossListVO.getBatchTime());
                    List<CrossDirDataHistPO> histPOS = crossDirDataHistMapper.selectList(histQuery);
                    if (!CollectionUtils.isEmpty(histPOS)) {
                        Map<Integer, List<CrossDirDataHistPO>> map = histPOS.stream().collect(Collectors.groupingBy(CrossDirDataHistPO::getBatchTime));
                        if (!map.isEmpty()) {
                            for (Map.Entry<Integer, List<CrossDirDataHistPO>> entry : map.entrySet()) {
                                HotspotCrossVO hotspotCrossVO = new HotspotCrossVO();
                                List<CrossDirDataHistPO> value = entry.getValue();
                                if (!CollectionUtils.isEmpty(value)) {
                                    List<HotspotCrossVO.DirDetail> dirDetails = new ArrayList<>(value.size());
                                    for (CrossDirDataHistPO crossDirDataHistPO : value) {
                                        HotspotCrossVO.DirDetail dirDetail = new HotspotCrossVO.DirDetail();
                                        dirDetail.setDir(crossDirDataHistPO.getDirType());
                                        dirDetail.setInOutType(crossDirDataHistPO.getInOutType());
                                        dirDetail.setQueueLength(crossDirDataHistPO.getQueueLength());
                                        dirDetail.setFlow(crossDirDataHistPO.getFlow());
                                        dirDetails.add(dirDetail);
                                    }
                                    hotspotCrossVO.setDetailList(dirDetails);
                                    Date currentDate = new Date(entry.getKey() * 1000L);
                                    hotspotCrossVO.setTimeStamp(currentDate);
                                    // 相差八小时
                                    hotspotCrossVO.setHourMinute(DateUtils.format(currentDate, "HH:mm"));
                                    hotspotCrossVO.setCrossId(abnormalCrossListVO.getId());
                                    hotspotCrossVOS.add(hotspotCrossVO);
                                }
                            }
                        }
                    }
                }
            }
            return hotspotCrossVOS.stream().sorted(Comparator.comparing(HotspotCrossVO::getTimeStamp)).collect(Collectors.toList());
        } catch (Exception e) {
            log.error("重点路口监测异常：", e);
            throw new OptServiceException(e);
        }
    }

    @Override
    public List<HotspotCrossTurnVO> crossTurnData(CommonCrossIdVO commonCrossIdVO) throws Exception {
        List<HotspotCrossTurnVO> results = new ArrayList<>();
        try {
            String crossId = commonCrossIdVO.getCrossId();
            //Date date = new Date(1705456500000L);
            Date date = new Date();
            long dateTime = date.getTime() / 1000;
            LambdaQueryWrapper<CrossTurnDataHistPO> turnQuery = new LambdaQueryWrapper<>();
            turnQuery.eq(CrossTurnDataHistPO::getCrossId, crossId)
                    .between(CrossTurnDataHistPO::getBatchTime, dateTime - 86400, dateTime);
            List<CrossTurnDataHistPO> turnDataHistPOList = crossTurnDataHistMapper.selectList(turnQuery);
            if (!CollectionUtils.isEmpty(turnDataHistPOList)) {
                Map<Integer, List<CrossTurnDataHistPO>> collect = turnDataHistPOList.stream().collect(Collectors.groupingBy(CrossTurnDataHistPO::getBatchTime));
                for (Map.Entry<Integer, List<CrossTurnDataHistPO>> entry : collect.entrySet()) {
                    Integer timeStamp = entry.getKey();
                    List<CrossTurnDataHistPO> value = entry.getValue();
                    if (!CollectionUtils.isEmpty(value)) {
                        Map<Integer, List<CrossTurnDataHistPO>> dirMap = value.stream().collect(Collectors.groupingBy(CrossTurnDataHistPO::getInDir));
                        for (Map.Entry<Integer, List<CrossTurnDataHistPO>> dirEntry : dirMap.entrySet()) {
                            HotspotCrossTurnVO hotspotCrossTurnVO = new HotspotCrossTurnVO();
                            Integer dir = dirEntry.getKey();
                            List<CrossTurnDataHistPO> list = dirEntry.getValue();
                            List<HotspotCrossTurnVO.TurnDetail> turnDetails = new ArrayList<>();
                            for (CrossTurnDataHistPO crossTurnDataHistPO : list) {
                                HotspotCrossTurnVO.TurnDetail turnDetail = new HotspotCrossTurnVO.TurnDetail();
                                turnDetail.setTurn(crossTurnDataHistPO.getTurnType());
                                turnDetail.setQueueLength(crossTurnDataHistPO.getQueueLength());
                                turnDetail.setFlow(crossTurnDataHistPO.getFlow());
                                turnDetails.add(turnDetail);
                            }
                            hotspotCrossTurnVO.setCrossId(crossId);
                            hotspotCrossTurnVO.setDir(dir);
                            Date currentDate = new Date(timeStamp * 1000L);
                            hotspotCrossTurnVO.setTimeStamp(currentDate);
                            hotspotCrossTurnVO.setHourMinute(DateUtil.format(currentDate, DateStyle.HH_MM.getValue()));
                            hotspotCrossTurnVO.setTurnList(turnDetails);
                            results.add(hotspotCrossTurnVO);
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("路口转向排队长度异常", e);
            throw new OptServiceException(e);
        }
        return results.stream().sorted(Comparator.comparing(HotspotCrossTurnVO::getTimeStamp)).collect(Collectors.toList());
    }

    @Override
    public List<HotspotCrossLaneVO> crossLaneData(CommonCrossIdVO commonCrossIdVO) throws Exception {
        List<HotspotCrossLaneVO> results = new ArrayList<>();
        try {
            String crossId = commonCrossIdVO.getCrossId();
            Date date = new Date();
            //Date date = new Date(1705456500000L);
            long dateTime = date.getTime() / 1000;
            List<CrossLaneDataHistPOExt> crossLaneDataHistPOS =
                    crossLaneDataHistMapper.selectByCrossIdAndTimeSpan(crossId, (int) (dateTime - 86400), (int) dateTime);
            if (!CollectionUtils.isEmpty(crossLaneDataHistPOS)) {
                Map<Integer, List<CrossLaneDataHistPOExt>> collect = crossLaneDataHistPOS.stream().collect(Collectors.groupingBy(CrossLaneDataHistPOExt::getBatchTime));
                for (Map.Entry<Integer, List<CrossLaneDataHistPOExt>> entry : collect.entrySet()) {
                    Integer timeStamp = entry.getKey();
                    List<CrossLaneDataHistPOExt> value = entry.getValue();
                    Map<Integer, List<CrossLaneDataHistPOExt>> dirLaneMap = value.stream().collect(Collectors.groupingBy(CrossLaneDataHistPOExt::getDir));
                    for (Map.Entry<Integer, List<CrossLaneDataHistPOExt>> dirEntry : dirLaneMap.entrySet()) {
                        HotspotCrossLaneVO hotspotCrossLaneVO = new HotspotCrossLaneVO();
                        Integer dir = dirEntry.getKey();
                        List<CrossLaneDataHistPOExt> dirList = dirEntry.getValue();
                        List<HotspotCrossLaneVO.LaneDetail> laneDetails = new ArrayList<>();
                        for (CrossLaneDataHistPOExt crossLaneDataHistPO : dirList) {
                            HotspotCrossLaneVO.LaneDetail laneDetail = new HotspotCrossLaneVO.LaneDetail();
                            String poId = crossLaneDataHistPO.getId();
                            laneDetail.setLaneId(poId.substring(poId.length() - 2));
                            laneDetail.setFlow(crossLaneDataHistPO.getFlow());
                            laneDetail.setQueueLength(crossLaneDataHistPO.getQueueLength());
                            laneDetails.add(laneDetail);
                        }
                        hotspotCrossLaneVO.setCrossId(crossId);
                        hotspotCrossLaneVO.setDir(dir);
                        Date dateStamp = new Date(timeStamp * 1000L);
                        hotspotCrossLaneVO.setTimeStamp(dateStamp);
                        hotspotCrossLaneVO.setHourMinute(DateUtil.format(dateStamp, DateStyle.HH_MM.getValue()));
                        hotspotCrossLaneVO.setTurnList(laneDetails);
                        results.add(hotspotCrossLaneVO);
                    }
                }
            }
        } catch (Exception e) {
            log.error("路口车道数据异常：", e);
            throw new OptServiceException(e);
        }
        return results.stream().sorted(Comparator.comparing(HotspotCrossLaneVO::getTimeStamp)).collect(Collectors.toList());
    }

    @Override
    public List<Top5IndexVO> top5Flow() throws Exception {
        List<Top5IndexVO> hotspotCrossVOS = new ArrayList<>();
        try {
            List<AbnormalCrossListVO> signalCrossRealTimeList = crossDataRealtimeMapper.selectAbnormalCross(null, null, null);
            if (!CollectionUtils.isEmpty(signalCrossRealTimeList)) {
                List<AbnormalCrossListVO> top5 = signalCrossRealTimeList.stream()
                        .sorted(Comparator.comparingDouble(AbnormalCrossListVO::getCongestionIndex))
                        .collect(Collectors.toList());
                Collections.reverse(top5);
                top5 = top5.subList(0, Math.min(top5.size(), 5));
                if (!CollectionUtils.isEmpty(top5)) {
                    for (AbnormalCrossListVO abnormalCrossListVO : top5) {
                        String crossId = abnormalCrossListVO.getId();
                        Top5IndexVO top5IndexVO = new Top5IndexVO();
                        top5IndexVO.setCrossId(crossId);
                        //metricsDetailBO.setDate(DateUtil.offsetDay(new Date(1705456500000L), -1));
                        Date date = new Date();
                        DateTime start = DateUtil.offsetDay(DateUtil.offsetHour(date, 2), -1);
                        DateTime end = DateUtil.offsetDay(DateUtil.offsetHour(date, 2), 0);
                        int startStamp = (int) (start.getTime() / 1000); // 10位时间戳
                        int endStamp = (int) (end.getTime() / 1000 + 300);
                        List<MetricHistDTO> metricHistDTOS = crossDataHistMapper.selectMetricHistDTO(crossId, startStamp, endStamp);
                        // 使用全量数据按时间粒度聚合指标
                        List<RunningEvaluateMetricsDetailVO.CrossMetrics> crossMetrics = RunningEvaluateServiceImpl.buildMetricsList(metricHistDTOS, 120);
                        List<RunningEvaluateMetricsDetailVO.CrossMetrics> collect = crossMetrics.stream().sorted(Comparator.comparing(RunningEvaluateMetricsDetailVO.CrossMetrics::getTimeStamp).reversed()).collect(Collectors.toList());
                        top5IndexVO.setMetricsList(collect);
                        hotspotCrossVOS.add(top5IndexVO);
                    }
                }
            }
        } catch (Exception e) {
            log.error("全天流量监测异常：", e);
            throw new OptServiceException(e);
        }
        return hotspotCrossVOS;
    }


    @Override
    public CrossStatusTimeRateVO crossStatusTimeRate(CommonCrossIdVO commonCrossIdVO) throws Exception {
        CrossStatusTimeRateVO crossStatusTimeRateVO = new CrossStatusTimeRateVO();
        try {
            String crossId = commonCrossIdVO.getCrossId();
            //Instant instant = Instant.ofEpochSecond(1705456500L);
            Date date = new Date();
            long time = date.getTime() / 1000;
            Instant instant = Instant.ofEpochSecond(time);
            LocalDateTime localDateTime = LocalDateTime.ofInstant(instant, ZoneId.of("+8"));
            // 00:00
            LocalDateTime startOfDay = localDateTime.withHour(0).withMinute(0).withSecond(0);
            int startBatchTime = Long.valueOf(Date.from(startOfDay.atZone(ZoneId.of("+8")).toInstant()).getTime() / 1000).intValue();
            // 23:59
            LocalDateTime endOfDay = startOfDay.plus(1, ChronoUnit.DAYS).minus(1, ChronoUnit.SECONDS);
            int endBatchTime = Long.valueOf(Date.from(endOfDay.atZone(ZoneId.of("+8")).toInstant()).getTime() / 1000).intValue();

            List<CrossDataHistPO> crossDataHistPOS = crossDataHistMapper.selectByCrossIdAndStartEnd(crossId, startBatchTime, endBatchTime);
            if (!CollectionUtils.isEmpty(crossDataHistPOS)) {
                Map<Integer, List<CrossDataHistPO>> statusMap = crossDataHistPOS.stream().collect(Collectors.groupingBy(CrossDataHistPO::getStatus));
                crossStatusTimeRateVO.setCrossId(crossId);
                for (Map.Entry<Integer, List<CrossDataHistPO>> entry : statusMap.entrySet()) {
                    Integer status = entry.getKey();
                    List<CrossDataHistPO> value = entry.getValue();
                    float temp = value.size() / (float) crossDataHistPOS.size() * 100;
                    temp = temp < 1 ? 0 : temp;
                    int rate = Math.round(temp);
                    setStatusTimeRate(crossStatusTimeRateVO, status, rate);
                }
            }
        } catch (Exception e) {
            log.error("交通状态时间比例异常：", e);
            throw new OptServiceException(e);
        }
        return crossStatusTimeRateVO;
    }

    /**
     * 设置状态时间比例
     * @param crossStatusTimeRateVO
     * @param status
     * @param rate
     */
    private static void setStatusTimeRate(CrossStatusTimeRateVO crossStatusTimeRateVO, Integer status, int rate) {
        if (status == 0) {
            crossStatusTimeRateVO.setUnblockedTimeRate(rate);
        }
        if (status == 1) {
            crossStatusTimeRateVO.setUnbalanceTimeRate(rate);
        }
        if (status == 2) {
            crossStatusTimeRateVO.setCongestionTimeRate(rate);
        }
        if (status == 3) {
            crossStatusTimeRateVO.setSpilloverTimeRate(rate);
        }
        if (status == 5) {
            crossStatusTimeRateVO.setPhaseEmptyTimeRate(rate);
        }
    }

    @Override
    public List<TableQueryVO.CycleDataElement> laneTrafficIndex(CommonCrossIdDateTimeVO crossIdDateTimeVO) throws Exception {
        try {
            String crossId = crossIdDateTimeVO.getCrossId();
            int start = (int) (crossIdDateTimeVO.getStart().getTime() / 1000);
            int end = (int) (crossIdDateTimeVO.getEnd().getTime() / 1000);

            // 查询周期数据
            List<CrossLaneDataHistPOExt> poExtList = crossLaneDataHistMapper.selectByCrossIdAndTimeSpan(crossId, start, end);
            return buildCycleData(crossId, poExtList);
        } catch (Exception e) {
            log.error("车道数据查询异常：", e);
            throw new OptServiceException(e);
        }
    }


    @Override
    public List<TableQueryVO.RealTimeDataElement> laneSnapshotIndex(LaneSnapshotIndexVO laneSnapshotIndexVO) throws Exception {
        try {
            String crossId = laneSnapshotIndexVO.getCrossId();
            int start = (int) (laneSnapshotIndexVO.getStart().getTime() / 1000);
            int end = (int) (laneSnapshotIndexVO.getEnd().getTime() / 1000);
            int pageNum = laneSnapshotIndexVO.getPageNum();
            String laneId = laneSnapshotIndexVO.getLaneId();
            boolean overFlow = laneSnapshotIndexVO.isOverFlow();

            // 查询秒级数据
            List<CrossLaneSnapshotDataDTO> dtoList =
                    laneSnapshotDataQueryService.queryByCrossIdAndTimeSpan(crossId, start, end, pageNum, laneId, overFlow);
            List<TableQueryVO.RealTimeDataElement> result = buildRealTimeList(dtoList);
            List<TableQueryVO.RealTimeDataElement> sorts = new ArrayList<>();
            if (!CollectionUtils.isEmpty(result)) {
                sorts = result.stream().sorted(Comparator.comparing(TableQueryVO.RealTimeDataElement::getTime).reversed()).collect(Collectors.toList());
            }
            return sorts;
        } catch (Exception e) {
            log.error("车道快照数据查询异常：", e);
            throw new OptServiceException(e);
        }
    }

    @Override
    public List<OverflowEvent> overFlowEvent(LaneSnapshotIndexVO laneSnapshotIndexVO) throws Exception {
        try {
            List<OverflowEvent> resultList = new ArrayList<>();
            List<CrossLaneSnapshotDataDTO> dtoList = getCrossLaneSnapshotDataDTOS(laneSnapshotIndexVO);
            if (!CollectionUtils.isEmpty(dtoList)) {
                Map<String, List<CrossLaneSnapshotDataDTO>> timeMap = dtoList.stream().collect(Collectors.groupingBy(CrossLaneSnapshotDataDTO::getTimeStamp));
                for (Map.Entry<String, List<CrossLaneSnapshotDataDTO>> timeEntry : timeMap.entrySet()) {
                    String time = timeEntry.getKey();
                    List<CrossLaneSnapshotDataDTO> timeList = timeEntry.getValue();
                    if (CollectionUtil.isNotEmpty(timeList)) {
                        Map<Integer, List<CrossLaneSnapshotDataDTO>> dirMap = timeList.stream().collect(Collectors.groupingBy(CrossLaneSnapshotDataDTO::getDir));
                        for (Map.Entry<Integer, List<CrossLaneSnapshotDataDTO>> dirEntry : dirMap.entrySet()) {
                            OverflowEvent resultPojo = getOverflowEvent(time, dirEntry);
                            resultList.add(resultPojo);
                        }
                    }
                }
            }
            return resultList;
        } catch (Exception e) {
            log.error("车道快照数据查询异常：", e);
            throw new OptServiceException(e);
        }
    }

    private static OverflowEvent getOverflowEvent(String time, Map.Entry<Integer, List<CrossLaneSnapshotDataDTO>> dirEntry) {
        Integer dir = dirEntry.getKey();
        List<CrossLaneSnapshotDataDTO> dirList = dirEntry.getValue();
        OverflowEvent resultPojo = new OverflowEvent();
        int overflowNums = 0;
        double vehicleLengthRatio = 0.0;
        double meanV = 0.0;
        for (CrossLaneSnapshotDataDTO dto : dirList) {
            overflowNums += dto.getOverflowNums();
            vehicleLengthRatio += dto.getVehicleLengthRatio();
            meanV += dto.getMeanV();
        }
        resultPojo.setDir(dir);
        resultPojo.setStartTime(new Date(Long.parseLong(time) * 1000));
        resultPojo.setEndTime(new Date(Long.parseLong(time) * 1000));
        resultPojo.setDirName(BaseEnum.SignalDirectionEnum.getNameByCode(dir));
        resultPojo.setOverflowId(String.valueOf(dir));
        resultPojo.setOverflowNums(overflowNums);
        resultPojo.setSpeed( Math.round((meanV / dirList.size()) * 100) / 100.0);
        resultPojo.setVehicleLengthRatio(Math.round(vehicleLengthRatio / dirList.size() * 100));
        return resultPojo;
    }

    private List<CrossLaneSnapshotDataDTO> getCrossLaneSnapshotDataDTOS(LaneSnapshotIndexVO laneSnapshotIndexVO) throws Exception {
        String crossId = laneSnapshotIndexVO.getCrossId();
        int start = (int) (laneSnapshotIndexVO.getStart().getTime() / 1000);
        int end = (int) (laneSnapshotIndexVO.getEnd().getTime() / 1000);
        int pageNum = laneSnapshotIndexVO.getPageNum();
        boolean overFlow = laneSnapshotIndexVO.isOverFlow();
        // 查询秒级数据
        return laneSnapshotDataQueryService.queryByCrossIdAndTimeSpan(crossId, start, end, pageNum, null, overFlow);
    }

    @Override
    public List<AnalysisRidTurnIndicators> lanePeriodTurnData(LanePeriodTurnVO lanePeriodTurnVO) throws Exception {
        try {
            String crossId = lanePeriodTurnVO.getCrossId();
            Date start = lanePeriodTurnVO.getStart();
            Date end = lanePeriodTurnVO.getEnd();
            String granularity = lanePeriodTurnVO.getGranularity();
            if (StringUtils.isBlank(granularity)) {
                granularity = "5m";
            }
            LambdaQueryWrapper<AnalysisRidTurnIndicators> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(AnalysisRidTurnIndicators::getCrossId, crossId);
            queryWrapper.eq(AnalysisRidTurnIndicators::getGranularity, granularity);
            queryWrapper.between(AnalysisRidTurnIndicators::getWindowStartTime, start, end);
            queryWrapper.orderByDesc(AnalysisRidTurnIndicators::getWindowStartTime);
            return ridTurnIndicatorsMapper.selectList(queryWrapper);
        } catch (Exception e) {
            log.error("全是周期车道数据方向转向查询异常：", e);
            throw new OptServiceException(e);
        }
    }

    @Override
    public List<HoloEventInfoPO> holoEvenList(HoloEventVO holoEventVO) throws Exception {
        try {
            LambdaQueryWrapper<HoloEventInfoPO> queryWrapper = new LambdaQueryWrapper<>();
            String crossId = holoEventVO.getCrossId();
            Date start = holoEventVO.getStart();
            LocalDateTime localDateTime1 = LocalDateTime.ofInstant(start.toInstant(), ZoneId.of("+8"));
            Date end = holoEventVO.getEnd();
            LocalDateTime localDateTime2 = LocalDateTime.ofInstant(end.toInstant(), ZoneId.of("+8"));
            String types = holoEventVO.getTypes();
            queryWrapper.eq(HoloEventInfoPO::getCrossId, crossId);
            queryWrapper.between(HoloEventInfoPO::getStartTime, localDateTime1, localDateTime2);
            List<HoloEventInfoPO> holoEventInfoPOS = holoEventMapper.selectList(queryWrapper);
            List<HoloEventInfoPO> results = new ArrayList<>();
            if (!CollectionUtils.isEmpty(holoEventInfoPOS) && StringUtils.isNotBlank(types)) {
                for (HoloEventInfoPO holoEventInfoPO : holoEventInfoPOS) {
                    String type = holoEventInfoPO.getType();
                    if (types.contains(type)) {
                        results.add(holoEventInfoPO);
                    }
                }
            } else {
                results = holoEventInfoPOS;
            }
            String crossName = crossInfoCache.getCrossName(crossId);
            results.forEach(holoEventInfoPO -> {
                if (Objects.isNull(holoEventInfoPO.getPlaceDesc())) {
                    holoEventInfoPO.setPlaceDesc(crossName);
                }
            });

            return results;
        } catch (Exception e) {
            log.error("全息事件查询异常：", e);
            throw new OptServiceException(e);
        }
    }

    @Override
    public List<LaneIdAliasNameVO> laneIdList(CommonCrossIdVO commonCrossIdVO) throws Exception {
        try {
            List<LaneIdAliasNameVO> laneIdAliasNameVOS = new ArrayList<>();
            LambdaQueryWrapper<CrossBaseLaneInfoPO> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(CrossBaseLaneInfoPO::getCrossId, commonCrossIdVO.getCrossId());
            queryWrapper.eq(CrossBaseLaneInfoPO::getType, "2");
            List<CrossBaseLaneInfoPO> laneInfoPOS = crossBaseLaneInfoMapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(laneInfoPOS)) {
                for (CrossBaseLaneInfoPO po : laneInfoPOS) {
                    LaneIdAliasNameVO laneIdAliasNameVO = new LaneIdAliasNameVO();
                    String laneId = po.getId();
                    Integer dir = po.getDir();
                    Integer turn = po.getTurn();
                    String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dir);
                    String turnName = TurnConvertEnum.getDescByKey(turn);
                    laneIdAliasNameVO.setLaneId(laneId);
                    laneIdAliasNameVO.setAliasName(String.join("-", dirName, turnName, laneId.substring(laneId.length() - 2)));
                    laneIdAliasNameVOS.add(laneIdAliasNameVO);
                }
            }
            return laneIdAliasNameVOS;
        } catch (Exception e) {
            log.error("车道基本信息查询失败：", e);
            throw new OptServiceException(e);
        }
    }

    @Override
    public void periodExcel(CommonCrossIdDateTimeVO crossIdDateTimeVO, HttpServletResponse response) throws Exception {
        String crossId = crossIdDateTimeVO.getCrossId();
        Date startDate = crossIdDateTimeVO.getStart();
        Date endDate = crossIdDateTimeVO.getEnd();
        String startStr = DateUtil.format(startDate, DateStyle.YYYY_MM_DD_HH_MM.getValue());
        String endStr = DateUtil.format(endDate, DateStyle.MM_DD_HH_MM.getValue());
        int start = (int) (startDate.getTime() / 1000);
        int end = (int) (endDate.getTime() / 1000);

        // 查询周期数据
        List<CrossLaneDataHistPOExt> poExtList = crossLaneDataHistMapper.selectByCrossIdAndTimeSpan(crossId, start, end);
        List<TableQueryVO.CycleDataElement> dataList = buildCycleData(crossId, poExtList);

        String crossName = crossInfoCache.getCrossName(crossId);
        ExcelExportUtils.exportExcel(response, startStr, endStr, dataList, crossName.concat("车道周期数据"), TableQueryVO.CycleDataElement.class);
    }

    @Override
    public void periodTurnExcel(LanePeriodTurnVO lanePeriodTurnVO, HttpServletResponse response) throws Exception {
        String startStr = null;
        String endStr = null;
        List<TurnDataIndexExcelVO> result = new ArrayList<>();
        try {
            String crossId = lanePeriodTurnVO.getCrossId();
            Date startDate = lanePeriodTurnVO.getStart();
            Date endDate = lanePeriodTurnVO.getEnd();
            startStr = DateUtil.format(startDate, DateStyle.YYYY_MM_DD_HH_MM.getValue());
            endStr = DateUtil.format(endDate, DateStyle.MM_DD_HH_MM.getValue());

            LambdaQueryWrapper<AnalysisRidTurnIndicators> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(AnalysisRidTurnIndicators::getCrossId, crossId);
            queryWrapper.between(AnalysisRidTurnIndicators::getWindowStartTime, startDate, endDate);
            queryWrapper.orderByDesc(AnalysisRidTurnIndicators::getWindowStartTime);
            List<AnalysisRidTurnIndicators> dataList = ridTurnIndicatorsMapper.selectList(queryWrapper);

            result = new ArrayList<>();
            Map<Date, List<AnalysisRidTurnIndicators>> timeMap = dataList.stream().collect(Collectors.groupingBy(AnalysisRidTurnIndicators::getWindowStartTime));
            for (Map.Entry<Date, List<AnalysisRidTurnIndicators>> dateListEntry : timeMap.entrySet()) {
                Date time = dateListEntry.getKey();
                List<AnalysisRidTurnIndicators> dateValueList = dateListEntry.getValue();
                Map<Integer, List<AnalysisRidTurnIndicators>> dirList = dateValueList.stream().collect(Collectors.groupingBy(AnalysisRidTurnIndicators::getFRidDir));
                for (Map.Entry<Integer, List<AnalysisRidTurnIndicators>> entry : dirList.entrySet()) {
                    Integer dir = entry.getKey();
                    List<AnalysisRidTurnIndicators> turnList = entry.getValue();
                    if (!CollectionUtils.isEmpty(turnList)) {
                        int allFlow = 0;
                        int leftFlow = 0;
                        int straightFlow = 0;
                        int rightFlow = 0;
                        int left = 0;
                        int straight = 0;
                        int right = 0;
                        BigDecimal leftSpeed = new BigDecimal(0);
                        BigDecimal straightSpeed = new BigDecimal(0);
                        BigDecimal rightSpeed = new BigDecimal(0);
                        TurnDataIndexExcelVO turnDataIndexExcelVO = new TurnDataIndexExcelVO();
                        for (AnalysisRidTurnIndicators analysisRidTurnIndicators : turnList) {
                            Integer turnDirNo = analysisRidTurnIndicators.getTurnDirNo();
                            time = analysisRidTurnIndicators.getWindowStartTime();
                            if (turnDirNo == 1) {
                                leftFlow += analysisRidTurnIndicators.getFlow();
                                leftSpeed = leftSpeed.add(analysisRidTurnIndicators.getSpeed());
                                ++left;
                            }

                            if (turnDirNo == 2) {
                                straightFlow += analysisRidTurnIndicators.getFlow();
                                straightSpeed = straightSpeed.add(analysisRidTurnIndicators.getSpeed());
                                ++straight;
                            }

                            if (turnDirNo == 3) {
                                rightFlow += analysisRidTurnIndicators.getFlow();
                                rightSpeed = straightSpeed.add(analysisRidTurnIndicators.getSpeed());
                                ++right;
                            }
                        }
                        allFlow = leftFlow + straightFlow + rightFlow;
                        turnDataIndexExcelVO.setTime(time);
                        turnDataIndexExcelVO.setDir(BaseEnum.SignalDirectionEnum.getNameByCode(dir));
                        turnDataIndexExcelVO.setFlow(allFlow);
                        turnDataIndexExcelVO.setLeftFlow(leftFlow);
                        BigDecimal leftAverage = new BigDecimal(0);
                        if (left > 0) {
                            leftAverage = leftSpeed.divide(new BigDecimal(left), 2, RoundingMode.HALF_UP);
                        }
                        turnDataIndexExcelVO.setLeftSpeed(leftAverage);
                        if (leftFlow == 0) {
                            turnDataIndexExcelVO.setLeftSpeed(new BigDecimal(0));
                        }
                        turnDataIndexExcelVO.setStraightFlow(straightFlow);
                        BigDecimal straightAverage = new BigDecimal(0);
                        if (straight > 0) {
                            straightAverage = straightSpeed.divide(new BigDecimal(straight), 2, RoundingMode.HALF_UP);
                        }
                        turnDataIndexExcelVO.setStraightSpeed(straightAverage);
                        if (straightFlow == 0) {
                            turnDataIndexExcelVO.setStraightSpeed(new BigDecimal(0));
                        }
                        turnDataIndexExcelVO.setRightFlow(rightFlow);
                        BigDecimal rightAverage = new BigDecimal(0);
                        if (right > 0) {
                            rightAverage = rightSpeed.divide(new BigDecimal(right), 2, RoundingMode.HALF_UP);
                        }
                        turnDataIndexExcelVO.setRightSpeed(rightAverage);
                        if (rightFlow == 0) {
                            turnDataIndexExcelVO.setRightSpeed(new BigDecimal(0));
                        }

                        turnDataIndexExcelVO.setSpeed(leftAverage.add(straightAverage).add(rightAverage).divide(BigDecimal.valueOf(3), 2, RoundingMode.HALF_UP));
                        result.add(turnDataIndexExcelVO);
                    }
                }
            }
            String crossName = crossInfoCache.getCrossName(crossId);
            ExcelExportUtils.exportExcel(response, startStr, endStr, result, crossName.concat("转向周期数据"), TurnDataIndexExcelVO.class);
        } catch (Exception e) {
            log.error("导出转向数据异常：", e);
            throw new OptServiceException(e);
        }
    }

    @Override
    public void laneSnapshotExcel(LaneSnapshotIndexVO laneSnapshotIndexVO, HttpServletResponse response) throws Exception {
        try {
            String crossId = laneSnapshotIndexVO.getCrossId();
            Date startDate = laneSnapshotIndexVO.getStart();
            Date endDate = laneSnapshotIndexVO.getEnd();
            String startStr = DateUtil.format(startDate, DateStyle.YYYY_MM_DD_HH_MM.getValue());
            String endStr = DateUtil.format(endDate, DateStyle.MM_DD_HH_MM.getValue());

            int start = (int) (startDate.getTime() / 1000);
            int end = (int) (endDate.getTime() / 1000);
            String laneId = laneSnapshotIndexVO.getLaneId();
            // 查询秒级数据
            List<CrossLaneSnapshotDataDTO> dtoList =
                    laneSnapshotDataQueryService.queryByCrossIdAndTimeSpan(crossId, start, end, -1, laneId, false);
            List<TableQueryVO.RealTimeDataElement> result = buildRealTimeList(dtoList);
            List<TableQueryVO.RealTimeDataElement> dataList = new ArrayList<>();
            if (!CollectionUtils.isEmpty(result)) {
                dataList = result.stream().sorted(Comparator.comparing(TableQueryVO.RealTimeDataElement::getTime).reversed()).collect(Collectors.toList());
            }
            String crossName = crossInfoCache.getCrossName(crossId);
            ExcelExportUtils.exportExcel(response, startStr, endStr, dataList, crossName.concat("车道快照数据"), TableQueryVO.RealTimeDataElement.class);
        } catch (Exception e) {
            log.error("快照数据导出失败：", e);
            throw new OptServiceException(e);
        }

    }
}
