package net.wanji.opt.synthesis.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.DateStyle;
import net.wanji.common.framework.Constants;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.opt.dao.mapper.StrategyControlInfoMapper;
import net.wanji.opt.dao.mapper.StrategyDailyPlanInfoMapper;
import net.wanji.opt.dao.mapper.StrategyPlanInfoMapper;
import net.wanji.opt.synthesis.pojo.*;
import net.wanji.opt.synthesis.service.PushStrategyControlService;
import net.wanji.opt.synthesis.service.StrategyControlService;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2024/11/03 17:37
 */
@Slf4j
@Service
public class StrategyControlServiceImpl implements StrategyControlService {

    @Resource
    private PushStrategyControlService pushStrategyControlService;
    @Resource
    private StrategyControlInfoMapper strategyControlInfoMapper;
    @Resource
    private StrategyPlanInfoMapper strategyPlanInfoMapper;
    @Resource
    private StrategyDailyPlanInfoMapper strategyDailyPlanInfoMapper;

    @Override
    public JsonViewObject strategyInfoOperation(StrategyControlVO strategyControlVO) throws Exception {

        try {
            // 存库
            insertAndUpdate(strategyControlVO);

            StrategyControlReq req = convertReq(strategyControlVO);
            Result result = pushStrategyControlService.push(req);
            if (Objects.nonNull(result)) {
                JsonViewObject jsonViewObject = JsonViewObject.newInstance();
                jsonViewObject.setCode(Integer.valueOf(result.getCode()));
                jsonViewObject.setMessage(result.getMsg());
                jsonViewObject.setContent(result.getData());
                jsonViewObject.setStatus(StringUtils.equals("200", result.getCode()) ? Constants.JsonView.STATUS_SUCCESS : Constants.JsonView.STATUS_FAIL);
                return jsonViewObject;
            } else {
                return JsonViewObject.newInstance().fail("推送神思策略控制失败！");
            }
        } catch (Exception e) {
            log.error("策略操作失败：{}", e);
            throw new Exception(e);
        }
    }

    private void insertAndUpdate(StrategyControlVO strategyControlVO) throws IllegalAccessException, InvocationTargetException, JsonProcessingException {
        List<StrategyControlDataVO> dataList = strategyControlVO.getDataList();
        for (StrategyControlDataVO dataVO : dataList) {
            StrategyControlDataEntity entity = new StrategyControlDataEntity();
            BeanUtils.copyProperties(entity, dataVO);
            entity.setTime(JacksonUtils.getInstance().writeValueAsString(dataVO.getTime()));
            if (StringUtils.equals("insert", dataVO.getAction())) {
                entity.setId(null);
                strategyControlInfoMapper.insert(entity);
            } else if (StringUtils.equals("update", dataVO.getAction())) {
                strategyControlInfoMapper.updateById(entity);
            } else if (StringUtils.equals("delete", dataVO.getAction())) {
                strategyControlInfoMapper.deleteById(entity);
            }
        }
    }

    private StrategyControlReq convertReq(StrategyControlVO vo) throws Exception {
        try {
            List<StrategyControlDataVO> dataList = vo.getDataList();
            StrategyControlReq req = new StrategyControlReq();
            if (!CollectionUtils.isEmpty(dataList)) {
                List<StrategyControlDataReq> dataReqList = new ArrayList<>(dataList.size());
                for (StrategyControlDataVO dataVO : dataList) {
                    StrategyControlDataReq detailData = new StrategyControlDataReq();
                    detailData.setBiz_id(dataVO.getBizId());
                    detailData.setBiz_type(dataVO.getBizType());
                    detailData.setStrategy(dataVO.getStrategy());
                    detailData.setSchedule_start(DateUtil.format(dataVO.getScheduleStart(), DateStyle.YYYY_MM_DD_HH_MM_SS.getValue()));
                    detailData.setSchedule_end(DateUtil.format(dataVO.getScheduleEnd(), DateStyle.YYYY_MM_DD_HH_MM_SS.getValue()));
                    List<StrategyControlDataVO.TimeTable> timeTableList = dataVO.getTime();
                    List<StrategyControlDataReq.Time_table> timeTables = new ArrayList<>(timeTableList.size());
                    timeTableList.forEach(item -> {
                        StrategyControlDataReq.Time_table timeTable = new StrategyControlDataReq.Time_table();
                        timeTable.setTime_list(item.getTimeList());
                        timeTable.setWeek(item.getWeek());
                        timeTables.add(timeTable);
                    });
                    detailData.setTime_table(timeTables);
                    detailData.setFreq(dataVO.getFrequency());
                    detailData.setStatus(dataVO.getStatus());
                    detailData.setAction(StringUtils.equals("update", dataVO.getAction()) ? "insert" : dataVO.getAction());
                    dataReqList.add(detailData);
                    req.setData(dataReqList);
                }
            }
            return req;
        } catch (Exception e) {
            log.error("策略控制操作数据转换异常：{}", e);
            throw new Exception(e);
        }
    }

    @Override
    public JsonViewObject strategyInfoPageList(StrategyControlDataEntity entity) throws Exception {
        LambdaQueryWrapper<StrategyControlDataEntity> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotBlank(entity.getBizId())) {
            queryWrapper.eq(StrategyControlDataEntity::getBizId, entity.getBizId());
        }
        if (Objects.nonNull(entity.getBizType())) {
            queryWrapper.eq(StrategyControlDataEntity::getBizType, entity.getBizType());
        }
        if (Objects.nonNull(entity.getStrategy())) {
            queryWrapper.eq(StrategyControlDataEntity::getStrategy, entity.getStrategy());
        }
        if (Objects.nonNull(entity.getStatus())) {
            queryWrapper.eq(StrategyControlDataEntity::getStatus, entity.getStatus());
        }
        Page<StrategyControlDataEntity> page = new Page<>(entity.getPageNum(), entity.getPageSize());
        Page<StrategyControlDataEntity> pageInfo = strategyControlInfoMapper.selectPage(page, queryWrapper);
        return JsonViewObject.newInstance().success(pageInfo.getRecords());
    }

    @Override
    public JsonViewObject crossStrategyInfoList(Integer type) throws Exception {
        ObjectMapper instance = JacksonUtils.getInstance();
        LambdaQueryWrapper<StrategyControlDataEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(StrategyControlDataEntity::getBizType, type);
        List<StrategyControlDataEntity> entities = strategyControlInfoMapper.selectList(queryWrapper);
        List<StrategyControlDataEntity> results = new ArrayList<>(entities.size());
        Date current = new Date();
        for (StrategyControlDataEntity entity : entities) {
            StrategyControlDataEntity result = new StrategyControlDataEntity();
            BeanUtils.copyProperties(result, entity);
            Date start = entity.getScheduleStart();
            Date end = entity.getScheduleEnd();
            if (current.before(start) || current.after(end)) {
                result.setTime("");
            } else {
                String time = entity.getTime();
                List<StrategyControlDataVO.TimeTable> timeTables = instance.readValue(time, new TypeReference<List<StrategyControlDataVO.TimeTable>>() {});
                for (StrategyControlDataVO.TimeTable timeTable : timeTables) {
                    int currentWeek = DateUtil.thisDayOfWeek();
                    if (currentWeek != timeTable.getWeek()) {
                        result.setTime("");
                    } else {
                        String[] timeList = timeTable.getTimeList();
                        for (String s : timeList) {
                            String[] hours = s.split(",");
                            for (String hour : hours) {
                                String[]  currentHour = hour.split("-");
                                String startHour = currentHour[0];
                                String entHour = currentHour[1];
                                String format = DateUtil.format(current, "HH:mm");
                                DateTime currentTime = DateUtil.parse(format, "HH:mm");
                                DateTime startHourDate = DateUtil.parse(startHour, "HH:mm");
                                DateTime endHourDate = DateUtil.parse(entHour, "HH:mm");
                                if (currentTime.before(startHourDate) || currentTime.after(endHourDate)) {
                                    result.setTime(hour);
                                }
                            }
                        }
                    }
                    results.add(result);
                }
            }
        }
        return JsonViewObject.newInstance().success(results);
    }


    @Override
    public JsonViewObject strategyPlanDetail(String crossId) throws Exception {
        try {
            ObjectMapper instance = JacksonUtils.getInstance();
            StrategyControlDetailList result = new StrategyControlDetailList();
            LambdaQueryWrapper<StrategyPlanInfoEntity> planQueryWrapper = new LambdaQueryWrapper<>();
            planQueryWrapper.eq(StrategyPlanInfoEntity::getCrossId, crossId);
            List<StrategyPlanInfoEntity> planInfoEntities = strategyPlanInfoMapper.selectList(planQueryWrapper);
            List<StrategyControlDetailList.ExecutePlan> executePlans = new ArrayList<>();
            for (StrategyPlanInfoEntity planInfoEntity : planInfoEntities) {
                StrategyControlDetailList.ExecutePlan executePlan = new StrategyControlDetailList.ExecutePlan();
                executePlan.setCompany(planInfoEntity.getCompany());
                executePlan.setCrossId(planInfoEntity.getCrossId());
                executePlan.setType(planInfoEntity.getType());
                executePlan.setPlanId(planInfoEntity.getPlanId());
                executePlan.setStartTime(planInfoEntity.getStartTime());
                executePlan.setEndTime(planInfoEntity.getEndTime());
                String planDetailsStr = planInfoEntity.getPlanDetails();
                List<StrategyControlDetailList.ExecutePlan.PlanDetail> planDetails = instance.readValue(planDetailsStr, new TypeReference<List<StrategyControlDetailList.ExecutePlan.PlanDetail>>() {});
                executePlan.setPlanDetails(planDetails);
                executePlans.add(executePlan);
            }
            result.setPlans(executePlans);

            LambdaQueryWrapper<StrategyDailyPlanInfoEntity> dailyPlanQueryWrapper = new LambdaQueryWrapper<>();
            dailyPlanQueryWrapper.eq(StrategyDailyPlanInfoEntity::getCrossId, crossId);
            List<StrategyDailyPlanInfoEntity> dailyPlanInfoEntities = strategyDailyPlanInfoMapper.selectList(dailyPlanQueryWrapper);
            List<StrategyControlDetailList.DailyPlan> dailyPlans = new ArrayList<>();
            for (StrategyDailyPlanInfoEntity dailyPlanInfoEntity : dailyPlanInfoEntities) {
                StrategyControlDetailList.DailyPlan dailyPlan = new StrategyControlDetailList.DailyPlan();
                dailyPlan.setCrossId(dailyPlanInfoEntity.getCrossId());
                dailyPlan.setDailyPlanId(dailyPlanInfoEntity.getDailyPlanId());
                String dailyPlanDetailsStr = dailyPlanInfoEntity.getDailyPlanDetails();
                List<StrategyControlDetailList.DailyPlan.DailyPlanDetail> dailyPlanDetails = instance.readValue(dailyPlanDetailsStr, new TypeReference<List<StrategyControlDetailList.DailyPlan.DailyPlanDetail>>() {});
                dailyPlan.setDailyPlanDetails(dailyPlanDetails);
                dailyPlans.add(dailyPlan);
            }
            result.setDailyPlans(dailyPlans);
            return JsonViewObject.newInstance().success(result);
        } catch (Exception e) {
            log.error("策略管理查询失败：{}", e);
            throw new Exception(e);
        }
    }

    @Override
    public JsonViewObject strategyPlanSave(StrategyControlDetailList list) throws Exception {
        try {
            if (Objects.nonNull(list)) {
                ObjectMapper mapper = JacksonUtils.getInstance();
                List<StrategyControlDetailList.ExecutePlan> plans = list.getPlans();
                if (!CollectionUtils.isEmpty(plans)) {
                    for (StrategyControlDetailList.ExecutePlan plan : plans) {
                        StrategyPlanInfoEntity planInfoEntity = new StrategyPlanInfoEntity();
                        planInfoEntity.setPlanId(plan.getPlanId());
                        planInfoEntity.setCrossId(plan.getCrossId());
                        planInfoEntity.setType(plan.getType());
                        planInfoEntity.setStartTime(plan.getStartTime());
                        planInfoEntity.setEndTime(plan.getEndTime());
                        planInfoEntity.setCompany(plan.getCompany());
                        planInfoEntity.setPlanDetails(mapper.writeValueAsString(plan.getPlanDetails()));
                        strategyPlanInfoMapper.insert(planInfoEntity);
                    }
                }

                List<StrategyControlDetailList.DailyPlan> dailyPlans = list.getDailyPlans();
                for (StrategyControlDetailList.DailyPlan dailyPlan : dailyPlans) {
                    Integer dailyPlanId = dailyPlan.getDailyPlanId();
                    String crossId = dailyPlan.getCrossId();
                    List<StrategyControlDetailList.DailyPlan.DailyPlanDetail> dailyPlanDetails = dailyPlan.getDailyPlanDetails();
                    StrategyDailyPlanInfoEntity dailyPlanInfoEntity = new StrategyDailyPlanInfoEntity();
                    dailyPlanInfoEntity.setCrossId(crossId);
                    dailyPlanInfoEntity.setDailyPlanId(dailyPlanId);
                    dailyPlanInfoEntity.setDailyPlanDetails(mapper.writeValueAsString(dailyPlanDetails));
                    strategyDailyPlanInfoMapper.insert(dailyPlanInfoEntity);
                }
            }
            convertDataList(list);
            return JsonViewObject.newInstance().success();
        } catch (Exception e) {
            log.error("策略管理数据保存失败，：{}", e);
            throw new Exception(e);
        }
    }


    /**
     * 页面管理数据转化神思数据
     *
     * @param list
     */
    public void convertDataList(StrategyControlDetailList list) throws Exception {
        try {
            if (Objects.nonNull(list)) {
                List<StrategyControlDetailList.DailyPlan> dailyPlans = list.getDailyPlans();
                List<StrategyPlanDTO> strategyPlanDTOS = new ArrayList<>();
                if (!CollectionUtils.isEmpty(dailyPlans)) {
                    for (StrategyControlDetailList.DailyPlan dailyPlan : dailyPlans) {
                        Integer dailyPlanId = dailyPlan.getDailyPlanId();
                        List<StrategyControlDetailList.DailyPlan.DailyPlanDetail> dailyPlanDetails = dailyPlan.getDailyPlanDetails();
                        for (StrategyControlDetailList.DailyPlan.DailyPlanDetail dailyPlanDetail : dailyPlanDetails) {
                            StrategyPlanDTO strategyPlanDTO = new StrategyPlanDTO();
                            strategyPlanDTO.setStrategy(dailyPlanDetail.getStrategy());
                            strategyPlanDTO.setTimes(dailyPlanDetail.getTimes());
                            strategyPlanDTO.setDailyPlanId(dailyPlanId);
                            strategyPlanDTOS.add(strategyPlanDTO);
                        }
                    }
                }

                List<StrategyPlanDTO> resultList = new ArrayList<>();
                List<StrategyControlDetailList.ExecutePlan> plans = list.getPlans();
                if (!CollectionUtils.isEmpty(plans) && !CollectionUtils.isEmpty(dailyPlans)) {
                    for (StrategyControlDetailList.ExecutePlan plan : plans) {
                        Integer planId = plan.getPlanId();
                        Date startTime = plan.getStartTime();
                        Date endTime = plan.getEndTime();
                        List<StrategyControlDetailList.ExecutePlan.PlanDetail> planDetails = plan.getPlanDetails();
                        if (!CollectionUtils.isEmpty(planDetails)) {
                            for (StrategyControlDetailList.ExecutePlan.PlanDetail planDetail : planDetails) {
                                Integer dailyPlanId = planDetail.getDailyPlanId();
                                List<Integer> weeks = planDetail.getWeeks();
                                for (StrategyPlanDTO strategyPlanDTO : strategyPlanDTOS) {
                                    if (dailyPlanId == strategyPlanDTO.getDailyPlanId()) {
                                        strategyPlanDTO.setPlanId(planId);
                                        strategyPlanDTO.setStartTime(startTime);
                                        strategyPlanDTO.setEndTime(endTime);
                                        strategyPlanDTO.setWeeks(weeks);
                                        resultList.add(strategyPlanDTO);
                                    }
                                }
                            }
                        }
                    }
                }
                for (StrategyPlanDTO strategyPlanDTO : strategyPlanDTOS) {
                    System.err.println(strategyPlanDTO);
                }
                log.error("神思数据：{}", JacksonUtils.getInstance().writeValueAsString(strategyPlanDTOS));
            }
        } catch (Exception e) {
            log.error("数据转化神思数据结构异常：{}", e);
            throw new Exception(e);
        }
    }
}
