package net.wanji.opt.vo;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;

/**
 * @author Kent HAN
 * @date 2023/2/9 8:38
 */
@Data
@NoArgsConstructor
@ApiModel(value = "EvaluateMetricsVO", description = "评价指标")
public class EvaluateMetricsVO {
    @ApiModelProperty(value = "路口流量")
    private List<CrossFlowElement> crossFlow;
    @ApiModelProperty(value = "路口排队长度")
    private List<QueueLengthElement> queueLength;
    @ApiModelProperty(value = "拥堵指数")
    private List<TrafficIndexElement> trafficIndex;
    @ApiModelProperty(value = "溢流率")
    private List<EffusionRateElement> effusionRate;

    @NoArgsConstructor
    @Data
    public static class CrossFlowElement {
        @ApiModelProperty(value = "方向：1北；2东北；3东；4东南；5南；6西南；7西；8西北")
        private Integer dir;
        @ApiModelProperty(value = "当前时段通行能力")
        private Integer currentCapacity;
        @ApiModelProperty(value = "对比时段通行能力")
        private Integer compareCapacity;
        @ApiModelProperty(value = "当前时段流量")
        private Integer currentFlow;
        @ApiModelProperty(value = "当前时段流量")
        private Integer compareFlow;
    }

    @NoArgsConstructor
    @Data
    public static class QueueLengthElement {
        @ApiModelProperty(value = "方向：1北；2东北；3东；4东南；5南；6西南；7西；8西北")
        private Integer dir;
        @ApiModelProperty(value = "当前时段排队长度")
        @JsonSerialize(using = net.wanji.common.framework.DoubleSerialize.class)
        private Double currentLength;
        @ApiModelProperty(value = "对比时段排队长度")
        @JsonSerialize(using = net.wanji.common.framework.DoubleSerialize.class)
        private Double compareLength;
    }

    @NoArgsConstructor
    @Data
    public static class TrafficIndexElement {
        @ApiModelProperty(value = "方向：1北；2东北；3东；4东南；5南；6西南；7西；8西北")
        private Integer dir;
        @ApiModelProperty(value = "当前时段拥堵指数")
        @JsonSerialize(using = net.wanji.common.framework.DoubleSerialize.class)
        private Double currentIndex;
        @ApiModelProperty(value = "对比时段拥堵指数")
        @JsonSerialize(using = net.wanji.common.framework.DoubleSerialize.class)
        private Double compareIndex;
    }

    @NoArgsConstructor
    @Data
    public static class EffusionRateElement {
        @ApiModelProperty(value = "方向：1北；2东北；3东；4东南；5南；6西南；7西；8西北")
        private Integer dir;
        @ApiModelProperty(value = "当前时段溢流率")
        @JsonSerialize(using = net.wanji.common.framework.DoubleSerialize.class)
        private Double currentRate;
        @ApiModelProperty(value = "对比时段溢流率")
        @JsonSerialize(using = net.wanji.common.framework.DoubleSerialize.class)
        private Double compareRate;
    }
}
