package net.wanji.opt.vo;

import com.fasterxml.jackson.annotation.JsonAnyGetter;
import com.fasterxml.jackson.annotation.JsonIgnore;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;
import java.util.Map;

/**
 * @author Kent HAN
 * @date 2023/2/9 8:38
 */
@Data
@NoArgsConstructor
@ApiModel(value = "SchemeEvaluateSchemeDetailedProblemVO", description = "方案详情-详细问题")
public class SchemeEvaluateSchemeDetailedProblemVO {
    @ApiModelProperty(value = "存在问题")
    private List<String> problems;

    @ApiModelProperty(value = "表格数据")
    private List<TableContent> tableContentList;

    @NoArgsConstructor
    @Data
    public static class TableContent {
        @ApiModelProperty(value = "位置代码 1，2，3...表示进口方向；l，r，s，u表示转向")
        private String positionCode;

        @ApiModelProperty(value = "位置")
        private String position;

        @ApiModelProperty(value = "流量")
        private Integer flow;

        @ApiModelProperty(value = "是否有问题，0否 1是")
        private Integer hasProblem = 0;

        @ApiModelProperty(value = "子列表")
        private List<SubTableContent> subList;

        // 可变指标数据
        @JsonIgnore
        private Map<String, String> metricsMap;

        @JsonAnyGetter
        public Map<String, String> any() {
            return metricsMap;
        }
    }

    @NoArgsConstructor
    @Data
    public static class SubTableContent {
        @ApiModelProperty(value = "位置代码 1，2，3...表示进口方向；l，r，s，u表示转向")
        private String positionCode;

        @ApiModelProperty(value = "方向代码")
        private String dirCode;

        @ApiModelProperty(value = "位置")
        private String position;

        @ApiModelProperty(value = "流量")
        private Integer flow;

        @ApiModelProperty(value = "是否有问题，0否 1是")
        private Integer hasProblem = 0;

        @ApiModelProperty(value = "子列表")
        private List<SubTableContent> subList;

        // 可变指标数据
        @JsonIgnore
        private Map<String, String> metricsMap;

        @JsonAnyGetter
        public Map<String, String> any() {
            return metricsMap;
        }
    }
}
